/*
Code source pour les effets appliqus au modle du joueur affich dans le hud (viewmodel)
Par Jean-Franois Vuagniaux

vuagniaux.jf@com.mcnet.ch

On implmente un nouvel lement au hud pour pouvoir recevoir des messages du serveur et 
galement afficher une nouvelle information sur le hud, par exemple la vision qui devient floue
ou qui est perturbe par des parasites.

On va aussi pouvoir agir sur le modle du joueur en tant que viewmodel, c'est  dire les mains et les armes
que l'on voit  l'cran, afin d'y appliquer des effets divers (GlowShell, transparence, etc...)
*/


#include "hud.h"
#include "cl_util.h"
#include "const.h"
#include "entity_state.h"
#include "cl_entity.h"
#include "r_efx.h"
#include "eventscripts.h"
#include "parsemsg.h"

#include <string.h>
#include <stdio.h>

//==================================================
//Application de l'effet GlowShell
//==================================================
void ApplyPlayerEffects( int Effect, int OnOff, int Param1, int Param2, int Param3, int Param4 )
{
	cl_entity_t *hands = gEngfuncs.GetViewModel();
	if ( !hands )
	{
		return;
	}

	if ( Effect == kRenderFxGlowShell )
	{
		if ( OnOff == 1 )
		{
			hands->curstate.rendermode = kRenderNormal;
			hands->curstate.renderamt = 255;
			hands->curstate.rendercolor.r = Param1;
			hands->curstate.rendercolor.g = Param2;
			hands->curstate.rendercolor.b = Param3;
			hands->curstate.renderamt = Param4;
			hands->curstate.renderfx = kRenderFxGlowShell;
		}
		else
		{
			hands->curstate.rendermode = kRenderNormal;
			hands->curstate.renderamt = 255;
			hands->curstate.renderfx = kRenderFxNone;
		}
	}

	if ( Effect == kRenderFxFadeFast )//Hack not really this effect, just to have a defined constant.
	{
		if ( OnOff == 1 )
		{
			
			hands->curstate.rendermode = kRenderNormal;
			hands->curstate.renderamt = Param1;
			hands->curstate.renderfx = kRenderFxHologram;
		}
		else
		{
			hands->curstate.rendermode = kRenderNormal;
			hands->curstate.renderamt = 255;
			hands->curstate.renderfx = kRenderFxNone;
		}



	}
}

DECLARE_MESSAGE(m_PlEffect, PlEffect)

int CHudPlayerEffects::Init( void )
{
	

	HOOK_MESSAGE(PlEffect);

	gHUD.AddHudElem(this);

	return 1;
};


int CHudPlayerEffects::VidInit(void)
{
	
	return 1;
};


//==============================================================================
//Va permettre de recevoir des messages du serveur 
//du type : MESSAGE_BEGIN(MSG_ONE, gmsgPlayerEffects, NULL, m_pPlayer->pev);
//avec deux valeurs transmises : le type d'effet et l'activation-dsactivation
//sous forme de BYTE : WRITE_BYTE(type);   WRITE_BYTE(OnOff);
//==============================================================================

int CHudPlayerEffects:: MsgFunc_PlEffect(const char *pszName,  int iSize, void *pbuf )
{

	
	BEGIN_READ( pbuf, iSize );
	int Effect = READ_BYTE();
	int OnOff  = READ_BYTE();
	int Param1 = READ_BYTE();
	int Param2 = READ_BYTE();
	int Param3 = READ_BYTE();
	int Param4 = READ_BYTE();

	ApplyPlayerEffects(Effect, OnOff, Param1, Param2, Param3, Param4);

	return 1;
}


int CHudPlayerEffects::Draw(float flTime)
{
	
	return 1;
};