/***
*
*	Code pour les personnages neutres
*	Par Jean-Franois Vuagniaux
*	vuagniaux.jf@com.mcnet.ch	
*
****/
//=========================================================
// Divers personnages pouvant parler
//=========================================================

#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"talkmonster.h"
#include	"schedule.h"
#include	"defaultai.h"
#include	"scripted.h"
#include	"animation.h"
#include	"soundent.h"

//===============================================
//	Possibilit de disposer de plusieurs ttes
//===============================================
#define		NUM_CIVIL_HEADS		4 // 4 ttes disponibles

enum { 
HEAD_BARMAN =	0, 
HEAD_DUCON =	1, 
HEAD_ALIEN =	2, 
HEAD_OUVRIER =	3, 
};

#define		NUM_CIVIL_BODIES	4 // 4 corps disponibles 

enum {

BODY_MAN_LEGS1	= 0,
BODY_MAN_LEGS2	= 1,
BODY_MAN_ALIEN	= 2,
BODY_MAN_WORKER	= 3,
};


//=========================
// Schedules spcifiques
//=========================
enum
{
	SCHED_HIDE = LAST_TALKMONSTER_SCHEDULE + 1,
	SCHED_FEAR,
	SCHED_PANIC,
	SCHED_STARTLE,
	SCHED_TARGET_CHASE_SCARED,
	SCHED_TARGET_FACE_SCARED,
};

//=========================
// Tches scifiques
//=========================
enum
{
	TASK_SAY_FEAR,
	TASK_RUN_PATH_SCARED,
	TASK_SCREAM,
	TASK_RANDOM_SCREAM,
	TASK_MOVE_TO_TARGET_RANGE_SCARED,
};

//=========================================================
// Evnements d'animation 
//=========================================================
#define		CIVIL_AE_SPEAK		( 1 )


//=======================================================
// Classe du civil
//=======================================================

class CCivil : public CTalkMonster
{
public:
	void Spawn( void );
	void Precache( void );

	void SetYawSpeed( void );
	int  Classify ( void );
	void HandleAnimEvent( MonsterEvent_t *pEvent );
	void RunTask( Task_t *pTask );
	void StartTask( Task_t *pTask );
	int	ObjectCaps( void ) { return CTalkMonster :: ObjectCaps() | FCAP_IMPULSE_USE; }
	int TakeDamage( entvars_t* pevInflictor, entvars_t* pevAttacker, float flDamage, int bitsDamageType);
	virtual int FriendNumber( int arrayNumber );
	void SetActivity ( Activity newActivity );
	Activity GetStoppedActivity( void );
	int ISoundMask( void );
	void DeclineFollowing( void );

	float	CoverRadius( void ) { return 1200; }	// agrandi car les civils sont trouillards et partent trs loin en cas de danger
	BOOL	DisregardEnemy( CBaseEntity *pEnemy ) { return !pEnemy->IsAlive() || (gpGlobals->time - m_fearTime) > 15; }

	void	Scream( void );

	// Override these to set behavior
	Schedule_t *GetScheduleOfType ( int Type );
	Schedule_t *GetSchedule ( void );
	MONSTERSTATE GetIdealState ( void );

	void DeathSound( void );
	void PainSound( void );
	
	void TalkInit( void );

	void			Killed( entvars_t *pevAttacker, int iGib );
	
	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	CUSTOM_SCHEDULES;

private:	
	float m_painTime;
	float m_fearTime;
};

LINK_ENTITY_TO_CLASS( monster_civil, CCivil );

TYPEDESCRIPTION	CCivil::m_SaveData[] = 
{
	DEFINE_FIELD( CCivil, m_painTime, FIELD_TIME ),
	DEFINE_FIELD( CCivil, m_fearTime, FIELD_TIME ),
};

IMPLEMENT_SAVERESTORE( CCivil, CTalkMonster );

//=========================================================
// Schedules d'AI specifiques au civil
//========================================================
Task_t	tlCivFollow[] =
{
	{ TASK_SET_FAIL_SCHEDULE,	(float)SCHED_CANT_FOLLOW },	// si echec, on renonce  suivre
	{ TASK_MOVE_TO_TARGET_RANGE,(float)128		},	// Va dans la zone de 128 de l'entit cible (client)
//	{ TASK_SET_SCHEDULE,		(float)SCHED_TARGET_FACE },
};

Schedule_t	slCivFollow[] =
{
	{
		tlCivFollow,
		ARRAYSIZE ( tlCivFollow ),
		bits_COND_NEW_ENEMY |
		bits_COND_LIGHT_DAMAGE |
		bits_COND_HEAVY_DAMAGE |
		bits_COND_HEAR_SOUND,
		bits_SOUND_COMBAT |
		bits_SOUND_DANGER,
		"CivFollow"
	},
};

Task_t	tlCivFollowScared[] =
{
	{ TASK_SET_FAIL_SCHEDULE,	(float)SCHED_TARGET_CHASE },// Si echec, suivre normalement
	{ TASK_MOVE_TO_TARGET_RANGE_SCARED,(float)128		},	// Va dans la zone de 128 de l'entit cible (client)
//	{ TASK_SET_SCHEDULE,		(float)SCHED_TARGET_FACE_SCARED },
};

Schedule_t	slCivFollowScared[] =
{
	{
		tlCivFollowScared,
		ARRAYSIZE ( tlCivFollowScared ),
		bits_COND_NEW_ENEMY |
		bits_COND_HEAR_SOUND |
		bits_COND_LIGHT_DAMAGE |
		bits_COND_HEAVY_DAMAGE,
		bits_SOUND_DANGER,
		"CivFollowScared"
	},
};

Task_t	tlCivFaceTargetScared[] =
{
	{ TASK_FACE_TARGET,			(float)0		},
	{ TASK_SET_ACTIVITY,		(float)ACT_CROUCHIDLE },
	{ TASK_SET_SCHEDULE,		(float)SCHED_TARGET_CHASE_SCARED },
};

Schedule_t	slCivFaceTargetScared[] =
{
	{
		tlCivFaceTargetScared,
		ARRAYSIZE ( tlCivFaceTargetScared ),
		bits_COND_HEAR_SOUND |
		bits_COND_NEW_ENEMY,
		bits_SOUND_DANGER,
		"CivFaceTargetScared"
	},
};

Task_t	tlCivStopFollowing[] =
{
	{ TASK_CANT_FOLLOW,		(float)0 },
};

Schedule_t	slCivStopFollowing[] =
{
	{
		tlCivStopFollowing,
		ARRAYSIZE ( tlCivStopFollowing ),
		0,
		0,
		"CivStopFollowing"
	},
};


Task_t	tlCivFaceTarget[] =
{
	{ TASK_STOP_MOVING,			(float)0		},
	{ TASK_FACE_TARGET,			(float)0		},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE },
	{ TASK_SET_SCHEDULE,		(float)SCHED_TARGET_CHASE },
};

Schedule_t	slCivFaceTarget[] =
{
	{
		tlCivFaceTarget,
		ARRAYSIZE ( tlCivFaceTarget ),
		bits_COND_CLIENT_PUSH |
		bits_COND_NEW_ENEMY |
		bits_COND_HEAR_SOUND,
		bits_SOUND_COMBAT |
		bits_SOUND_DANGER,
		"CivFaceTarget"
	},
};


Task_t	tlCivPanic[] =
{
	{ TASK_STOP_MOVING,			(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_SCREAM,				(float)0		},
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,		(float)ACT_EXCITED	},	
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE	},
};

Schedule_t	slCivPanic[] =
{
	{
		tlCivPanic,
		ARRAYSIZE ( tlCivPanic ),
		0,
		0,
		"CivPanic"
	},
};


Task_t	tlIdleCivStand[] =
{
	{ TASK_STOP_MOVING,			0				},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE },
	{ TASK_WAIT,				(float)2		}, // reprend IDLESTAND toutes les 2 secondes
	{ TASK_TLK_HEADRESET,		(float)0		}, // reset de la position de la tte
};

Schedule_t	slIdleCivStand[] =
{
	{ 
		tlIdleCivStand,
		ARRAYSIZE ( tlIdleCivStand ), 
		bits_COND_NEW_ENEMY		|
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE	|
		bits_COND_HEAR_SOUND	|
		bits_COND_SMELL			|
		bits_COND_CLIENT_PUSH	|
		bits_COND_PROVOKED,

		bits_SOUND_COMBAT		|// sound flags
		//bits_SOUND_PLAYER		|
		//bits_SOUND_WORLD		|
		bits_SOUND_DANGER		|
		bits_SOUND_MEAT			|// scents
		bits_SOUND_CARCASS		|
		bits_SOUND_GARBAGE,
		"IdleCivStand"

	},
};


Task_t	tlCivilCover[] =
{
	{ TASK_SET_FAIL_SCHEDULE,		(float)SCHED_PANIC },		// Pas de couvert alors on panique!
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_FIND_COVER_FROM_ENEMY,	(float)0					},
	{ TASK_RUN_PATH_SCARED,			(float)0					},
	{ TASK_TURN_LEFT,				(float)179					},
	{ TASK_SET_SCHEDULE,			(float)SCHED_HIDE			},
};

Schedule_t	slCivilCover[] =
{
	{ 
		tlCivilCover,
		ARRAYSIZE ( tlCivilCover ), 
		bits_COND_NEW_ENEMY,
		0,
		"CivilCover"
	},
};



Task_t	tlCivilHide[] =
{
	{ TASK_SET_FAIL_SCHEDULE,		(float)SCHED_PANIC },		// Pas de couvert alors on panique!
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_PLAY_SEQUENCE,			(float)ACT_CROUCH			},
	{ TASK_SET_ACTIVITY,			(float)ACT_CROUCHIDLE		},	
	{ TASK_WAIT_RANDOM,				(float)10.0					},
};

Schedule_t	slCivilHide[] =
{
	{ 
		tlCivilHide,
		ARRAYSIZE ( tlCivilHide ), 
		bits_COND_NEW_ENEMY |
		bits_COND_HEAR_SOUND |
		bits_COND_SEE_ENEMY |
		bits_COND_SEE_HATE |
		bits_COND_SEE_FEAR |
		bits_COND_SEE_DISLIKE,
		bits_SOUND_DANGER,
		"CivilHide"
	},
};


Task_t	tlCivilStartle[] =
{
	{ TASK_SET_FAIL_SCHEDULE,		(float)SCHED_PANIC },		// Pas de couvert alors on panique!
	{ TASK_RANDOM_SCREAM,			(float)0.3 },				// Crie 30% du temps
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,			(float)ACT_CROUCH			},
	{ TASK_RANDOM_SCREAM,			(float)0.1 },				// Crie  nouveau 10% du temps
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,			(float)ACT_CROUCHIDLE		},
	{ TASK_WAIT_RANDOM,				(float)1.0					},
};

Schedule_t	slCivilStartle[] =
{
	{ 
		tlCivilStartle,
		ARRAYSIZE ( tlCivilStartle ), 
		bits_COND_NEW_ENEMY |
		bits_COND_SEE_ENEMY |
		bits_COND_SEE_HATE |
		bits_COND_SEE_FEAR |
		bits_COND_SEE_DISLIKE,
		0,
		"CivilStartle"
	},
};



Task_t	tlCivFear[] =
{
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_FACE_ENEMY,				(float)0					},
	{ TASK_SAY_FEAR,				(float)0					},
//	{ TASK_PLAY_SEQUENCE,			(float)ACT_FEAR_DISPLAY		},
};

Schedule_t	slCivFear[] =
{
	{ 
		tlCivFear,
		ARRAYSIZE ( tlCivFear ), 
		bits_COND_NEW_ENEMY,
		0,
		"CivFear"
	},
};


DEFINE_CUSTOM_SCHEDULES( CCivil )
{
	slCivFollow,
	slCivFaceTarget,
	slIdleCivStand,
	slCivFear,
	slCivilCover,
	slCivilHide,
	slCivilStartle,
	slCivStopFollowing,
	slCivPanic,
	slCivFollowScared,
	slCivFaceTargetScared,
};


IMPLEMENT_CUSTOM_SCHEDULES( CCivil, CTalkMonster );


void CCivil::DeclineFollowing( void )
{
	Talk( 10 );
	m_hTalkTarget = m_hEnemy;
	PlaySentence( "SC_POK", 2, VOL_NORM, ATTN_NORM );
}


void CCivil :: Scream( void )
{
	if ( FOkToSpeak() )
	{
		Talk( 10 );
		m_hTalkTarget = m_hEnemy;
		PlaySentence( "SC_SCREAM", RANDOM_FLOAT(3, 6), VOL_NORM, ATTN_NORM );
	}
}


Activity CCivil::GetStoppedActivity( void )
{ 
	if ( m_hEnemy != NULL ) 
		return ACT_EXCITED;
	return CTalkMonster::GetStoppedActivity();
}


void CCivil :: StartTask( Task_t *pTask )
{
	switch( pTask->iTask )
	{

	case TASK_SCREAM:
		Scream();
		TaskComplete();
		break;

	case TASK_RANDOM_SCREAM:
		if ( RANDOM_FLOAT( 0, 1 ) < pTask->flData )
			Scream();
		TaskComplete();
		break;

	case TASK_SAY_FEAR:
		if ( FOkToSpeak() )
		{
			Talk( 2 );
			m_hTalkTarget = m_hEnemy;
			if ( m_hEnemy->IsPlayer() )
				PlaySentence( "SC_PLFEAR", 5, VOL_NORM, ATTN_NORM );
			else
				PlaySentence( "SC_FEAR", 5, VOL_NORM, ATTN_NORM );
		}
		TaskComplete();
		break;

	case TASK_RUN_PATH_SCARED:
		m_movementActivity = ACT_RUN_SCARED;
		break;

	case TASK_MOVE_TO_TARGET_RANGE_SCARED:
		{
			if ( (m_hTargetEnt->pev->origin - pev->origin).Length() < 1 )
				TaskComplete();
			else
			{
				m_vecMoveGoal = m_hTargetEnt->pev->origin;
				if ( !MoveToTarget( ACT_WALK_SCARED, 0.5 ) )
					TaskFail();
			}
		}
		break;

	default:
		CTalkMonster::StartTask( pTask );
		break;
	}
}

void CCivil :: RunTask( Task_t *pTask )
{
	switch ( pTask->iTask )
	{
	case TASK_RUN_PATH_SCARED:
		if ( MovementIsComplete() )
			TaskComplete();
		if ( RANDOM_LONG(0,31) < 8 )
			Scream();
		break;

	case TASK_MOVE_TO_TARGET_RANGE_SCARED:
		{
			if ( RANDOM_LONG(0,63)< 8 )
				Scream();

			if ( m_hEnemy == NULL )
			{
				TaskFail();
			}
			else
			{
				float distance;

				distance = ( m_vecMoveGoal - pev->origin ).Length2D();
				
				// Revaluer quand c'est fini ou que la cible est trop loin
				if ( (distance < pTask->flData) || (m_vecMoveGoal - m_hTargetEnt->pev->origin).Length() > pTask->flData * 0.5 )
				{
					m_vecMoveGoal = m_hTargetEnt->pev->origin;
					distance = ( m_vecMoveGoal - pev->origin ).Length2D();
					FRefreshRoute();
				}

				// Choisi l'activit approprie selon la distance
				if ( distance < pTask->flData )
				{
					TaskComplete();
					RouteClear();		// Stoppe le mouvement
				}
				else if ( distance < 190 && m_movementActivity != ACT_WALK_SCARED )
					m_movementActivity = ACT_WALK_SCARED;
				else if ( distance >= 270 && m_movementActivity != ACT_RUN_SCARED )
					m_movementActivity = ACT_RUN_SCARED;
			}
		}
		break;

	default:
		CTalkMonster::RunTask( pTask );
		break;
	}
}

//=========================================================
// Classification du personnage
//=========================================================
int	CCivil :: Classify ( void )
{
	return	CLASS_HUMAN_PASSIVE;
}


//=========================================================
// Vitesse de rotation dpendant de l'activit
//=========================================================
void CCivil :: SetYawSpeed ( void )
{
	int ys;

	ys = 90;

	switch ( m_Activity )
	{
	case ACT_IDLE:
		ys = 120;
		break;
	case ACT_WALK:
		ys = 180;
		break;
	case ACT_RUN:
		ys = 150;
		break;
	case ACT_TURN_LEFT:
	case ACT_TURN_RIGHT:
		ys = 120;
		break;
	}

	pev->yaw_speed = ys;
}

//=========================================================
// HandleAnimEvent 
//=========================================================

void CCivil :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
	switch( pEvent->event )
	{		
	case CIVIL_AE_SPEAK:		
		{

		}
		break;

	default:
		CTalkMonster::HandleAnimEvent( pEvent );
	}
}

//=========================================================
// Spawn
//=========================================================
void CCivil :: Spawn( void )
{
	Precache( );

	SET_MODEL(ENT(pev), "models/civil.mdl");
	UTIL_SetSize(pev, VEC_HUMAN_HULL_MIN, VEC_HUMAN_HULL_MAX);

	pev->solid			= SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_STEP;
	m_bloodColor		= BLOOD_COLOR_RED;
	pev->health			= gSkillData.civilHealth;
	pev->view_ofs		= Vector ( 0, 0, 50 );// hauteur des yeux
	m_flFieldOfView		= VIEW_FIELD_WIDE; 
	m_MonsterState		= MONSTERSTATE_NONE;

//	m_flDistTooFar		= 256.0;

	m_afCapability		= bits_CAP_HEAR | bits_CAP_TURN_HEAD | bits_CAP_OPEN_DOORS | bits_CAP_AUTO_DOORS | bits_CAP_USE;

	// Skin de dpart
	pev->skin = 0;
	pev->iuser4 = 1;//incendie possible
	pev->iuser3 = 1;//masse

	if ( pev->body == -1 )
	{// choisit une tte au hasard si aucune dfinie
		pev->body = RANDOM_LONG(0, NUM_CIVIL_HEADS-1);
	}

	
	
	MonsterInit();
	SetUse( FollowerUse );
}

//=========================================================
// Precache 
//=========================================================

void CCivil :: Precache( void )
{
	PRECACHE_MODEL("models/civil.mdl");
	PRECACHE_SOUND("civil/civ_pain1.wav");
	PRECACHE_SOUND("civil/civ_pain2.wav");
	PRECACHE_SOUND("civil/civ_pain3.wav");
	PRECACHE_SOUND("civil/civ_pain4.wav");
	PRECACHE_SOUND("civil/civ_pain5.wav");

	// chaque civil doit appeler ceci sinon lorsque un niveau est charg personne ne parlera(time est remis  0)
	TalkInit();

	CTalkMonster::Precache();
}	

// Dmarre les discussions

void CCivil :: TalkInit()
{
	
	CTalkMonster::TalkInit();

	// le civil va essayer de parler aux autres personnes prsentes dans l'ordre suivant:

	m_szFriends[0] = "monster_civil";
	m_szFriends[1] = "monster_femme";
	m_szFriends[2] = "monster_police";

	// non des groupes de discussion (dans sentences.txt)

	m_szGrp[TLK_ANSWER]  =	"CI_ANSWER";
	m_szGrp[TLK_QUESTION] =	"CI_QUESTION";
	m_szGrp[TLK_IDLE] =		"CI_IDLE";
	m_szGrp[TLK_STARE] =	"CI_STARE";
	m_szGrp[TLK_USE] =		"CI_OK";
	m_szGrp[TLK_UNUSE] =	"CI_WAIT";
	m_szGrp[TLK_STOP] =		"CI_STOP";
	m_szGrp[TLK_NOSHOOT] =	"CI_SCARED";
	m_szGrp[TLK_HELLO] =	"CI_HELLO";

	m_szGrp[TLK_PLHURT1] =	"!CI_CUREA";
	m_szGrp[TLK_PLHURT2] =	"!CI_CUREB"; 
	m_szGrp[TLK_PLHURT3] =	"!CI_CUREC";

	m_szGrp[TLK_PHELLO] =	"CI_PHELLO";
	m_szGrp[TLK_PIDLE] =	"CI_PIDLE";
	m_szGrp[TLK_PQUESTION] = "CI_PQUEST";
	m_szGrp[TLK_SMELL] =	"CI_SMELL";
	
	m_szGrp[TLK_WOUND] =	"CI_WOUND";
	m_szGrp[TLK_MORTAL] =	"CI_MORTAL";

	// get voice for head
	switch (pev->body % 3)
	{
	default:
	case HEAD_BARMAN:	m_voicePitch = 105; break;	
	case HEAD_DUCON: m_voicePitch = 100; break;	
	case HEAD_ALIEN:	m_voicePitch = 90;  break;	
	case HEAD_OUVRIER:	m_voicePitch = 95;  break;
	}
}

int CCivil :: TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType)
{

	if ( pevInflictor && pevInflictor->flags & FL_CLIENT )
	{
		Remember( bits_MEMORY_PROVOKED );
		StopFollowing( TRUE );
	}

	// fait parler les autres en cas d'attaque d'un civil
	return CTalkMonster::TakeDamage(pevInflictor, pevAttacker, flDamage, bitsDamageType);
}


//=========================================================
// ISoundMask - returns a bit mask indicating which types
// of sounds this monster regards. In the base class implementation,
// monsters care about all sounds, but no scents.
//=========================================================
int CCivil :: ISoundMask ( void )
{
	return	bits_SOUND_WORLD	|
			bits_SOUND_COMBAT	|
			bits_SOUND_DANGER	|
			bits_SOUND_PLAYER;
}
	
//=========================================================
// PainSound
//=========================================================
void CCivil :: PainSound ( void )
{
	if (gpGlobals->time < m_painTime )
		return;
	
	m_painTime = gpGlobals->time + RANDOM_FLOAT(0.5, 0.75);

	switch (RANDOM_LONG(0,4))
	{
	case 0: EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "civil/civ_pain1.wav", 1, ATTN_NORM, 0, GetVoicePitch()); break;
	case 1: EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "civil/civ_pain2.wav", 1, ATTN_NORM, 0, GetVoicePitch()); break;
	case 2: EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "civil/civ_pain3.wav", 1, ATTN_NORM, 0, GetVoicePitch()); break;
	case 3: EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "civil/civ_pain4.wav", 1, ATTN_NORM, 0, GetVoicePitch()); break;
	case 4: EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "civil/civ_pain5.wav", 1, ATTN_NORM, 0, GetVoicePitch()); break;
	}
}

//=========================================================
// DeathSound 
//=========================================================
void CCivil :: DeathSound ( void )
{
	PainSound();
}


void CCivil::Killed( entvars_t *pevAttacker, int iGib )
{
	SetUse( NULL );	
	CTalkMonster::Killed( pevAttacker, iGib );
}


void CCivil :: SetActivity ( Activity newActivity )
{
	int	iSequence;

	iSequence = LookupActivity ( newActivity );

	// Choisi idle si pas d'animation disponible
	if ( iSequence == ACTIVITY_NOT_AVAILABLE )
		newActivity = ACT_IDLE;
	CTalkMonster::SetActivity( newActivity );
}


Schedule_t* CCivil :: GetScheduleOfType ( int Type )
{
	Schedule_t *psched;

	switch( Type )
	{
	// Raccrocher ici pour faire une boucle
	case SCHED_TARGET_FACE:
		// appelle la classe de base pour faire parler si on l'active 
		psched = CTalkMonster::GetScheduleOfType(Type);

		if (psched == slIdleStand)
			return slCivFaceTarget;	// override pour divers comportements
		else
			return psched;

	case SCHED_TARGET_CHASE:
		return slCivFollow;
	
	case SCHED_CANT_FOLLOW:
		return slCivStopFollowing;

	case SCHED_PANIC:
		return slCivPanic;

	case SCHED_TARGET_CHASE_SCARED:
		return slCivFollowScared;

	case SCHED_TARGET_FACE_SCARED:
		return slCivFaceTargetScared;

	case SCHED_IDLE_STAND:
		// appelle la classe de base pour que le civil puisse parler pendant idle
		psched = CTalkMonster::GetScheduleOfType(Type);

		if (psched == slIdleStand)
			return slIdleCivStand;
		else
			return psched;

	case SCHED_HIDE:
		return slCivilHide;

	case SCHED_STARTLE:
		return slCivilStartle;

	case SCHED_FEAR:
		return slCivFear;
	}

	return CTalkMonster::GetScheduleOfType( Type );
}

Schedule_t *CCivil :: GetSchedule ( void )
{
	// pour ne pas continuer  appeler par EHANDLE 
	CBaseEntity *pEnemy = m_hEnemy;

	if ( HasConditions( bits_COND_HEAR_SOUND ) )
	{
		CSound *pSound;
		pSound = PBestSound();

		ASSERT( pSound != NULL );
		if ( pSound && (pSound->m_iType & bits_SOUND_DANGER) )
			return GetScheduleOfType( SCHED_TAKE_COVER_FROM_BEST_SOUND );
	}

	switch( m_MonsterState )
	{
	case MONSTERSTATE_ALERT:	
	case MONSTERSTATE_IDLE:
		
		if ( pev->iuser4 == 2 )//on fuit si on brle
				{
					return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN );
				}

		if ( pEnemy )
		{
			if ( HasConditions( bits_COND_SEE_ENEMY ) )
				m_fearTime = gpGlobals->time;
			else if ( DisregardEnemy( pEnemy ) )		// Aprs 15 secondes cach, on retourne  alert
			{
				m_hEnemy = NULL;
				pEnemy = NULL;
			}
		}

		if ( HasConditions(bits_COND_LIGHT_DAMAGE | bits_COND_HEAVY_DAMAGE))
		{
		
			return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN );
		}

		// A couvert si on entend un son de danger
		if ( HasConditions( bits_COND_HEAR_SOUND ) )
		{
			CSound *pSound;
			pSound = PBestSound();

			ASSERT( pSound != NULL );
			if ( pSound )
			{
				if ( pSound->m_iType & (bits_SOUND_DANGER | bits_SOUND_COMBAT) )
				{
					if ( gpGlobals->time - m_fearTime > 3 )	// A couvert seulement toutes les 3 secondes
					{
						m_fearTime = gpGlobals->time;		// Met  jour la dernire peur
						return GetScheduleOfType( SCHED_STARTLE );	// Se baisse pour une seconde
					}
				}
			}
		}

		// Comportement pour suivre le joueur
		if ( IsFollowing() )
		{
			if ( !m_hTargetEnt->IsAlive() )
			{
				
				StopFollowing( FALSE );
				break;
			}

			int relationship = R_NO;

			// Pas de danger, juste moi et le joueur
			if ( pEnemy != NULL )
				relationship = IRelationship( pEnemy );

		
			if ( relationship != R_DL && relationship != R_HT )
			{
				// Si je suis dj assez prs
				if ( TargetDistance() <= 128 )
				{
					
					if ( HasConditions( bits_COND_CLIENT_PUSH ) )	// Bouge de l, tu me cache le soleil!
						return GetScheduleOfType( SCHED_MOVE_AWAY_FOLLOW );
				}
				return GetScheduleOfType( SCHED_TARGET_FACE );	// Fait face et suit
			}
			else	// ajout pour le cas o le civil a peur, il bloque alors le chemin
			{
				if ( HasConditions( bits_COND_NEW_ENEMY ) ) // je viens de voir un danger
					return GetScheduleOfType( SCHED_FEAR );					// reaction de peur
				return GetScheduleOfType( SCHED_TARGET_FACE_SCARED );	// fait face et suit apeur.
			}
		}

		if ( HasConditions( bits_COND_CLIENT_PUSH ) )	// Bouge de l, tu me cache le soleil!
			return GetScheduleOfType( SCHED_MOVE_AWAY );

		// Dit donc a pue par ici!!
		TrySmellTalk();
		break;
	
	
	case MONSTERSTATE_COMBAT:
		
		if ( pev->iuser4 == 2 )//on fuit si on brle
			{
				return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN );
			}

		if ( HasConditions( bits_COND_NEW_ENEMY ) )
			return slCivFear;					// Montre et crie!
		if ( HasConditions( bits_COND_SEE_ENEMY ) )
			return slCivilCover;		// A couvert
		
		if ( HasConditions( bits_COND_HEAR_SOUND ) )
			return slTakeCoverFromBestSound;	// A couvert et panique 
		return slCivilCover;			// Court et  couvert
		break;
	}
	
	return CTalkMonster::GetSchedule();
}

MONSTERSTATE CCivil :: GetIdealState ( void )
{
	switch ( m_MonsterState )
	{
	case MONSTERSTATE_ALERT:
	case MONSTERSTATE_IDLE:
		
		
		if ( HasConditions( bits_COND_NEW_ENEMY ) )
		{
			if ( IsFollowing() )
			{
				int relationship = IRelationship( m_hEnemy );
				if ( relationship != R_FR || relationship != R_HT && !HasConditions( bits_COND_LIGHT_DAMAGE | bits_COND_HEAVY_DAMAGE ) )
				{
					// Ne change pas d'tat vers combat s'il suit le joueur
					m_IdealMonsterState = MONSTERSTATE_ALERT;
					return m_IdealMonsterState;
				}
				StopFollowing( TRUE );
			}
		}
		else if ( HasConditions( bits_COND_LIGHT_DAMAGE | bits_COND_HEAVY_DAMAGE ) )
		{
			// Arrte de suivre si tu est bless
			if ( IsFollowing() )
				StopFollowing( TRUE );
		}
		break;

	case MONSTERSTATE_COMBAT:
		{
			CBaseEntity *pEnemy = m_hEnemy;
			if ( pEnemy != NULL )
			{
				
				
				if ( DisregardEnemy( pEnemy ) )		// Aprs 15 secondes cach, retour  alert
				{
					// On oublie l'ennemi lorsqu'on retourne  alert
					m_IdealMonsterState = MONSTERSTATE_ALERT;
					m_hEnemy = NULL;
					return m_IdealMonsterState;
				}
				// Suit si pas trop peur
				if ( m_hTargetEnt != NULL )
				{
					m_IdealMonsterState = MONSTERSTATE_ALERT;
					return m_IdealMonsterState;
				}

				if ( HasConditions ( bits_COND_SEE_ENEMY ) )
				{
					m_fearTime = gpGlobals->time;
					m_IdealMonsterState = MONSTERSTATE_COMBAT;
					return m_IdealMonsterState;
				}

			}
		}
		break;
	}

	return CTalkMonster::GetIdealState();
}



int CCivil::FriendNumber( int arrayNumber )
{
	static int array[3] = { 1, 2, 0 };
	if ( arrayNumber < 3 )
		return array[ arrayNumber ];
	return arrayNumber;
}


//=========================================================
// Civil mort (De profundis!!!!)
//=========================================================
class CDeadCivil : public CBaseMonster
{
public:
	void Spawn( void );
	int	Classify ( void ) { return	CLASS_HUMAN_PASSIVE; }

	void KeyValue( KeyValueData *pkvd );
	int	m_iPose;// quelle sequence de pose
	static char *m_szPoses[7];
};
char *CDeadCivil::m_szPoses[] = { "lying_on_back", "lying_on_stomach", "dead_sitting", "dead_hang", "dead_table1", "dead_table2", "dead_table3" };

void CDeadCivil::KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "pose"))
	{
		m_iPose = atoi(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else
		CBaseMonster::KeyValue( pkvd );
}
LINK_ENTITY_TO_CLASS( monster_civil_dead, CDeadCivil );

//
// ********** DeadScientist SPAWN **********
//
void CDeadCivil :: Spawn( )
{
	PRECACHE_MODEL("models/civil.mdl");
	SET_MODEL(ENT(pev), "models/civil.mdl");
	
	pev->effects		= 0;
	pev->sequence		= 0;
	// les cadavres sont plus fragiles
	pev->health			= 8;
	
	m_bloodColor = BLOOD_COLOR_RED;

	if ( pev->body == -1 )
	{// -1 choisit une tte au hasard
		pev->body = RANDOM_LONG(0, NUM_CIVIL_HEADS-1);
	}
	

	pev->sequence = LookupSequence( m_szPoses[m_iPose] );
	if (pev->sequence == -1)
	{
		ALERT ( at_console, "Dead civil with bad pose\n" );
	}

	
	MonsterInitDead();
}


