/***
*
*	Code pour le fusil  concussion
*	Par Jean-Franois Vuagniaux
*	vuagniaux.jf@com.mcnet.ch
*
****/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "soundent.h"
#include "gamerules.h"
#include "effects.h"
#include "decals.h"
#include "customentity.h"
#include "monsters.h"




int m_iconcglow;
int m_iconcballs;


int  m_iTrace;

enum conc_e
{
	CONC_LONGIDLE = 0,
	CONC_IDLE1,
	CONC_RELOAD,
	CONC_FIRE1,
	CONC_HOLSTER,
	CONC_DEPLOY,
	CONC_FIRE2,
	
};

#define CONC_BLOWSPEED  10 // facteur de correction pour la vitesse de projection
#define CONC_BLOWRADIUS 256 // rayon d'action de l'explosion
#define JET_SPRITE	"sprites/laserbeam.spr"

class CConcBall : public CBaseEntity // le boule de gaz cre
{
public:
	void Spawn( void );
	void EXPORT AnimThink( void );
	int m_count;
	int m_maxFrame;

	virtual int Save( CSave &save );
	virtual int Restore(CRestore &restore );
	static TYPEDESCRIPTION m_SaveData[];

		
	static CConcBall *CreateConcBall( Vector vecOrigin, Vector vecVelocity);

};
LINK_ENTITY_TO_CLASS( conc_ball, CConcBall );

TYPEDESCRIPTION CConcBall::m_SaveData[] =
{
	DEFINE_FIELD(CConcBall, m_maxFrame, FIELD_INTEGER ),
	DEFINE_FIELD(CConcBall, m_count, FIELD_INTEGER ),
};

  IMPLEMENT_SAVERESTORE( CConcBall, CBaseEntity );


//=========================================================
// Gnre une boule de gaz  partir de sa dfinition de classe
//=========================================================



void CConcBall :: Spawn( void )
{

	
	pev->movetype = MOVETYPE_BOUNCEMISSILE;
	pev->classname = MAKE_STRING( "Conc_ball");
	pev->solid = SOLID_NOT;
	pev->rendermode	= kRenderTransAdd;
	pev->renderamt = 255;


	SET_MODEL(ENT(pev), "sprites/concball.spr" );

	pev->frame = 0;
	pev->scale = 1;

	UTIL_SetOrigin( pev, pev->origin );
	UTIL_SetSize( pev, Vector(0, 0, 0), Vector(0, 0, 0));

	
	m_count = 0;

	m_maxFrame = (float) MODEL_FRAMES( pev->modelindex ) - 1;
}

void CConcBall :: AnimThink( void )
{

	pev->nextthink = gpGlobals->time + 0.1;
	

	if ( pev->frame++ ) // animation de la sprite
	{
		if ( pev->frame > m_maxFrame )
		{
			pev->frame = m_maxFrame - 2; // rejoue seulement la fin de la squence
		}

		pev->renderamt = 255 - ( m_count * 5 );// fading

		pev->velocity = ( pev->velocity ) * 0.8; // ralentissement
	}
	
	  
	if ( m_count > 50 )//supprime aprs 5 secondes
	{
		UTIL_Remove( this );
		return;
	}

	m_count++;

	


}

CConcBall *CConcBall::CreateConcBall( Vector vecOrigin, Vector vecVelocity)
{
	CConcBall *pBall = GetClassPtr( (CConcBall *)NULL );

	pBall->Spawn();

	UTIL_SetOrigin( pBall->pev, vecOrigin );

	pBall->pev->velocity = vecVelocity * 50;
	pBall->pev->scale= 1.0;
	pBall->SetThink( AnimThink );
	pBall->pev->nextthink = gpGlobals->time +0.1;
	
	
	return pBall;
}

//=======================================================================
// Effet de flammes pour immoler l'ennemi
//=======================================================================

class CConcFlam : public CBaseEntity
{
public:

	void Spawn(void);
	static void Allume( CBaseEntity *pEntity );
	void EXPORT Animate(void);
	float m_flStartTime;
	int m_maxFrame;
	int m_animCount;
	int m_scaleCount;
	int m_clock;

	CBaseEntity *pBrule;


};

void CConcFlam::Spawn(void)
{

	pev->movetype = MOVETYPE_FLY;
	pev->classname = MAKE_STRING( "flammes" );
	pev->solid = SOLID_NOT;
	pev->rendermode = kRenderTransAdd;
	pev->renderamt = 255;

	SET_MODEL(ENT(pev), "sprites/concflam.spr" );
	pev->frame = 0;
	pev->scale = 1.3;

	UTIL_SetSize( pev, Vector( 0, 0, 0 ), Vector( 0, 0, 0 ) );
	
	m_flStartTime = gpGlobals->time;

	m_maxFrame = (float) MODEL_FRAMES( pev->modelindex ) - 1;

	m_animCount = 0;
	m_scaleCount = 0;
	m_clock = 0;
	
	EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/conc1.wav", 1.0, ATTN_NORM );
	
	CSoundEnt::InsertSound ( bits_SOUND_DANGER, pev->origin, 400, 1.0 );//son de danger pour permettre  la cible de ragir et de se mettre  couvert


}

void CConcFlam::Animate(void)
{
	
	if (!pBrule )
	{
		UTIL_Remove( this );
		return;
	}
	
	Vector vecOrigin = pBrule->pev->origin;

	Vector taille = pBrule->pev->size;

	vecOrigin.z = vecOrigin.z + (taille.z / 1);//on monte un peu

	UTIL_SetOrigin( pev, vecOrigin );//on raligne l'origine de la flamme sur celle du modle brl

	//ajout de lumire mise par les flammes

	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
			WRITE_BYTE(TE_DLIGHT);
			WRITE_COORD(vecOrigin.x);	// X
			WRITE_COORD(vecOrigin.y);	// Y
			WRITE_COORD(vecOrigin.z);	// Z
			WRITE_BYTE( 25 );		// radius * 0.1
			WRITE_BYTE( 250 );		// r
			WRITE_BYTE( 110 );		// g
			WRITE_BYTE( 20 );		// b
			WRITE_BYTE( 1 );		// time * 10   1 doit tre 0.1 seconde
			WRITE_BYTE( 0 );		// decay * 0.1
		MESSAGE_END( );

	//ALERT( at_console, "SANTE %.0f\n: ", pBrule->pev->health);

	//le feu continue  infliger des dommages
	if ( pBrule->pev->health > 0 )
	{
		ClearMultiDamage( );
		pBrule->TakeDamage ( pev, pev, 0.5, DMG_BURN );
		ApplyMultiDamage( pev, pev );
	}
	
	
	if ( pev->frame++ )//animation en boucle
	{
		if ( pev->frame > m_maxFrame )
		{
			pev->frame = 0;
		}
	}
	
	// l'attenuation dure 3 secondes aprs deux secondes de brulure et si la sant a diminu suffisemment
	
	if (gpGlobals->time > m_flStartTime + 2 && pBrule->pev->health < 15)	//15 parce que l'attnuation va durer 3 secondes 
																			//ce qui donne 30 impulsion de 0.5 points de sant		
	{
		pev->renderamt = 255 - ( m_animCount * 8 );//le feu diminue d'intensit

		if ( pev->renderamt < 10 )
		{
			UTIL_Remove( this );
			return;
		}
		/*
		pev->scale = 1 - (m_scaleCount / 10 );// on rduit l'chelle pour diminuer la flamme

		//ALERT( at_console, "SCALECOUNT %i\n: ", m_scaleCount);
		
		if ( m_clock == 3 )
		{
			m_clock = -1;
			m_scaleCount++;
		}
		
		m_clock++;
		*/
		m_animCount++;
	}
	
	if ( gpGlobals->time > m_flStartTime + 5 && pBrule->pev->deadflag == DEAD_DEAD)//le feu s'teint aprs 5 secondes et si le brl est mort
	{
		
		UTIL_Remove( this );
		return;
	}

	pev->nextthink = gpGlobals->time + 0.1;

}

void CConcFlam::Allume(CBaseEntity *pEntity)
{

	CConcFlam *pFlam = GetClassPtr( (CConcFlam *)NULL );
	pFlam->Spawn();
	
	UTIL_SetOrigin( pFlam->pev, pEntity->pev->origin );
	
	pFlam->pBrule = pEntity;//on assigne l'entit brle  une variable de la classe pour qu'Animate puisse l'utiliser

	
	pFlam->SetThink ( Animate );
	pFlam->pev->nextthink = gpGlobals->time + 0.1;

}
//=======================================================================
// Nuages de gaz enrobant le sillage pour le jet
//=======================================================================


class CConcSpray : public CBaseEntity
{
public:

	void Spawn(void);
	static void Shoot( Vector vecStart, Vector vecVelocity );
	void EXPORT Animate( void );
	float m_flShootTime;

};

void CConcSpray::Spawn(void)
{
	pev->movetype = MOVETYPE_BOUNCEMISSILE;
	pev->classname = MAKE_STRING( "concspray" );
	pev->solid = SOLID_NOT;
	pev->rendermode = kRenderTransAdd;
	pev->renderamt = 150;

	SET_MODEL(ENT(pev), "sprites/concspray.spr" );
	pev->frame = 1;
	pev->scale = 0.8;

	UTIL_SetSize( pev, Vector( 0, 0, 0 ), Vector( 0, 0, 0 ) );
	
	m_flShootTime = gpGlobals->time;
}

void CConcSpray::Animate( void )
{
	if ( gpGlobals->time > m_flShootTime + 0.1 )
	{
		pev->renderamt = 50;
		pev->velocity = pev->velocity * 0.5;
	}

	if ( gpGlobals->time > m_flShootTime + 0.2 )
	{
		UTIL_Remove( this );
		return;
	}

	pev->nextthink = gpGlobals->time + 0.1;
}

void CConcSpray::Shoot(Vector vecStart, Vector vecVelocity)
{

	CConcSpray *pSpray = GetClassPtr( (CConcSpray *)NULL );
	pSpray->Spawn();
	
	UTIL_SetOrigin( pSpray->pev, vecStart );
	pSpray->pev->velocity = vecVelocity;

	pSpray->SetThink ( Animate );
	pSpray->pev->nextthink = gpGlobals->time + 0.1;

}

//========================================================================
// Projectile de gaz ionis condens projet par le fusil  concussion
//========================================================================

class CConcgas : public CBaseEntity
{
public:
	void Spawn( void );

	static void Shoot( entvars_t *pevOwner, Vector vecStart, Vector vecVelocity );
	void Touch( CBaseEntity *pOther );
	void EXPORT Animate( void );

	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];
	void BlastAround( Vector vecSrc, entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, float flCorrection, float flRadius, int bitsDamageType );
	int  m_maxFrame;
	
};

LINK_ENTITY_TO_CLASS( concgaz, CConcgas );

TYPEDESCRIPTION	CConcgas::m_SaveData[] = 
{
	DEFINE_FIELD( CConcgas, m_maxFrame, FIELD_INTEGER ),
};

IMPLEMENT_SAVERESTORE( CConcgas, CBaseEntity );


void CConcgas:: Spawn( void )
{
	pev->movetype = MOVETYPE_FLY;
	pev->classname = MAKE_STRING( "concgas" );
	
	pev->solid = SOLID_BBOX;
	pev->rendermode = kRenderTransAdd;
	pev->renderamt = 255;

	SET_MODEL(ENT(pev), "sprites/concgas.spr");
	pev->frame = 0;
	pev->scale = 0.5;

	UTIL_SetSize( pev, Vector( 0, 0, 0), Vector(0, 0, 0) );
	
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );

		WRITE_BYTE( TE_BEAMFOLLOW ); // le sillage suit le projectile
		WRITE_SHORT(entindex());	// entit
		WRITE_SHORT(m_iTrace );	// modle
		WRITE_BYTE( 1 ); // dure
		WRITE_BYTE( 4 );  // largeur
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );	// luminosit

	MESSAGE_END();  

	m_maxFrame = (float) MODEL_FRAMES( pev->modelindex ) - 1;
}

void CConcgas::Animate( void )
{
	pev->nextthink = gpGlobals->time + 0.1;

	if ( pev->frame++ )
	{
		if ( pev->frame > m_maxFrame )
		{
			pev->frame = 0;
		}
	}
}

void CConcgas::Shoot( entvars_t *pevOwner, Vector vecStart, Vector vecVelocity )
{
	CConcgas *pConc = GetClassPtr( (CConcgas *)NULL );
	pConc->Spawn();
	
	UTIL_SetOrigin( pConc->pev, vecStart );
	pConc->pev->velocity = vecVelocity;
	pConc->pev->owner = ENT(pevOwner);

	pConc->SetThink ( Animate );
	pConc->pev->nextthink = gpGlobals->time + 0.1;
}

void CConcgas :: BlastAround( Vector vecSrc, entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, float flCorrection, float flRadius, int bitsDamageType )
{

	// vecScr est le centre du rayon, origine de l'explosion
	// flDamage est la quantit de dommage caus
	// flRadius est le rayon d'action de l'explosion

	CBaseEntity *pEntity = NULL;
	TraceResult	tr;
	float		flAdjustedDamage, falloff;
	Vector		vecEnt;
	Vector		vecBlowSpeed;
	int			m_MassFactor;

	if ( flRadius )
		falloff = flDamage / flRadius;
	else
		falloff = 1.0;



	int bInWater = (UTIL_PointContents ( vecSrc ) == CONTENTS_WATER);

	vecSrc.z += 1;// surlve l'explosion lgrement au-dessus du sol

	if ( !pevAttacker )
		pevAttacker = pevInflictor;

	// parcourt toutes les entites contenues dans un rayon donn

	while ((pEntity = UTIL_FindEntityInSphere( pEntity, vecSrc, flRadius )) != NULL)
	{
		if ( pEntity->pev->iuser3 != 0 )//agit seulement sur les entits ayant une masse dfinie
		{

			// le souffle ne se propage pas d'un milieu  l'autre, 
			// si c'est dans l'eau a n'affecte pas les entites hors de l'eau

			if (bInWater && pEntity->pev->waterlevel == 0)
				continue;
			if (!bInWater && pEntity->pev->waterlevel == 3)
				continue;

			vecEnt = pEntity->BodyTarget( vecSrc );// position de l'entit
			
			// on va s'assurer que l'entit n'est pas protge par le dcor, 
			// on trace une ligne pour vrifier

			UTIL_TraceLine ( vecSrc, vecEnt, dont_ignore_monsters, ENT(pevInflictor), &tr );

			if ( tr.flFraction == 1.0 || tr.pHit == pEntity->edict() )
			{
								
				if (tr.fStartSolid)
				{
					// si on est bloqu  l'intrieur on ajuste
					tr.vecEndPos = vecSrc;
					tr.flFraction = 0.0;
				}
				
				// le domage diminue avec la distance
				
				flAdjustedDamage = ( vecSrc - tr.vecEndPos ).Length() * falloff;
				flAdjustedDamage = flDamage - flAdjustedDamage;
					
				if ( flAdjustedDamage < 0 )
				{
					flAdjustedDamage = 0;
				}
				// vitesse de projection de l'entit

				vecBlowSpeed = ( tr.vecEndPos - vecSrc ); 

				vecBlowSpeed = vecBlowSpeed.Normalize();

				vecBlowSpeed = (vecBlowSpeed * flAdjustedDamage) * flCorrection;

				//ALERT( at_console, "BlowSpeed %.0f %.0f %.0f\n: ", vecBlowSpeed.x, vecBlowSpeed.y, vecBlowSpeed.z );
				
				m_MassFactor = pEntity->pev->iuser3;//la vitesse de projection dpend de la masse

				if ( m_MassFactor == 0 )//attention  la division par 0 !!
				{
					m_MassFactor = 1;
				}
				

				
				pEntity->pev->velocity = pEntity->pev->velocity + ( vecBlowSpeed / m_MassFactor );
			
				// ALERT( at_console, "hit %s\n", STRING( pEntity->pev->classname ) );

				if( pEntity->pev->takedamage != DAMAGE_NO )
				{
					if (tr.flFraction != 1.0 )
					{
						ClearMultiDamage( );
						pEntity->TakeDamage ( pevInflictor, pevAttacker, flAdjustedDamage, bitsDamageType );
						ApplyMultiDamage( pevInflictor, pevAttacker );
					}
					else
					{
						pEntity->TakeDamage ( pevInflictor, pevAttacker, flAdjustedDamage, bitsDamageType );
					}
				}
			}
		}
	}
}



void CConcgas :: Touch ( CBaseEntity *pOther )
{
	//ALERT( at_console, "TOUCH\n");
	TraceResult tr;
	int		iPitch;

	int i ;

	entvars_t	*pevOwner;

		pevOwner = VARS( pev->owner );

	iPitch = RANDOM_FLOAT( 90, 110 );
		
	UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );
	//pas d'effet si on touche le ciel

	if ( UTIL_IsTextureSky( &tr, pev->origin, pev->origin + pev->velocity * 10 ) )
	{
		SetThink ( SUB_Remove );
		pev->nextthink = gpGlobals->time;

		return;

	}

	
	//son d'explosion 
	EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "weapons/concboum.wav", 1, 0.4, 0, iPitch );	
	
	// cration d'un effet de souffle d'explosion dans un rayon donn, 
	// toutes les entits sont projetes et endommages
	
	
	
	Vector vecExplo = tr.vecEndPos;

	CSoundEnt::InsertSound ( bits_SOUND_DANGER, vecExplo, 400, 0.3 );//son de danger pour ceux qui ont survcu  l'explosion

	BlastAround(vecExplo, pev, pevOwner, gSkillData.concussionDmg, CONC_BLOWSPEED, CONC_BLOWRADIUS, DMG_NEVERGIB);	
	
	Vector vecInverse = -1 * ( pev->velocity );// direction inverse  celle du projectile

	vecInverse = vecInverse.Normalize();

	// cre un nuage de gas
	vecExplo = vecExplo + ( vecInverse * 30 );// loigne la sprite du point d'impacte pour viter la pntration

	for ( i = 0; i < 8; i++ )// 8 boules de gaz sont mises avec des directions variables
	{
		vecInverse.x = vecInverse.x + RANDOM_FLOAT( -1, 1 );
		vecInverse.y = vecInverse.y + RANDOM_FLOAT( -1, 1 );
		vecInverse.z = (vecInverse.x * vecInverse.x) + (vecInverse.y * vecInverse.y);

		CConcBall *pBall = CConcBall::CreateConcBall( vecExplo, vecInverse );
	}
	

	//Nuage de vapeur :

	MESSAGE_BEGIN( MSG_PAS, SVC_TEMPENTITY, vecExplo );
		WRITE_BYTE( TE_GLOWSPRITE );
		WRITE_COORD( vecExplo.x);		// pos
		WRITE_COORD( vecExplo.y);
		WRITE_COORD( vecExplo.z);
		WRITE_SHORT( m_iconcglow );				// modle
		WRITE_BYTE( 3 );	// dure
		WRITE_BYTE( 128 );					// taille
		WRITE_BYTE( 3000 );			// luminosit
	MESSAGE_END();

			
/*
	
	MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, tr.vecEndPos );
		WRITE_BYTE( TE_SPRITETRAIL );
		WRITE_COORD( tr.vecEndPos.x );
		WRITE_COORD( tr.vecEndPos.y );
		WRITE_COORD( tr.vecEndPos.z );
		WRITE_COORD( tr.vecEndPos.x + tr.vecPlaneNormal.x );
		WRITE_COORD( tr.vecEndPos.y + tr.vecPlaneNormal.y );
		WRITE_COORD( tr.vecEndPos.z + tr.vecPlaneNormal.z );
		WRITE_SHORT( m_iconcballs );		// modle
		WRITE_BYTE( 8 );				// nombre
		WRITE_BYTE( 6 );				// dure
		WRITE_BYTE( 20 );				// taille
		WRITE_BYTE( 30 );				// amplitude * 0.1
		WRITE_BYTE( 30 );				// vitesse * 100
	MESSAGE_END();
	

*/	

	SetThink ( SUB_Remove );
	pev->nextthink = gpGlobals->time;
}

//===================================================================
// Fusil  concussion projetant un gaz ionis hautement comprim qui 
// se detend  l'impacte en provoquant un effet de souffle important
//===================================================================

class CConc : public CBasePlayerWeapon
{
public:
	int		Save( CSave &save );
	int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];
	
	void Spawn( void );
	void Precache( void );
	int iItemSlot( void ) { return 4; }
	int GetItemInfo(ItemInfo *p);
	int AddToPlayer( CBasePlayer *pPlayer );
	void PrimaryAttack( void );
	void SecondaryAttack( void );
	BOOL Deploy( void );
	void Holster( int skiplocal = 0 );
	void Reload( void );
	void WeaponIdle( void );

	void DrawJet(void);
	void DestroyJet(void);
	void  StopJet(void);
	void EXPORT FadeJet(void);
	float m_flNextAnimTime;
	float m_flNextAmmoUse;
	float m_flNextPush;
	int m_StartFade;
	int m_Accelerator;
	void JetPush(void);
	void CaPiqueUnMax(TraceResult tr, Vector vecDir, float m_flPortee);
	void AuFeuLesPompiers( CBaseEntity *pEntity ); 
	CBeam *m_pJet;
	BOOL m_JetStarted;
	BOOL m_Reload;
	Vector vecPush;
	BOOL immediat;
	
	
};
LINK_ENTITY_TO_CLASS( weapon_concussion, CConc );


TYPEDESCRIPTION	CConc::m_SaveData[] = 
{
	DEFINE_FIELD( CConc, m_pJet, FIELD_CLASSPTR ),
	DEFINE_FIELD( CConc, m_flNextAnimTime, FIELD_TIME ),
	DEFINE_FIELD( CConc, m_StartFade, FIELD_INTEGER ),
	DEFINE_FIELD( CConc, m_Accelerator, FIELD_INTEGER ),
	DEFINE_FIELD( CConc, m_flNextPush, FIELD_TIME ),
	DEFINE_FIELD( CConc, m_flNextAmmoUse, FIELD_TIME ),
	DEFINE_FIELD( CConc, m_JetStarted, FIELD_BOOLEAN ),
	DEFINE_FIELD( CConc, m_Reload, FIELD_BOOLEAN ),
	DEFINE_FIELD( CConc, immediat, FIELD_BOOLEAN ),
};
IMPLEMENT_SAVERESTORE( CConc, CBasePlayerWeapon );

//=========================================================
//=========================================================

void CConc::Spawn( )
{
	pev->classname = MAKE_STRING("weapon_concussion"); 
	Precache( );
	SET_MODEL(ENT(pev), "models/w_concussion.mdl");
	m_iId = WEAPON_CONC;

	m_iDefaultAmmo = CONC_DEFAULT_GIVE;

	pev->iuser3 = 1;

	FallInit();// prt  tomber au sol.
}


void CConc::Precache( void )
{
	PRECACHE_MODEL("models/v_concussion.mdl");
	PRECACHE_MODEL("models/w_concussion.mdl");
	PRECACHE_MODEL("models/p_concussion.mdl");


	PRECACHE_MODEL("sprites/concgas.spr");
	PRECACHE_MODEL("sprites/concspray.spr");
	PRECACHE_MODEL("sprites/concflam.spr");
	
	m_iconcglow = PRECACHE_MODEL("sprites/animglow01.spr");
	m_iconcballs = PRECACHE_MODEL("sprites/concball.spr");
	
	PRECACHE_MODEL("models/w_gasclip.mdl");
	PRECACHE_SOUND("items/gasclip1.wav");              

	PRECACHE_SOUND("weapons/gasclipout.wav");	// les sons de recharges sont dfinis dans le ficier strifle.qc
	PRECACHE_SOUND("weapons/gasclipin.wav");	// ils sont lancs lors de l'animation de recharge.
	PRECACHE_SOUND("weapons/chute1.wav");
	PRECACHE_SOUND("weapons/clack.wav");	// son pour idle1

	PRECACHE_SOUND ("weapons/conc1.wav");
	PRECACHE_SOUND ("weapons/conc2.wav");	//son d'attaque secondaire

	PRECACHE_SOUND ("weapons/concboum.wav");
	PRECACHE_SOUND ("weapons/conc_out.wav");// son de panne de fuel

	
	m_iTrace = PRECACHE_MODEL( "sprites/zbeam4.spr" );
	

}

int CConc::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "gas";
	p->iMaxAmmo1 = _GAS_MAX_CARRY;
	p->iMaxClip = CONC_MAX_CLIP;
	p->iSlot = 3;
	p->iPosition = 0;
	p->iFlags = 0;
	p->iId = m_iId = WEAPON_CONC;
	p->iWeight = CONC_WEIGHT;

	return 1;
}

int CConc::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

BOOL CConc::Deploy( )
{
	immediat = 0;
	return DefaultDeploy( "models/v_concussion.mdl", "models/p_concussion.mdl", CONC_DEPLOY, "concussion" );
}




void CConc::PrimaryAttack()
{
	// ne pas tirer sous l'eau
	if (m_pPlayer->pev->waterlevel == 3)
	{
		PlayEmptySound( );
		m_flNextPrimaryAttack = gpGlobals->time + 0.15;
		return;
	}
	
	// plus de munitions
	if (m_iClip <= 0)
	{
		PlayEmptySound();
		m_flNextPrimaryAttack = gpGlobals->time + 0.15;
		return;
	}

	m_pPlayer->m_iWeaponVolume = NORMAL_GUN_VOLUME;
	m_pPlayer->m_iWeaponFlash = NORMAL_GUN_FLASH;

	m_iClip--;

	m_pPlayer->pev->effects = (int)(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;


	if (1 || m_flNextAnimTime < gpGlobals->time)
	{
		SendWeaponAnim( CONC_FIRE1 );
		m_flNextAnimTime = gpGlobals->time + 2;
	}

	// animation de tir du joueur
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	 EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/conc1.wav", 1, ATTN_NORM, 0, 94 + RANDOM_LONG(0,0xf)); 
	

	UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );

	//modifier pour prendre en compte le dsalignement de la man par rapport  la ligne de vise 
	//car le coup porte trop bas  droite:
	//
	//tracer une ligne avec le vecteur: vector aiming et trouver le point d'impacte
	//corriger le vecteur de tir en consquence.
	
	Vector vecSrc	 = m_pPlayer->GetGunPosition( ) ; //point de vise du HUD
	Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );
	
	TraceResult tr;
	UTIL_TraceLine ( vecSrc, vecSrc + vecAiming * 8192, dont_ignore_monsters, ENT(m_pPlayer->pev), &tr ); //ligne de vise

	vecSrc = vecSrc + gpGlobals->v_forward * 35 + gpGlobals->v_right * 5 + gpGlobals->v_up * -13;//position du canon
	
	vecAiming = tr.vecEndPos - vecSrc;
	
	vecAiming = vecAiming.Normalize();

	// fin de correction
	
	//Vector vecSrc	 = m_pPlayer->GetGunPosition( ) + gpGlobals->v_forward * 1 + gpGlobals->v_right * 4 + gpGlobals->v_up * -10;
	//Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );
	//Vector vecAiming = gpGlobals->v_forward;

	// ajout de la dispersion du tir 
	
	Vector vecRight = gpGlobals->v_right;
	Vector vecUp = gpGlobals->v_up;

	float x, y, z;
	
			x = RANDOM_FLOAT(-0.5,0.5) + RANDOM_FLOAT(-0.5,0.5);
			y = RANDOM_FLOAT(-0.5,0.5) + RANDOM_FLOAT(-0.5,0.5);
			z = x*x+y*y;
	
	Vector vecSpread = VECTOR_CONE_3DEGREES ;
	vecAiming = vecAiming + x * vecSpread.x * vecRight + y * vecSpread.y * vecUp ;


	
// code pour tirer un projectile de concussion
	
	CConcgas::Shoot(m_pPlayer->pev,vecSrc,vecAiming * 2000);
	
	
	
	if (!m_iClip && m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		//la combinaison HEV  indique : plus de munitions
		m_pPlayer->SetSuitUpdate("!HEV_AMO0", FALSE, 0);
	
	// prochain coup possible dans 0.5 seconde
	
	m_flNextPrimaryAttack = m_flNextPrimaryAttack + 2;
	if (m_flNextPrimaryAttack < gpGlobals->time)
		m_flNextPrimaryAttack = gpGlobals->time + 2;

	m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );

	m_pPlayer->pev->punchangle.x = RANDOM_FLOAT( -5, -10 );

	m_pPlayer->pev->velocity = m_pPlayer->pev->velocity - gpGlobals->v_forward * 400;
}

void CConc::SecondaryAttack()
{
	// plus de munitions
	if (m_iClip <= 0)
	{
		if ( m_JetStarted )//si on a un jet en cours on stoppe le son et on stoppe le jet
		{
			StopJet();

			m_JetStarted = 0;
		}
		
		else
		{
			PlayEmptySound();
		}
		
		m_flNextSecondaryAttack = gpGlobals->time + 0.15;
		return;
	}

	m_pPlayer->m_iWeaponVolume = NORMAL_GUN_VOLUME;

	if ( !m_JetStarted )
	{
		SendWeaponAnim( CONC_FIRE2 );

		m_Accelerator = 0;
		
	}

	if ( m_flNextAmmoUse < gpGlobals->time )// utilise 2 units par seconde, le chargeur tient 5 seconde
	{
		m_iClip--;

		m_flNextAmmoUse = gpGlobals->time + 0.5;
	}


		// animation de tir du joueur
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );
	
	DrawJet(); // affiche le jet de gaz propulsif

	if (!m_iClip && m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		//la combinaison HEV  indique : plus de munitions
		m_pPlayer->SetSuitUpdate("!HEV_AMO0", FALSE, 0);
	

	m_flTimeWeaponIdle = gpGlobals->time + 0.1;// il faut que ce soit trs vite pour que WeaponIdle puisse stopper les effets


	//code pour la position et vitesse du sillage de gaz
	
	UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );
	
	Vector vecSrc	 = m_pPlayer->GetGunPosition( ) ; //point de vise du HUD
	Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );
	
	TraceResult tr;
	UTIL_TraceLine ( vecSrc, vecSrc + vecAiming * 8192, dont_ignore_monsters, ENT(m_pPlayer->pev), &tr ); //ligne de vise

	vecSrc = vecSrc + gpGlobals->v_forward * 35 + gpGlobals->v_right * 5 + gpGlobals->v_up * -13;//position du canon
	
	vecAiming = tr.vecEndPos - vecSrc;

	float m_flPortee;
	
	m_flPortee = vecAiming.Length();
	
	vecAiming = vecAiming.Normalize();

	
	CaPiqueUnMax( tr, vecAiming, m_flPortee);//a brle si on est trop prs
	
		
		
	Vector vecRight = gpGlobals->v_right;
	Vector vecUp = gpGlobals->v_up;

	float x, y, z;

	
			x = RANDOM_FLOAT(-0.5,0.5) + RANDOM_FLOAT(-0.5,0.5);
			y = RANDOM_FLOAT(-0.5,0.5) + RANDOM_FLOAT(-0.5,0.5);
			z = x*x+y*y;
	
	Vector vecSpread = VECTOR_CONE_10DEGREES ;
	vecAiming = vecAiming + x * vecSpread.x * vecRight + y * vecSpread.y * vecUp ;


	
// code pour crer le sillage de vapeur
	
	CConcSpray::Shoot(vecSrc,vecAiming * 1800);
	
	JetPush(); //on pousse le joueur
	
	if ( vecPush.z > 10 )//ajout pour que l'impulsion de dcollage ne dure pas plus d'une frame
		vecPush.z = 10;
	
	m_pPlayer->pev->velocity = m_pPlayer->pev->velocity + vecPush;
}


//==========================================================
// Ajout de dommages (brlures) pour qui se trouve dans le flux
//==========================================================

void CConc::CaPiqueUnMax(TraceResult tr, Vector vecDir, float m_flPortee)
{
	CBaseEntity *pEntity = CBaseEntity::Instance(tr.pHit);

	//ALERT(at_console, "BRULE %s\n", STRING(pEntity->pev->classname) );

	if (pEntity == NULL)
		return;
	
	float m_flReduction;

	if (tr.fAllSolid)
		return;
	
	
	if ( m_flPortee > 200 )
		return;
	
	
	if ( m_flPortee <= 200 )
		m_flReduction = 0.3;
	
	if ( m_flPortee < 150 )//on allume seulement  moyenne porte
	{
		m_flReduction  = 0.6;
		AuFeuLesPompiers(pEntity);	// on allume d'abord pour que le flag d'incendie soit mis avant que des dommages soient infligs
								// sinon le domage inflig va rduire le monstre en pure avant que la flamme puisse aparatre
								// dans combat.cpp j'ai ajout la condition d'incendie  ShouldGibMonster.
	}
	
	if ( m_flPortee < 100 )
		m_flReduction = 1;

	if ( pEntity->pev->iuser4 == 2 )//si l'entit est dj en flammes, on la laisse brler en paix sans rajouter de dommages
		m_flReduction = 0;
	
	if ( pev->dmgtime < gpGlobals->time )
		{
			
			ClearMultiDamage();
			if (pEntity->pev->takedamage)
			{
				if ( pEntity->pev->iuser4 == 3 )//c'est une fonction breakable certainement
				{
					pEntity->TakeDamage( m_pPlayer->pev, m_pPlayer->pev, (gSkillData.concussionDmg * m_flReduction) / 10, DMG_BURN );
				}
				else
				{	
					pEntity->TraceAttack( m_pPlayer->pev, (gSkillData.concussionDmg * m_flReduction) / 10, vecDir, &tr, DMG_BURN );
				}
			}

			ApplyMultiDamage(m_pPlayer->pev, m_pPlayer->pev);
			

			pev->dmgtime = gpGlobals->time + 0.1;
		}


	
}


//===================================================
// Pour se rchauffer auprs d'un bon feu
//===================================================

void CConc::AuFeuLesPompiers( CBaseEntity *pEntity )
{

	if ( !(pEntity->pev->takedamage))//on ne met pas le feu  ce qui ne peut pas tre endommag
		return;

	if ( pEntity->IsPlayer() )//pour les maladroits!!
		return;

	if ( pEntity->pev->iuser3 < 5 && pEntity->pev->iuser4 == 1 )	//on n'incendie pas les gros bidules, 
																//iuser4 prend le flag d'incendie pour ne pas allumer 2 fois la mme entit 
	{
		
		if ( pEntity->pev->deadflag == DEAD_NO )//on ne va pas incinrer nos morts
		{
			CConcFlam::Allume(pEntity);
			pEntity->pev->iuser4 = 2;
		}

	}
	else if (pEntity->pev->iuser4 == 0)//pour les func_breakable
	{
		pEntity->pev->iuser4 = 3;
	}
}

void CConc::JetPush( void )
{

	BOOL m_decole;

	if ( m_flNextPush < gpGlobals->time )
	{
		if ( m_Accelerator > 5 )	//on limite l'effet pour viter qu'aprs quelques secondes la vitesse 
									//sur les axes X et Y soient toujours au maximum, mme si l'angle de pousse est vertical
			m_Accelerator = 5;

		vecPush = ( gpGlobals->v_forward ) * -1;

		if ( vecPush.z > 0.5 && (m_pPlayer->pev->flags & FL_ONGROUND))//si l'arme pointe vers le bas de plus de 45 on donne une impulsion pour faire dcoller le joueur
		{
			m_decole = 1;
		}
		else
		{
			m_decole = 0;
		}


		vecPush.x = vecPush.x * 10 * m_Accelerator;//on acclre au dbut
		vecPush.y = vecPush.y * 10 * m_Accelerator;
		vecPush.z = vecPush.z * 2 * m_Accelerator;

		if ( vecPush.x > 50 )//on limite la vitesse  50 sur les axes X et Y
			vecPush.x = 50;
		if ( vecPush.y > 50 )
			vecPush.y = 50;
		
		if (vecPush.z > 10 )// on limite la vitesse verticale  10 (axe Z)
			vecPush.z = 10;

		if ( m_decole )
			vecPush.z = 200;//impulsion pour dcoller

		m_Accelerator++;
		m_flNextPush = gpGlobals->time + 0.1;//on va pousser  nouveau dans 0.1 seconde
	}
	

	
	
	m_pPlayer->pev->velocity = m_pPlayer->pev->velocity + vecPush;
}


void CConc::StopJet( void )
{
	
	STOP_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/conc2.wav");//stoppe le bruit du jet
			
	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/conc_out.wav", 1.0, ATTN_NORM);
		
	m_StartFade = 0;

	if ( immediat == 0 )
	{
		//ALERT(at_console, "SETTHINK\n" );
		SetThink( FadeJet );
		pev->nextthink = gpGlobals->time + 0.1;
	}
	else
	{
		//ALERT(at_console, "IMMEDIAT\n");
		if ( m_pJet )
			DestroyJet();
	}
	

}

void CConc::FadeJet( void )
{

	if ( m_StartFade < 6 && m_pJet)
	{

		
		m_StartFade++;

		m_pJet->SetBrightness(200 - ( m_StartFade * 30 ) );
		
		SetThink( FadeJet );
		pev->nextthink = gpGlobals->time + 0.1;

	}
	else
	{
		if ( m_pJet )
			DestroyJet();
		
		if ( m_Reload )
		{
			
			DefaultReload( CONC_MAX_CLIP, CONC_RELOAD, 4 );
			
			m_Reload = 0;
		}
		
	}
}


void CConc::DrawJet(void)
{
	DestroyJet(); //pour un meilleur raffraichissement si on change de secteur

	m_pJet = CBeam::BeamCreate( JET_SPRITE, 25 );
	
	m_pJet->EntsInit( m_pPlayer->entindex(), m_pPlayer->entindex() );

	m_pJet->SetEndAttachment( 2 );
	m_pJet->SetStartAttachment( 1 );

	m_pJet->pev->spawnflags |= SF_BEAM_TEMPORARY;
	m_pJet->SetFlags( BEAM_FSHADEOUT );
	m_pJet->SetScrollRate( 50 );
	m_pJet->SetNoise( 2 );
	m_pJet->SetColor( 243, 235, 162 );
	m_pJet->SetBrightness( 255 );
	m_pJet->SetWidth( 25 );

	if ( !m_JetStarted )
	{

		EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/conc2.wav", 1.0, ATTN_NORM);
		
		m_JetStarted = 1;
	
	}
}

void CConc::DestroyJet( void )
{

	if ( m_pJet )
	{
		UTIL_Remove( m_pJet );
		m_pJet = NULL;

	}

}


void CConc::Reload( void )
{
	
	
	if ( m_JetStarted )
	{
		m_Reload = 1;
		
		StopJet();
	}
	else
	{
		DefaultReload( CONC_MAX_CLIP, CONC_RELOAD, 4 );
	}
}

void CConc::Holster( int skiplocal )
{
	immediat = 1;//pour que le jet stoppe immdiatement puisque l'animation n'est pas encore prise en charge actuellement
	
	if ( m_JetStarted )
	{
		StopJet();
	}
	m_fInReload = FALSE;	// arrte l'animation de recharge

	m_pPlayer->m_flNextAttack = gpGlobals->time + 0.5;

	SendWeaponAnim( CONC_HOLSTER );
	

}
void CConc::WeaponIdle( void )//animations de repos
{
	ResetEmptySound( );

	

	if ( m_JetStarted )
	{

		StopJet();

		m_JetStarted = 0;
	}

	m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );

	if (m_flTimeWeaponIdle > gpGlobals->time)
		return;

	int iAnim;
	switch ( RANDOM_LONG( 0, 1 ) )
	{
	case 0:	
		iAnim = CONC_LONGIDLE;	
		break;
	
	default:
	case 1:
		iAnim = CONC_IDLE1;
		break;
	}

	SendWeaponAnim( iAnim );

	m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 5, 15 );// temps d'attente avant la prochaine animation.
}



class CConcAmmoClip : public CBasePlayerAmmo // classe de la munitions du fusil  concussion
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_gasclip.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_gasclip.mdl");
		PRECACHE_SOUND("items/gasclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		int bResult = (pOther->GiveAmmo( AMMO_CONCCLIP_GIVE, "gas", _GAS_MAX_CARRY) != -1);
		if (bResult)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/gasclip1.wav", 1, ATTN_NORM);
		}
		return bResult;
	}
};
LINK_ENTITY_TO_CLASS( ammo_gasclip, CConcAmmoClip );




















