//===============================================
//
// Champs de force 
// Cre un champs de force anim, avec son et dommage caus  celui qui le touche
// On peut le commuter pour le rendre actif ou inactif et rendre le passage bloqu ou libre.
// Il va se comporter comme une surface blinde avec les blasters et va les rflchir comme un mirroir.
//
// 
// Par Jean-Franois Vuagniaux 
//
// vuagniaux.jf@bluewin.ch
//
//==============================================================


#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "player.h"
#include "saverestore.h"
#include "trains.h"			// trigger_camera has train functionality
#include "gamerules.h"
#include "..\apifmod\fmod.h"

#define FIELD_START_ON	2 //le champ de force dmarre allum
#define FIELD_DAMAGE_CLIENT_ONLY 8 // le champs de force ne blesse pas les monstres



//===============================================

class CTriggerField : public CBaseEntity
{
	void Spawn( void );
	void KeyValue( KeyValueData *pkvd );
	void Touch( CBaseEntity *pOther );
	void EXPORT Toggle ( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	void PlayHumSound( int OnOff );  // boudonnement du champs de force
	void EXPORT InitHumSound( void );
	void Precache();
	void EXPORT ResetRender();
	void EXPORT Animate();
	void EXPORT AnimStart();
	void EXPORT AnimClose();

	float m_renderamt;

	BOOL demarre;

	int animcount;
	
	virtual int	ObjectCaps( void ) { return CBaseEntity :: ObjectCaps() & ~FCAP_ACROSS_TRANSITION; }

	float m_firstTouch;
	int magsealed;

	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );

	static	TYPEDESCRIPTION m_SaveData[];


};
LINK_ENTITY_TO_CLASS( trigger_forcefield, CTriggerField );

TYPEDESCRIPTION	CTriggerField::m_SaveData[] = 
{
	DEFINE_FIELD( CTriggerField, m_renderamt, FIELD_FLOAT ),
	DEFINE_FIELD( CTriggerField, m_firstTouch, FIELD_FLOAT ),
	DEFINE_FIELD( CTriggerField, magsealed, FIELD_INTEGER ),
};

//IMPLEMENT_SAVERESTORE( CTriggerField, CBaseEntity );

int CTriggerField::Save( CSave &save )
{
	if ( !CBaseEntity::Save(save) )
		return 0;
	return save.WriteFields( "CTriggerField", this, m_SaveData, ARRAYSIZE(m_SaveData) );
}

int CTriggerField ::Restore( CRestore &restore )
{

	if ( !CBaseEntity::Restore(restore) )
		return 0;

	int status = restore.ReadFields( "CTriggerField", this, m_SaveData, ARRAYSIZE(m_SaveData) );

	SetThink(InitHumSound); //relance le son et l'animation

	pev->nextthink = pev->ltime + 0.5;

	pev->iuser3 = magsealed;//remet le falg de blindage  la bonne valeur

	return status;
}

void CTriggerField :: KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "damage"))
	{
		pev->dmg = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "health"))
	{
		pev->health = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "renderamt"))
	{
		pev->renderamt = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else
	{
		CBaseEntity::KeyValue( pkvd );
	}
}

void CTriggerField :: Spawn( )
{

	Precache();

	magsealed		= 1000;	
	pev->angles		= g_vecZero;
	pev->movetype	= MOVETYPE_PUSH;  // so it doesn't get pushed by anything
	pev->solid		= SOLID_BSP;
	pev->iuser3		= magsealed; //surface blinde

	m_renderamt = pev->renderamt;//pour stocker la valeur de render defini dans la carte

	//ALERT( at_console, "IUSER3 %d\n RENDERAMT %f\n", pev->iuser3, m_renderamt);
	
	SET_MODEL(ENT(pev), STRING(pev->model));    // dfini la taille et la position dans le monde

	pev->flags |= FL_WORLDBRUSH;

	SetUse ( Toggle );

	demarre = 0;
	animcount = 0;
	
	if (pev->spawnflags == FIELD_START_ON) //dmarre allum
	{
		
		SetThink( InitHumSound );
		pev->nextthink = pev->ltime + 1;

		pev->skin = 1; //pour mettre en mmoire l'tat de fonctionnement
	}
	else
	{
		pev->solid = SOLID_NOT;
		pev->effects |= EF_NODRAW;
		UTIL_SetOrigin( pev, pev->origin );
		
		pev->skin = 0; //pour mettre en mmoire l'tat de fonctionnement
	}
}


void CTriggerField :: Touch( CBaseEntity *pOther )//Aiiieee!! a fait bobo!!
{

	TraceResult tr;//impacte
	Vector vecVelocity = -1 * (pOther->pev->velocity);//on inverse car le blaster a dj la nouvelle direction de ricochet
	Vector vecNormal ;//perpandiculaire  la surface
	Vector vecImpact ;
	
	float fl_vitesse = vecVelocity.Length();
	int m_quelson = 0;

	//ALERT( at_console, "IUSER3 %d\n RENDERAMT %f\n", pev->iuser3, m_renderamt);

	if ( fl_vitesse > 1000 && !pOther->IsPlayer())//c'est certainement un blaster
	{

		//ALERT(at_console, "VecVelocity %f %f %f\n", vecVelocity.x, vecVelocity.y, vecVelocity.z);
		
		UTIL_TraceLine( pOther->pev->origin, pOther->pev->origin + vecVelocity * 10, dont_ignore_monsters, ENT( pOther->pev ), &tr );

		vecNormal = tr.vecPlaneNormal;
		vecImpact = tr.vecEndPos;

		UTIL_Sparks( vecImpact );

		MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
			WRITE_BYTE(TE_DLIGHT);
			WRITE_COORD(vecImpact.x + (vecNormal.x * 10));	// X
			WRITE_COORD(vecImpact.y + (vecNormal.y * 10));	// Y
			WRITE_COORD(vecImpact.z + (vecNormal.z * 10));	// Z
			WRITE_BYTE( 20 );		// radius * 0.1
			WRITE_BYTE( 255 );		// r
			WRITE_BYTE( 35 );		// g
			WRITE_BYTE( 25 );		// b
			WRITE_BYTE( 1 );		// time * 10
			WRITE_BYTE( 10 );		// decay * 0.1
		MESSAGE_END( );

		m_quelson = 1;

		pev->frame = 0;
	}


	if ( gpGlobals->time > m_firstTouch + 0.5 )//on ne blesse que chaque demi-seconde
	{

		m_firstTouch = gpGlobals->time;

		pev->renderamt = pev->renderamt + 80;

		SetThink(ResetRender);
		pev->nextthink = pev->ltime + 0.2;

		if ( !pOther->pev->takedamage )
			return;

		if ( pev->spawnflags == FIELD_DAMAGE_CLIENT_ONLY && !pOther->IsPlayer())//si le champs ne doit pas blesser les monstres
			return;

		pOther->TakeDamage( pev, pev, pev->dmg, DMG_SHOCK );//on prend la valeur de dommage de dmg

		if ( !m_quelson )
		{
			EMIT_SOUND(ENT(pev), CHAN_VOICE, "items/field_touch.wav", 1, ATTN_NORM);
		}

	}

	demarre = 0; //pour liminer le bug si un objet touche le champs de force lors du dmarrage  cause de Think()
}

void CTriggerField :: ResetRender( )
{

	pev->renderamt = pev->renderamt - 80;

	SetThink(Animate);//animation du champs de force

	pev->nextthink = pev->ltime + 0.1;
}

void CTriggerField :: Animate( )
{

	float m_Variation = RANDOM_FLOAT( -5.0, 5.0 );

	if ( (pev->renderamt + m_Variation) >= (m_renderamt + 30) )// dtection de la valeur maximale du render
	{
		pev->renderamt = m_renderamt + 30;
	}
	else if ( (pev->renderamt + m_Variation) <= (m_renderamt - 30) )// dtection de la valeur minimale du render
	{
		pev->renderamt = m_renderamt - 30;
	}
	else
	{
		pev->renderamt = pev->renderamt + m_Variation;
	}

	pev->nextthink = pev->ltime + 0.1;
}


void CTriggerField :: Toggle( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{

	//ALERT(at_console, "TOGGLE SKIN %d Demarre %d\n", pev->skin, demarre);
	if ( pev->skin == 0 )//allumage
	{
		if ( demarre ) 
			return;

		PlayHumSound( 1 );
		pev->skin = 1; //pour mettre en mmoire l'tat de fonctionnement

		demarre = 1;
	}
	else//extinction
	{
		if ( demarre ) 
			return;
		
		PlayHumSound( 0 );
		pev->skin = 0; //pour mettre en mmoire l'tat de fonctionnement

		demarre = 1;
	}
}


void CTriggerField :: Precache( )
{
	PRECACHE_SOUND("items/forcefield.wav");
	PRECACHE_SOUND("items/field_on.wav");
	PRECACHE_SOUND("items/field_off.wav");
	PRECACHE_SOUND("items/field_touch.wav");

}

void CTriggerField :: InitHumSound( )
{

	//ALERT(at_console, "INITHUMSOUND");

	EMIT_SOUND(ENT(pev), CHAN_STATIC, "items/forcefield.wav", (pev->health) * 0.1 , ATTN_NORM);

	SetThink(Animate);//animation du champs de force

	pev->nextthink = pev->ltime + 0.1;

}

void CTriggerField :: PlayHumSound( int OnOff )
{

	if ( OnOff == 1 )
	{
		//ALERT(at_console, "Humsound start\n");

		//pev->health contient le volume du son de bourdonnement

		EMIT_SOUND(ENT(pev), CHAN_VOICE, "items/field_on.wav", (pev->health) * 0.1, ATTN_NORM);

		SetThink(AnimStart);

		pev->nextthink = pev->ltime + 0.2;
		
	}
	else
	{
		//ALERT(at_console, "Humsound stop\n");

		EMIT_SOUND(ENT(pev), CHAN_VOICE, "items/field_off.wav", (pev->health) * 0.1, ATTN_NORM);

		pev->renderamt = 70;

		SetThink(AnimClose);
		
		pev->nextthink = pev->ltime + 0.2;
	}

}


void CTriggerField ::AnimStart()//dmarrage progressif
{

	if ( animcount == 0 )
	{
		pev->effects &= ~EF_NODRAW;	
		pev->renderamt = 30;
		animcount++;
		SetThink(AnimStart);
		pev->nextthink = pev->ltime + 0.2;
	}
	else if ( animcount == 1 )
	{
		pev->renderamt = 10;
		pev->solid = SOLID_BSP;
		UTIL_SetOrigin( pev, pev->origin );
		animcount++;
		SetThink(AnimStart);
		pev->nextthink = pev->ltime + 0.1;
	}
	else if ( animcount == 2 )
	{
		pev->renderamt = 60;
		animcount++;
		SetThink(AnimStart);
		pev->nextthink = pev->ltime + 0.2;
	}
	else if ( animcount == 3 )
	{
		pev->renderamt = m_renderamt;
		EMIT_SOUND(ENT(pev), CHAN_STATIC, "items/forcefield.wav", (pev->health) * 0.1, ATTN_NORM);
		animcount = 0;
		SetThink(Animate);
		pev->nextthink = pev->ltime + 0.1;

		demarre = 0; //on peut  nouveau commuter le champs de force
	}

}

void CTriggerField ::AnimClose()
{

	if ( animcount == 0 )
	{
		pev->renderamt = 30;
		animcount++;
		SetThink(AnimClose);
		pev->nextthink = pev->ltime + 0.2;
	}

	else if ( animcount == 1 )
	{
		STOP_SOUND(ENT(pev), CHAN_STATIC, "items/forcefield.wav" );
		pev->renderamt = 80;
		animcount++;
		SetThink(AnimClose);
		pev->nextthink = pev->ltime + 0.1;
	}

	else if ( animcount == 2 )
	{
		pev->solid = SOLID_NOT;
		pev->renderamt = 15;
		
		animcount++;
		SetThink(AnimClose);
		pev->nextthink = pev->ltime + 0.2;
	}

	else if ( animcount == 3 )
	{
		
		pev->effects |= EF_NODRAW;
		UTIL_SetOrigin( pev, pev->origin );
		
		animcount = 0;
		
		demarre = 0; //on peut  nouveau commuter le champs de force
	}

}