/***
*
*	Code pour le dtonateur thermique
*	Par Jean-Franois Vuagniaux
*	vuagniaux.jf@com.mcnet.ch
*
****/
#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"


#define	HANDTHERMAL_PRIMARY_VOLUME		450

enum handthermal_e {
	HANDTHERMAL_IDLE = 0,
	HANDTHERMAL_FIDGET,
	HANDTHERMAL_SWITCH,
	HANDTHERMAL_THROW1,	// lancer
	HANDTHERMAL_THROW2,	// moyen
	HANDTHERMAL_THROW3,	// difficile
	HANDTHERMAL_HOLSTER,
	HANDTHERMAL_DRAW
};


class CHandThermal : public CBasePlayerWeapon
{
public:
	void Spawn( void );
	void Precache( void );
	int iItemSlot( void ) { return 5; }
	int GetItemInfo(ItemInfo *p);

	void PrimaryAttack( void );
	BOOL Deploy( void );
	BOOL CanHolster( void );
	void Holster( int skiplocal = 0 );
	void WeaponIdle( void );
	float m_flStartThrow;
	float m_flReleaseThrow;
};
LINK_ENTITY_TO_CLASS( weapon_handthermal, CHandThermal );


void CHandThermal::Spawn( )
{
	Precache( );
	m_iId = WEAPON_HANDTHERMAL;
	SET_MODEL(ENT(pev), "models/w_thermal.mdl");

	pev->dmg = gSkillData.plrDmgHandGrenade * 1.5;

	m_iDefaultAmmo = HANDTHERMAL_DEFAULT_GIVE;

	pev->iuser3 = 1;

	FallInit();// get ready to fall down.
}


void CHandThermal::Precache( void )
{
	PRECACHE_MODEL("models/w_thermal.mdl");
	PRECACHE_MODEL("models/v_thermal.mdl");
	PRECACHE_MODEL("models/p_thermal.mdl");
}

int CHandThermal::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "Hand Thermal";
	p->iMaxAmmo1 = HANDTHERMAL_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 4;
	p->iPosition = 0;
	p->iId = m_iId = WEAPON_HANDTHERMAL;
	p->iWeight = HANDTHERMAL_WEIGHT;
	p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;

	return 1;
}


BOOL CHandThermal::Deploy( )
{
	m_flReleaseThrow = -1;
	return DefaultDeploy( "models/v_thermal.mdl", "models/p_thermal.mdl", HANDTHERMAL_DRAW, "thermal" );
}

BOOL CHandThermal::CanHolster( void )
{
	// tu va tout de mme pas remettre un dtonateur thermique dgoupill dans ta poche!!
	return ( m_flStartThrow == 0 );
}

void CHandThermal::Holster( int skiplocal /* = 0 */ )
{
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;
	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		SendWeaponAnim( HANDTHERMAL_HOLSTER );
	}
	else
	{
		// plus de dtonateur thermique
		m_pPlayer->pev->weapons &= ~(1<<WEAPON_HANDTHERMAL);
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "common/null.wav", 1.0, ATTN_NORM);
}

void CHandThermal::PrimaryAttack()
{
	if (!m_flStartThrow && m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0)
	{
		m_flStartThrow = gpGlobals->time;
		m_flReleaseThrow = 0;

		SendWeaponAnim( HANDTHERMAL_SWITCH );
		m_flTimeWeaponIdle = gpGlobals->time + 0.5;
	}
}


void CHandThermal::WeaponIdle( void )
{
	if (m_flReleaseThrow == 0)
		m_flReleaseThrow = gpGlobals->time;

	if (m_flTimeWeaponIdle > gpGlobals->time)
		return;

	if (m_flStartThrow)
	{
		Vector angThrow = m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle;

		if (angThrow.x < 0)
			angThrow.x = -10 + angThrow.x * ((90 - 10) / 90.0);
		else
			angThrow.x = -10 + angThrow.x * ((90 + 10) / 90.0);

		float flVel = (90 - angThrow.x) * 4;
		if (flVel > 500)
			flVel = 500;

		UTIL_MakeVectors( angThrow );

		Vector vecSrc = m_pPlayer->pev->origin + m_pPlayer->pev->view_ofs + gpGlobals->v_forward * 16;

		Vector vecThrow = gpGlobals->v_forward * flVel + m_pPlayer->pev->velocity;

		// attention a va pter dans les 3 secondes!!!
		float time = m_flStartThrow - gpGlobals->time + 3.0;
		if (time < 0)
			time = 0;
		//tir temporis d'un dtonateur thermique
		CThermal::ShootTimed( m_pPlayer->pev, vecSrc, vecThrow, time );

		if (flVel < 500)
		{
			SendWeaponAnim( HANDTHERMAL_THROW1 );
		}
		else if (flVel < 1000)
		{
			SendWeaponAnim( HANDTHERMAL_THROW2 );
		}
		else
		{
			SendWeaponAnim( HANDTHERMAL_THROW3 );
		}

		// ziva tire p'tit gars!
		m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

		m_flStartThrow = 0;
		m_flNextPrimaryAttack = gpGlobals->time + 0.5;
		m_flTimeWeaponIdle = gpGlobals->time + 0.5;

		m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

		if ( !m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] )
		{
			// oh!! c'tait la dernire
			// pas si vite il faut qu'on voie bien le tir avant de recommencer 
			// weapon idle va retirer l'arme automatiquement s'il n'y en a plus.
			m_flTimeWeaponIdle = m_flNextSecondaryAttack = m_flNextPrimaryAttack = gpGlobals->time + 0.5;
		}
		return;
	}
	else if (m_flReleaseThrow > 0)
	{
		// a y est t'as fini, alors qu'est-ce que t'attend ?
		m_flStartThrow = 0;

		if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
		{
			SendWeaponAnim( HANDTHERMAL_DRAW );
		}
		else
		{
			RetireWeapon();
			return;
		}

		m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );
		m_flReleaseThrow = -1;
		return;
	}

	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		int iAnim;
		float flRand = RANDOM_FLOAT(0, 1);
		if (flRand <= 0.65)
		{
			iAnim = HANDTHERMAL_IDLE;
			m_flTimeWeaponIdle = gpGlobals->time + 90.0 / 30.0 ;//RANDOM_FLOAT ( 10, 15 );// on ne se fatigue pas trop
		}
		else 
		{
			iAnim = HANDTHERMAL_FIDGET;
			m_flTimeWeaponIdle = gpGlobals->time + 90.0 / 30.0;
		}

		SendWeaponAnim( iAnim );
	}
}




