/***
*
*	Code pour le dragon Kell par Jean-Franois Vuagniaux
*   vuagniaux.jf@com.mcnet.ch
*
****/
//=========================================================
// Dragon Kell, type de dinosaure quadripode
//=========================================================

#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"schedule.h"
#include	"squadmonster.h"
#include	"weapons.h"
#include	"soundent.h"
#include	"animation.h"


//=========================================================
// tasks (tches) spcifiques 
//=========================================================
enum 
{
	TASK_KELL_GET_PATH_TO_ENEMY_CORPSE = LAST_COMMON_TASK + 1,
};

//=========================================================
// types de schedule spcifiques
//=========================================================
enum
{
	SCHED_KELL_RECHERCHE = LAST_COMMON_SCHEDULE + 1,
};


//=========================================================
// Evnements d'animations de l'ATST
//=========================================================

#define		KELL_AE_MORD	( 1 )
#define		KELL_AE_MARCHE_G  ( 2 )
#define		KELL_AE_MARCHE_D ( 3 )
#define		KELL_AE_COURSE_G  ( 4 )
#define		KELL_AE_COURSE_D ( 5 )

#define		KELL_MELEE_DIST	180



class CKell : public CSquadMonster
{
public:
	void Spawn( void );
	void Precache( void );
	void SetYawSpeed ( void );
	int  Classify ( void );
	int  ISoundMask ( void );
	void HandleAnimEvent( MonsterEvent_t *pEvent );
	void SetObjectCollisionBox( void )
	{
		pev->absmin = pev->origin + Vector( -80, -80, -8 );
		pev->absmax = pev->origin + Vector( 80, 80, 75 );
	}
	void Killed( entvars_t *pevAttacker, int iGib );
	Schedule_t* GetSchedule ( void );
	Schedule_t* GetScheduleOfType ( int Type );
	BOOL FCanCheckAttacks ( void );
	BOOL CheckMeleeAttack1 ( float flDot, float flDist );
	BOOL CheckRangeAttack1 ( float flDot, float flDist );
	void StartTask ( Task_t *pTask );
	void DeathSound ( void );
	void StopTalking ( void );
	BOOL ShouldSpeak( void );
	void AttackSound ( void );
	void AlertSound ( void );
	void PainSound ( void );
	void PrescheduleThink ( void );
	void TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType);
	int TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType );
	int IRelationship( CBaseEntity *pTarget );
	void SetActivity ( Activity NewActivity );

	CUSTOM_SCHEDULES;

	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	static const char *pAttackHitSounds[];
	static const char *pAttackMissSounds[];
	static const char *pAttackSounds[];
	static const char *pDieSounds[];
	static const char *pIdleSounds[];
	static const char *pPainSounds[];
	static const char *pAlertSounds[];
	
	float   m_flNextPainTime;
	float	m_flNextSpeakTime;
	float	m_flNextWordTime;
	int		m_iLastWord;
	int		num;

	BOOL	m_fCanAttack;
	float	m_flNextAttackCheck;
	float   m_flNextAttack;
	
};
LINK_ENTITY_TO_CLASS( monster_kell, CKell );

TYPEDESCRIPTION	CKell::m_SaveData[] = 
{
	DEFINE_FIELD( CKell, m_fCanAttack, FIELD_BOOLEAN ),
	DEFINE_FIELD( CKell, m_flNextAttackCheck, FIELD_TIME ),
	DEFINE_FIELD( CKell, m_flNextSpeakTime, FIELD_TIME ),
	DEFINE_FIELD( CKell, m_flNextWordTime, FIELD_TIME ),
	DEFINE_FIELD( CKell, m_iLastWord, FIELD_INTEGER ),
};

IMPLEMENT_SAVERESTORE( CKell, CSquadMonster );

const char *CKell::pAttackHitSounds[] = 
{
	"kell/bite1.wav",
	"kell/bite2.wav",
	
};

const char *CKell::pAttackMissSounds[] = 
{
	"kell/miss1.wav",
	"kell/miss2.wav",
};

const char *CKell::pAttackSounds[] =
{
	"kell/attack1.wav",
	"kell/attack2.wav",
	
};

const char *CKell::pDieSounds[] =
{
	"kell/die1.wav",
	"kell/die2.wav",
	
};

const char *CKell::pIdleSounds[] =
{
	"kell/idle1.wav",
	"kell/idle2.wav",
	"kell/idle3.wav",
	
};

const char *CKell::pPainSounds[] =
{
	"kell/pain1.wav",
	"kell/pain2.wav",
	
};

const char *CKell::pAlertSounds[] =
{
	"kell/attack1.wav",
	"kell/attack2.wav",
	
};
//=========================================================
// IRelationship - l'ennemi principal est le stromtrooper
//=========================================================
int CKell::IRelationship ( CBaseEntity *pTarget )
{
	if ( FClassnameIs( pTarget->pev, "monster_stormtrooper" ) )
	{
		return R_NM;
	}

	return CSquadMonster :: IRelationship( pTarget );
}

//=========================================================
// ISoundMask 
//=========================================================
int CKell :: ISoundMask ( void )
{
	return	bits_SOUND_WORLD	|
			bits_SOUND_PLAYER	|
			bits_SOUND_COMBAT	;
			
}


void CKell::Killed( entvars_t *pevAttacker, int iGib )
{
	
	CBaseMonster::Killed( pevAttacker, GIB_NEVER );

	STOP_SOUND(ENT(pev), CHAN_VOICE, pIdleSounds[ num ]);
	
	

}

//=========================================================
// TraceAttack
//=========================================================
void CKell :: TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType)
{
	
		m_LastHitGroup = ptr->iHitgroup;

		switch ( ptr->iHitgroup )
		{
		case HITGROUP_GENERIC:
			break;
		case HITGROUP_HEAD:
			flDamage *= gSkillData.monHead;
			break;
		case HITGROUP_CHEST:
			flDamage *= gSkillData.monChest;
			break;
		case HITGROUP_STOMACH:
			flDamage *= gSkillData.monStomach;
			break;
		case HITGROUP_LEFTARM:
		case HITGROUP_RIGHTARM:
			flDamage *= gSkillData.monArm;
			break;
		case HITGROUP_LEFTLEG:
		case HITGROUP_RIGHTLEG:
			flDamage *= gSkillData.monLeg;
			break;
		default:
			break;
		}



	// Le dragon Kell ne reoit que trs peu de dommage de la part d'armes lgres comme le pied de biche
	// ou une balle ou un coup. Il provoque donc facilement des ricochets.
    if  (bitsDamageType & (DMG_ENERGYBEAM | DMG_MORTAR | DMG_BLAST))
	{
		
	}

	if  (bitsDamageType & (DMG_GENERIC | DMG_NEVERGIB | DMG_CRUSH))
	{
		// hit armor
		flDamage -= 30; // diminue le dommage de 30 units
		if (flDamage <= 0)
			flDamage = 0.1;// ne blesse que trs lgrement mais permet que bits_COND_LIGHT_DAMAGE soit gnr.

	}

	if  (bitsDamageType & (DMG_BULLET | DMG_SLASH | DMG_CLUB))
	{
		// hit armor
		//ALERT ( at_console, "faible dommages\n" ) ;

		if ( pev->dmgtime != gpGlobals->time || (RANDOM_LONG(0,2) < 1) )
		{
			UTIL_Ricochet( ptr->vecEndPos, RANDOM_FLOAT( 1, 2) );
			pev->dmgtime = gpGlobals->time;
		}

		if ( RANDOM_LONG( 0, 1 ) == 0 )
		{
			Vector vecTracerDir = vecDir;

			vecTracerDir.x += RANDOM_FLOAT( -0.3, 0.3 );
			vecTracerDir.y += RANDOM_FLOAT( -0.3, 0.3 );
			vecTracerDir.z += RANDOM_FLOAT( -0.3, 0.3 );

			vecTracerDir = vecTracerDir * -512;

			MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, ptr->vecEndPos );
			WRITE_BYTE( TE_TRACER );
				WRITE_COORD( ptr->vecEndPos.x );
				WRITE_COORD( ptr->vecEndPos.y );
				WRITE_COORD( ptr->vecEndPos.z );

				WRITE_COORD( vecTracerDir.x );
				WRITE_COORD( vecTracerDir.y );
				WRITE_COORD( vecTracerDir.z );
			MESSAGE_END();
		}

		flDamage -= 30; // diminue le dommage de 30 units
		if (flDamage <= 0)
			flDamage = 0.1;// ne blesse que trs lgrement mais permet que bits_COND_LIGHT_DAMAGE soit gnr.

		
	}
		
	AddMultiDamage( pevAttacker, this, flDamage, bitsDamageType );
	
}

int CKell :: TakeDamage(entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType)
{
	
	// Le dragon Kell ne reoit que trs peu de dommage de la part d'armes lgres comme le pied de biche
	// ou une balle ou un coup. Il provoque donc facilement des ricochets.
    if  (bitsDamageType & (DMG_ENERGYBEAM | DMG_MORTAR | DMG_BLAST))
	{
		//ALERT ( at_console, "graves dommages\n" ) ;
		
		if (bitsDamageType & DMG_ENERGYBEAM)
		{
			flDamage -= 9;
			if (flDamage <= 0)
				flDamage = 0.1;
		}

		return CSquadMonster :: TakeDamage ( pevInflictor, pevAttacker, flDamage, bitsDamageType );
	}

	if  (bitsDamageType & (DMG_GENERIC | DMG_NEVERGIB | DMG_CRUSH))
	{
		// hit armor
		flDamage -= 30; // diminue le dommage de 30 units
		if (flDamage <= 0)
			flDamage = 0.1;// ne blesse que trs lgrement mais permet que bits_COND_LIGHT_DAMAGE soit gnr.

		return CSquadMonster :: TakeDamage ( pevInflictor, pevAttacker, flDamage, bitsDamageType );
	}

	else
	{
		
		// hit armor
		flDamage -= 30; // diminue le dommage de 30 units
		if (flDamage <= 0)
			flDamage = 0.1;// ne blesse que trs lgrement mais permet que bits_COND_LIGHT_DAMAGE soit gnr.

		return CSquadMonster :: TakeDamage ( pevInflictor, pevAttacker, flDamage, bitsDamageType );

	}


}


//=========================================================
// PrescheduleThink 
//=========================================================
void CKell :: PrescheduleThink ( void )
{
	if ( InSquad() && m_hEnemy != NULL )
	{
		if ( HasConditions ( bits_COND_SEE_ENEMY ) )
		{
			// update the squad's last enemy sighting time.
			MySquadLeader()->m_flLastEnemySightTime = gpGlobals->time;
		}
		else
		{
			if ( gpGlobals->time - MySquadLeader()->m_flLastEnemySightTime > 5 )
			{
				// been a while since we've seen the enemy
				MySquadLeader()->m_fEnemyEluded = TRUE;
			}
		}
	}

	if ( ShouldSpeak() )
	{
		if ( m_flNextWordTime < gpGlobals->time )
		{
			num = -1;

			do
			{
				num = RANDOM_LONG(0,ARRAYSIZE(pIdleSounds)-1);
			} while( num == m_iLastWord );

			m_iLastWord = num;

			// play a new sound
			EMIT_SOUND ( ENT(pev), CHAN_VOICE, pIdleSounds[ num ], 1.0, ATTN_NORM );

			// is this word our last?
			if ( RANDOM_LONG( 1, 10 ) <= 4 )
			{
				// stop talking.
				StopTalking();
			}
			else
			{
				m_flNextWordTime = gpGlobals->time + RANDOM_FLOAT( 3.5, 6 );
			}
		}
	}
}

//=========================================================
// AlertSound
//=========================================================
void CKell :: AlertSound ( void )
{
	StopTalking();

	EMIT_SOUND ( ENT(pev), CHAN_VOICE, pAlertSounds[RANDOM_LONG(0,ARRAYSIZE(pAlertSounds)-1)], 1.0, ATTN_NORM );
}

//=========================================================
// PainSound
//=========================================================
void CKell :: PainSound ( void )
{
	if ( m_flNextPainTime > gpGlobals->time )
	{
		return;
	}

	m_flNextPainTime = gpGlobals->time + 0.6;

	StopTalking();

	EMIT_SOUND ( ENT(pev), CHAN_VOICE, pPainSounds[RANDOM_LONG(0,ARRAYSIZE(pPainSounds)-1)], 1.0, ATTN_NORM );
}

//=========================================================
// Sons de mort
//=========================================================
void CKell :: DeathSound ( void )
{

	StopTalking();

	EMIT_SOUND ( ENT(pev), CHAN_VOICE, pDieSounds[RANDOM_LONG(0,ARRAYSIZE(pDieSounds)-1)], 1.0, ATTN_NORM );
}


//=========================================================
// StopTalking - pas de sons pendant 10-20 secondes.
//=========================================================
void CKell::StopTalking( void )
{
	m_flNextWordTime = m_flNextSpeakTime = gpGlobals->time + 10 + RANDOM_LONG(0, 10);
}

//=========================================================
// ShouldSpeak - Est-ce le moment de jouer un son
//=========================================================
BOOL CKell::ShouldSpeak( void )
{
	if ( m_flNextSpeakTime > gpGlobals->time )
	{
		// pas pour le moment.
		return FALSE;
	}

	if ( pev->spawnflags & SF_MONSTER_GAG )
	{
		if ( m_MonsterState != MONSTERSTATE_COMBAT )
		{
			
			m_flNextSpeakTime = gpGlobals->time + 6;
			return FALSE;
		}
	}

	return TRUE;
}


//=========================================================
// Sons d'attaque
//=========================================================
void CKell :: AttackSound ( void )
{

	EMIT_SOUND ( ENT(pev), CHAN_VOICE, pAttackSounds[RANDOM_LONG(0,ARRAYSIZE(pAttackSounds)-1)], 1.0, ATTN_NORM );
}

//=========================================================
// Classify - place du monstre dans la table des relations 
//=========================================================
int	CKell :: Classify ( void )
{
	return	CLASS_ALIEN_MONSTER;
}

//=========================================================
// SetYawSpeed - permet  chaque squence d'avoir un taux de lacet diffrent
//=========================================================
void CKell :: SetYawSpeed ( void )
{
	int ys;

	switch ( m_Activity )
	{
	case ACT_TURN_LEFT:
	case ACT_TURN_RIGHT:
		ys = 100;
		break;
	default:			ys = 90;
	}

	pev->yaw_speed = ys;
}


//=========================================================
// HandleAnimEvent - s'occupe des messages spcifiques qui apparaissent dans les animations
//
// Retourne le nombre d'vnements traits, 0 si aucun.
//=========================================================
void CKell :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
	switch( pEvent->event )
	{
	

	case KELL_AE_MARCHE_G:
		switch (RANDOM_LONG(0,1))
		{
		// pied gauche
		case 0:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/walk1.wav", 1, 0.3, 0, 100 );	break;
		case 1:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/walk2.wav", 1, 0.3, 0, 100 );	break;
		}
		break;
	
	case KELL_AE_MARCHE_D:
		// pied droit
		switch (RANDOM_LONG(0,1))
		{
		case 0:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/walk1.wav", 1, 0.3, 0, 100 );	break;
		case 1:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/walk2.wav", 1, 0.3, 0 ,100);		break;
		}
		break;

	case KELL_AE_COURSE_G:
		
		// secoue le dcor en courant
		UTIL_ScreenShake( pev->origin, 2.5, 3.0, 0.8, 400 );

		switch (RANDOM_LONG(0,1))
		{
		// pied gauche
		case 0:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/run1.wav", 1, 0.3, 0, 100 );	break;
		case 1:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/run2.wav", 1, 0.3, 0, 100 );	break;
				
		}
		break;
	
	case KELL_AE_COURSE_D:
		
		// pied droit
		// secoue le dcor en courant
		UTIL_ScreenShake( pev->origin, 2.5, 3.0, 0.8, 400 );

		switch (RANDOM_LONG(0,1))
		{
		case 0:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/run1.wav", 1, 0.3, 0, 100 );	break;
		case 1:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "kell/run2.wav", 1, 0.3, 0 ,100);		break;
		
		}
		break;

	case KELL_AE_MORD:
		{
			
				CBaseEntity *pHurt = CheckTraceHullAttack( KELL_MELEE_DIST, gSkillData.kellDmgBite, DMG_CRUSH );
			
			if ( pHurt )//touch
			{
				pHurt->pev->punchangle.y = -25;
				pHurt->pev->punchangle.x = -40;
				pHurt->pev->velocity = pHurt->pev->velocity + gpGlobals->v_forward * 90 + gpGlobals->v_up * 200;


				EMIT_SOUND_DYN ( ENT(pev), CHAN_WEAPON, pAttackHitSounds[ RANDOM_LONG(0,ARRAYSIZE(pAttackHitSounds)-1) ], 1.0, ATTN_NORM, 0, 100 + RANDOM_LONG(-5,5) );

				Vector vecArmPos, vecArmAng;
				GetAttachment( 0, vecArmPos, vecArmAng ); //attachement 0 vers la gueule du dragon
				SpawnBlood(vecArmPos, pHurt->BloodColor(), 100);// une gicle de sang
			}
			else
			{
				// c'est rat
				EMIT_SOUND_DYN ( ENT(pev), CHAN_WEAPON, pAttackMissSounds[ RANDOM_LONG(0,ARRAYSIZE(pAttackMissSounds)-1) ], 1.0, ATTN_NORM, 0, 100 + RANDOM_LONG(-5,5) );
			}

		}
		break;


	default:
		CSquadMonster::HandleAnimEvent( pEvent );
		break;
	}
}

//=========================================================
// Spawn
//=========================================================
void CKell :: Spawn()
{
	Precache( );

	SET_MODEL(ENT(pev), "models/kell.mdl");
	UTIL_SetSize(pev, Vector(-80, -80, -8), Vector(80, 80, 75));

	pev->solid			= SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_STEP;
	m_bloodColor		= DONT_BLEED;
	pev->effects		= 0;
	pev->health			= gSkillData.kellHealth;
	m_flFieldOfView		= 0.2;// champs de vision du dragon
	m_MonsterState		= MONSTERSTATE_NONE;
	m_afCapability		= 0;
	m_afCapability		|= bits_CAP_SQUAD;
	pev->iuser3			= 80;//masse
	pev->iuser4			= 0;


	MonsterInit();
}

//=========================================================
// Precache - precache toutes les ressources
//=========================================================
void CKell :: Precache()
{
	int i;

	PRECACHE_MODEL("models/kell.mdl");

	for ( i = 0; i < ARRAYSIZE( pAttackHitSounds ); i++ )
		PRECACHE_SOUND((char *)pAttackHitSounds[i]);

	for ( i = 0; i < ARRAYSIZE( pAttackMissSounds ); i++ )
		PRECACHE_SOUND((char *)pAttackMissSounds[i]);

	for ( i = 0; i < ARRAYSIZE( pDieSounds ); i++ )
		PRECACHE_SOUND((char *)pDieSounds[i]);

	for ( i = 0; i < ARRAYSIZE( pAttackSounds ); i++ )
		PRECACHE_SOUND((char *)pAttackSounds[i]);

	for ( i = 0; i < ARRAYSIZE( pIdleSounds ); i++ )
		PRECACHE_SOUND((char *)pIdleSounds[i]);

	for ( i = 0; i < ARRAYSIZE( pPainSounds ); i++ )
		PRECACHE_SOUND((char *)pPainSounds[i]);

	for ( i = 0; i < ARRAYSIZE( pAlertSounds ); i++ )
		PRECACHE_SOUND((char *)pAlertSounds[i]);


	// son de la marche
	PRECACHE_SOUND( "kell/walk1.wav" );
	PRECACHE_SOUND( "kell/walk2.wav" );
	PRECACHE_SOUND( "kell/run1.wav" );
	PRECACHE_SOUND( "kell/run2.wav" );
}	
	
//=========================================================
// AI Schedules particuliers au monstre
//=========================================================

//=========================================================
// Fail Schedule (chec)
//=========================================================
Task_t	tlKellFail[] =
{
	{ TASK_STOP_MOVING,			0				},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE },
	{ TASK_WAIT,				(float)2		},
	{ TASK_WAIT_PVS,			(float)0		},
};

Schedule_t	slKellFail[] =
{
	{
		tlKellFail,
		ARRAYSIZE ( tlKellFail ),
		bits_COND_CAN_MELEE_ATTACK1,
		0,
		"Kell Fail"
	},
};

//=========================================================
// Combat Fail Schedule ( chec au combat )
//=========================================================
Task_t	tlKellCombatFail[] =
{
	{ TASK_STOP_MOVING,			0				},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE_ANGRY },
	{ TASK_WAIT_FACE_ENEMY,		(float)2		},
	{ TASK_WAIT_PVS,			(float)0		},
};

Schedule_t	slKellCombatFail[] =
{
	{
		tlKellCombatFail,
		ARRAYSIZE ( tlKellCombatFail ),
		bits_COND_CAN_MELEE_ATTACK1,
		0,
		"Kell Combat Fail"
	},
};

//=========================================================
// Standoff schedule. Utilis en combat lorsque le monstre a perdu de vue l'ennemi.
// On recherche l'ennemi?
//=========================================================
Task_t	tlKellStandoff[] =
{
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_SET_ACTIVITY,			(float)ACT_IDLE				},
	{ TASK_WAIT_FACE_ENEMY,			(float)2					},
};

Schedule_t slKellStandoff[] = 
{
	{
		tlKellStandoff,
		ARRAYSIZE ( tlKellStandoff ),
		bits_COND_CAN_MELEE_ATTACK1		|
		bits_COND_SEE_ENEMY				|
		bits_COND_NEW_ENEMY				|
		bits_COND_HEAR_SOUND,
		
		0,
		"Kell Standoff"
	}
};


//=========================================================
// Se met  couvert de l'ennemi sur le ct avant de rechercher un noeud  couvert. 
//=========================================================
Task_t	tlKellTakeCoverFromEnemy[] =
{
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_WAIT,					(float)0.2					},
	{ TASK_FIND_COVER_FROM_ENEMY,	(float)0					},
	{ TASK_RUN_PATH,				(float)0					},
	{ TASK_WAIT_FOR_MOVEMENT,		(float)0					},
	{ TASK_REMEMBER,				(float)bits_MEMORY_INCOVER	},
	{ TASK_FACE_ENEMY,				(float)0					},
};

Schedule_t	slKellTakeCoverFromEnemy[] =
{
	{ 
		tlKellTakeCoverFromEnemy,
		ARRAYSIZE ( tlKellTakeCoverFromEnemy ), 
		bits_COND_NEW_ENEMY,
		0,
		"KellTakeCoverFromEnemy"
	},
};

//=========================================================
// Youpiiieee ! je t'ai eu !, tu l'as dans l'cul.
//=========================================================
Task_t	tlKellVictoryDance[] =
{
	{ TASK_STOP_MOVING,						(float)0					},
	{ TASK_WAIT,							(float)0.2					},
	{ TASK_KELL_GET_PATH_TO_ENEMY_CORPSE,	(float)0					},
	{ TASK_WALK_PATH,						(float)0					},
	{ TASK_WAIT_FOR_MOVEMENT,				(float)0					},
	{ TASK_FACE_ENEMY,						(float)0					},
	{ TASK_PLAY_SEQUENCE,					(float)ACT_VICTORY_DANCE				},
	{ TASK_PLAY_SEQUENCE,					(float)ACT_IDLE_ANGRY		},
	{ TASK_PLAY_SEQUENCE,					(float)ACT_STAND			},

};

Schedule_t	slKellVictoryDance[] =
{
	{ 
		tlKellVictoryDance,
		ARRAYSIZE ( tlKellVictoryDance ), 
		bits_COND_NEW_ENEMY		|
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE,
		0,
		"KellVictoryDance"
	},
};


Task_t tlKellRecherche[] =
{
	{ TASK_GET_PATH_TO_SPOT,		(float)0		},
	{ TASK_RUN_PATH,				(float)0		},
	{ TASK_WAIT_FOR_MOVEMENT,		(float)0		},
};

Schedule_t slKellRecherche[] =
{
	{
		tlKellRecherche,
		ARRAYSIZE ( tlKellRecherche ),
		bits_COND_NEW_ENEMY			|
		bits_COND_ENEMY_DEAD		|
		bits_COND_CAN_RANGE_ATTACK1	|
		bits_COND_CAN_MELEE_ATTACK1	|
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER,
		"KellRecherche"
	},
};



//=========================================================
//=========================================================

DEFINE_CUSTOM_SCHEDULES( CKell )
{
	slKellFail,
	slKellCombatFail,
	slKellStandoff,
	slKellTakeCoverFromEnemy,
	slKellVictoryDance,
	slKellRecherche,
};

IMPLEMENT_CUSTOM_SCHEDULES( CKell, CSquadMonster );

//=========================================================
// FCanCheckAttacks vrifie si l'attaque est possible
//=========================================================
BOOL CKell :: FCanCheckAttacks ( void )
{
	if ( !HasConditions( bits_COND_ENEMY_TOOFAR ) )
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

//=========================================================
// CheckMeleeAttack1 vrifie si l'attaque de mle est possible
//=========================================================
BOOL CKell :: CheckMeleeAttack1 ( float flDot, float flDist )
{
	
	if ( gpGlobals->time < m_flNextAttack )
	{
		return FALSE;
	}
	
	if ( gpGlobals->time < m_flNextAttackCheck )
	{
		return m_fCanAttack;
	}
	
	if ( HasConditions ( bits_COND_SEE_ENEMY ) && flDist <= KELL_MELEE_DIST && flDot >= 0.6 && m_hEnemy != NULL )
	{
		//ALERT ( at_console, "c'est la melee\n" ) ;
		m_flNextAttackCheck = gpGlobals->time + RANDOM_FLOAT( 1, 2 );
		m_fCanAttack = TRUE;
		m_flNextAttack = gpGlobals->time + 2;
		return m_fCanAttack;

	}
	m_flNextAttackCheck = gpGlobals->time + 0.1;// don't check for half second if this check wasn't successful
	m_fCanAttack = FALSE;
	return m_fCanAttack;
}

//=========================================================
// CheckRangeAttack1 vrifie si l'attaque  distance est possible
//=========================================================
BOOL CKell :: CheckRangeAttack1 ( float flDot, float flDist )
{
	

	return FALSE;
}
	

//==========================================================
// Le dragon Kell utilise l'animation pour tourner, mme lgrement
//==========================================================
void CBaseMonster :: KellTurn ( void )
{
	
	float flYD;
	flYD = FlYawDiff();

	if ( LookupActivity ( ACT_TURN_RIGHT ) != ACTIVITY_NOT_AVAILABLE )
	{// big right turn
		m_IdealActivity = ACT_TURN_RIGHT;
	}
	else if ( LookupActivity ( ACT_TURN_LEFT ) != ACTIVITY_NOT_AVAILABLE )
	{// big left turn
		m_IdealActivity = ACT_TURN_LEFT;
	}
}
//=========================================================
// StartTask
//=========================================================
void CKell :: StartTask ( Task_t *pTask )
{
	switch ( pTask->iTask )
	{
	case TASK_KELL_GET_PATH_TO_ENEMY_CORPSE:
		{
			UTIL_MakeVectors( pev->angles );
			if ( BuildRoute ( m_vecEnemyLKP - gpGlobals->v_forward * 50, bits_MF_TO_LOCATION, NULL ) )
			{
				TaskComplete();
			}
			else
			{
				ALERT ( at_aiconsole, "AtstGetPathToEnemyCorpse failed!!\n" );
				TaskFail();
			}
		}
		break;
	
	case TASK_TURN_RIGHT:
		{
		float flCurrentYaw;
			
			flCurrentYaw = UTIL_AngleMod( pev->angles.y );
			pev->ideal_yaw = UTIL_AngleMod( flCurrentYaw - pTask->flData );
			AtstTurn() ;

			break;
		
		}
	
	case TASK_TURN_LEFT:
		{
		
		float flCurrentYaw;
			
			flCurrentYaw = UTIL_AngleMod( pev->angles.y );
			pev->ideal_yaw = UTIL_AngleMod( flCurrentYaw + pTask->flData );
			AtstTurn() ;

			break;
		}
		
	
	default:
		CSquadMonster :: StartTask ( pTask );
		break;

	}

}

// ACTIVITY

void CKell :: SetActivity ( Activity NewActivity )
{
	int	iSequence = ACTIVITY_NOT_AVAILABLE;
	void *pmodel = GET_MODEL_PTR( ENT(pev) );
	
	switch ( NewActivity)
	{
	case ACT_IDLE:
		{
			iSequence = LookupActivity ( ACT_IDLE );
		}

	default:
		iSequence = LookupActivity ( NewActivity );
		break;

	}
	
	m_Activity = NewActivity; 

	
	if ( iSequence > ACTIVITY_NOT_AVAILABLE )
	{
		if ( pev->sequence != iSequence || !m_fSequenceLoops )
		{
			pev->frame = 0;
		}

		pev->sequence		= iSequence;	
		ResetSequenceInfo( );
		SetYawSpeed();
	}
	else
	{
		
		ALERT ( at_console, "%s has no sequence for act:%d\n", STRING(pev->classname), NewActivity );
		pev->sequence		= 0;	
	}
}
//=========================================================
// GetSchedule - Decides which type of schedule best suits
// the monster's current state and conditions. Then calls
// monster's member function to get a pointer to a schedule
// of the proper type.
//=========================================================
Schedule_t *CKell :: GetSchedule ( void )
{
	if ( HasConditions(bits_COND_HEAR_SOUND) )
	{
		CSound *pSound;
		pSound = PBestSound();

		ASSERT( pSound != NULL );
		if ( pSound && (pSound->m_iType & bits_SOUND_DANGER) )
		{
			// dangerous sound nearby!
			return GetScheduleOfType( SCHED_TAKE_COVER_FROM_BEST_SOUND );
		}
	}

	switch	( m_MonsterState )
	{

	case MONSTERSTATE_IDLE:
	case MONSTERSTATE_ALERT:
		{

			if ( HasConditions ( bits_COND_HEAVY_DAMAGE ) || HasConditions ( bits_COND_LIGHT_DAMAGE ) )
			{

				return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN  );
			}


		}
		break;

	case MONSTERSTATE_COMBAT:
		{
	// dead enemy
			if ( HasConditions( bits_COND_ENEMY_DEAD ) )
			{
				// call base class, all code to handle dead enemies is centralized there.
				return CBaseMonster :: GetSchedule();
			}

			if ( HasConditions(bits_COND_NEW_ENEMY) )
			{
				return GetScheduleOfType( SCHED_WAKE_ANGRY );
			}

	// zap player!
			if ( HasConditions ( bits_COND_CAN_MELEE_ATTACK1 ) )
			{
				AttackSound();// this is a total hack. Should be part of the schedule
				return GetScheduleOfType ( SCHED_MELEE_ATTACK1 );
			}

			if ( HasConditions( bits_COND_SEE_ENEMY ) && !HasConditions ( bits_COND_CAN_MELEE_ATTACK1 ) )
			{
				return GetScheduleOfType ( SCHED_CHASE_ENEMY );
			}


			if ( OccupySlot ( bits_SLOT_ATST_CHASE ) )
			{
				return GetScheduleOfType ( SCHED_CHASE_ENEMY );
			}

			return GetScheduleOfType ( SCHED_STANDOFF );
		}
	}

	return CSquadMonster :: GetSchedule();
}

//=========================================================
//=========================================================
Schedule_t* CKell :: GetScheduleOfType ( int Type ) 
{
	switch	( Type )
	{
	case SCHED_TAKE_COVER_FROM_ENEMY:
		return &slKellTakeCoverFromEnemy[ 0 ];
		break;

	case SCHED_STANDOFF:
		return &slKellStandoff[ 0 ];
		break;
	
	case SCHED_KELL_RECHERCHE:
		return &slKellRecherche[ 0 ];
		break;

	case SCHED_VICTORY_DANCE:
		return &slKellVictoryDance[ 0 ];
		break;

	case SCHED_FAIL:
		// no fail schedule specified, so pick a good generic one.
		{
			if ( m_hEnemy != NULL )
			{
				// I have an enemy
				// !!!LATER - what if this enemy is really far away and i'm chasing him?
				// this schedule will make me stop, face his last known position for 2 
				// seconds, and then try to move again
				return &slKellCombatFail[ 0 ];
			}

			return &slKellFail[ 0 ];
		}
		break;

	}

	return CSquadMonster :: GetScheduleOfType( Type );
}

