

//==============================================================
//
// Ascenseur  plusieurs niveaux command par boutons.
// Pour rsoudre les problmes pineux engendrs par les ascenseur complexes dans HL
// En effet crer un ascenseur  trois niveaux avec les fonctions standards du jeu est un casse-tte
// Avec cette nouvelle func_lift, il suffit de crer un ascenseur dans sa position de repos, de placer 
// trois trigger_relay comme points de rfrence pour les arrts 
// et d'ensuite placer deux boutons en bas et en haut et trois au millieu.
// Chaque tage a un bouton d'appel qui a comme target le trigger_relay de son tage
// Chaque tage d'extrmit a un bouton de commande qui a comme target le trigger_relay de l'tage suivant.
// Chaque tage intermdiaire a deux boutons de commande qui ont comme target les trigger_relay des deux tages proches.
// Le nombre d'tages n'est pas limit, j'ai pris comme exemple trois niveaux, mais on peut en crer beaucoup plus.
// 
// Par Jean-Franois Vuagniaux 
//
// vuagniaux.jf@bluewin.ch
//
//==============================================================

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "saverestore.h"

#define LIFT_RETURN_START 2 //l'ascenseur retourne au point de dpart aprs l'arrt


class CLift : public CBaseToggle
{

public:
	virtual int	ObjectCaps( void ) { return CBaseEntity :: ObjectCaps() & ~FCAP_ACROSS_TRANSITION; }
	void KeyValue( KeyValueData* pkvd);
	void Precache( void );
	void EXPORT Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );

	void Spawn(void);

	void EXPORT LaunchLift(void);
	void EXPORT ReturnHome(void);
	void StartMove(void);
	void StopMove(void);
	void EXPORT CallStopMove( void  ) { StopMove(); }
	virtual void Blocked( CBaseEntity *pOther );

	virtual int Save(CSave &save);
	virtual int Restore(CRestore &restore);
	static	TYPEDESCRIPTION m_SaveData[];

	int m_startsound;
	int m_movesound;
	int m_stopsound;
	int m_locked;


	float m_volume;//volume du son
	float movespeed;//vitesse de dplacement
	
	Vector vecStartPosition;
	Vector vecNewPosition;
	Vector vecBloqueRet;
	Vector vecOffset;
	Vector vecOrigine;//l'origine n'est pas sauvegarde!!!
	
	BOOL Moving;
	int Bloque;
	BOOL MoveSound;
	BOOL saveOrigin;//si on sauvegarde et recharge la partie avant d'avoir utilis l'ascenseur, il va partir vers les coordonnes 0 0 0
					
};

LINK_ENTITY_TO_CLASS( func_lift, CLift);

TYPEDESCRIPTION	CLift::m_SaveData[] = 
{
	DEFINE_FIELD( CLift, m_volume, FIELD_FLOAT ),
	DEFINE_FIELD( CLift, movespeed, FIELD_FLOAT ),
	DEFINE_FIELD( CLift, vecNewPosition, FIELD_VECTOR ),
	DEFINE_FIELD( CLift, vecOffset, FIELD_VECTOR ),
	DEFINE_FIELD( CLift, vecBloqueRet, FIELD_VECTOR ),
	DEFINE_FIELD( CLift, vecStartPosition, FIELD_VECTOR ),
	DEFINE_FIELD( CLift, Moving, FIELD_BOOLEAN ),
	DEFINE_FIELD( CLift, Bloque, FIELD_INTEGER ),
	DEFINE_FIELD( CLift, MoveSound, FIELD_BOOLEAN ),
	DEFINE_FIELD( CLift, vecOrigine, FIELD_VECTOR ),
	DEFINE_FIELD( CLift, saveOrigin, FIELD_BOOLEAN ),
};

//IMPLEMENT_SAVERESTORE( CLift, CBaseToggle );


int CLift::Save( CSave &save )
{


	if ( !CBaseToggle::Save(save) )
		return 0;
	return save.WriteFields( "CLift", this, m_SaveData, ARRAYSIZE(m_SaveData) );
}

int CLift ::Restore( CRestore &restore )
{

	if ( !CBaseToggle::Restore(restore) )
		return 0;

/*	if (saveOrigin)
	{
		pev->origin = vecOrigine;//on remet l'origine si on n'a pas encore utilis l'ascenseur
		UTIL_SetOrigin(pev, pev->origin);
	}
*/
	m_startsound = pev->noise;
	m_movesound = pev->noise1;
	m_stopsound = pev->noise2;
	m_locked = pev->noise3;

	
	int status = restore.ReadFields( "CLift", this, m_SaveData, ARRAYSIZE(m_SaveData) );

	if (!saveOrigin)//seulement si on a dj utilis l'ascenseur, sinon il va aller aux coordonnes 0 0 0 
	{
		SetThink(LaunchLift);

		pev->nextthink = pev->ltime +0.5;
	}

	return status;
}

void CLift :: KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "movespeed"))
	{
		movespeed = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "startsound"))
	{
		m_startsound = ALLOC_STRING(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "movesound"))
	{
		m_movesound = ALLOC_STRING(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "stopsound"))
	{
		m_stopsound = ALLOC_STRING(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "lockedsound"))
	{
		m_locked = ALLOC_STRING(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "x_offset"))
	{
		vecOffset.x = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "y_offset"))
	{
		vecOffset.y = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "z_offset"))
	{
		vecOffset.z = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "volume"))
	{
		m_volume = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else
	{
		CBaseToggle::KeyValue( pkvd );
	}
}

void CLift::Spawn()
{

	saveOrigin = 1;//on va sauvegarder l'origine au dpart
	
	//pev->angles		= g_vecZero;

	pev->solid		= SOLID_BSP;
	pev->movetype	= MOVETYPE_PUSH;
	pev->ltime		= gpGlobals->time;

	pev->noise		= m_startsound;
	pev->noise1		= m_movesound;
	pev->noise2		= m_stopsound;
	pev->noise3		= m_locked;

	MoveSound = 0;

	UTIL_SetOrigin(pev, pev->origin);		// set size and link into world
	UTIL_SetSize(pev, pev->mins, pev->maxs);
	SET_MODEL(ENT(pev), STRING(pev->model) );
	vecOrigine = pev->origin;//pour sauvegarder l'origine.
	Precache();

}

void CLift::Precache()
{
	if ( !FStringNull( m_movesound ) && strlen( STRING(m_movesound) ) > 4 )
	{
		PRECACHE_SOUND( (char*) STRING(m_movesound));
	}
	else
	{
		PRECACHE_SOUND ("plats/df_elev1-2.wav");

		m_movesound = ALLOC_STRING("plats/df_elev1-2.wav");
	}

	if ( !FStringNull( m_startsound ) && strlen( STRING(m_startsound) ) > 4 )
	{
		PRECACHE_SOUND( (char*) STRING(m_startsound));
	}
	else
	{
		PRECACHE_SOUND ("plats/df_elev2-1.wav");

		m_startsound = ALLOC_STRING("plats/df_elev2-1.wav");
	}

	if ( !FStringNull( m_stopsound ) && strlen( STRING(m_stopsound) ) > 4 )
	{
		PRECACHE_SOUND( (char*) STRING(m_stopsound));
	}
	else
	{
		PRECACHE_SOUND ("plats/df_elev2-3.wav");

		m_stopsound = ALLOC_STRING("plats/df_elev2-3.wav");
	}

	if ( !FStringNull( m_locked ) && strlen( STRING(m_locked) ) > 4 )
	{
		PRECACHE_SOUND( (char*) STRING(m_locked));
	}
	else
	{
		PRECACHE_SOUND ("items/locked1.wav");

		m_locked = ALLOC_STRING("items/locked1.wav");
	}	

}

void CLift::Use(CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	saveOrigin = 0; //aprs une utilisation il ne faut plus sauvegarder l'origine

	//ALERT(at_console, "ACTIVATEUR %s\n", STRING(pActivator->pev->classname) );
	//ALERT(at_console, "STARTSOUND %s\n", STRING(m_startsound));
	//ALERT(at_console, "STOPSOUND %s\n", STRING(m_stopsound));
	//ALERT(at_console, "MOVESOUND %s\n", STRING(m_movesound));
	if ( !Moving )
	{
		vecNewPosition = pActivator->pev->origin + vecOffset;

		vecStartPosition = pev->origin;//enregistre la position avant le mouvement

		if ( vecNewPosition == vecStartPosition )
		return;

		vecBloqueRet = vecNewPosition;//enregistre la valeur en cas de mutiples bloquages

		//vecOffset utilis si l'ascenseur ne doit pas tre centr sur le multisource
		//par exemple plusieurs ascenseurs commands par le mme multisource.
		//Ca peut servir si on veut qu'un bouton de commande factice suive l'ascenseur.
		//On peut penser qu'il suffit d'intgrer le bouton  l'ascenseur, 
		//mais on ne peut pas intgrer une func_button dans une autre func_lift.
		//On pourrait alors crer un bouton factice sur l'ascenseur et un bouton rel invisible sur le mur.
		//Mais pour commuter la texture du bouton a pose problme, la texture ne serait pas commute au bon moment,
		//l'ascenseur tant activ avec un temps de retard, de plus on ne pourrait mettre qu'un bouton, 
		//si on en met deux, les deux textures seraient commutes et on ne saurait pas quel bouton est activ.

		//ALERT(at_console, "NewPosition %f %f %f\n", vecNewPosition.x, vecNewPosition.y, vecNewPosition.z);
		

		EMIT_SOUND(ENT(pev), CHAN_VOICE, STRING(m_startsound), m_volume , ATTN_NORM);

		
		SetThink(LaunchLift);

		//LaunchLift();

		//ALERT(at_console, "LTIME %f\n", pev->ltime);
		//ALERT(at_console, "GLOBALTIME %f\n", gpGlobals->time);


		pev->nextthink = pev->ltime + 0.3;
		
		Moving = 1;//bloque les commandes
	}
	else//on joue un son de bip au bouton
	{

		EMIT_SOUND(ENT(pCaller->pev), CHAN_VOICE, STRING(m_locked), m_volume, ATTN_NORM);
	}
}


void CLift::LaunchLift()
{
	ALERT(at_console, "Launchlift %d\n", saveOrigin);
	//ALERT(at_console, "LTIME %f\n", pev->ltime);
	//ALERT(at_console, "GLOBALTIME %f\n", gpGlobals->time);
	//ALERT(at_console, "ORIGINE %f %f %f \n", pev->origin.x, pev->origin.y, pev->origin.z );

	EMIT_SOUND(ENT(pev), CHAN_STATIC, STRING(m_movesound), m_volume , ATTN_NORM);

	
	StartMove();
}


void CLift::StartMove()
{

	//ALERT(at_console, "Startmove\n");
	//ALERT(at_console, "MoveSpeed %f\n", movespeed);

	MoveSound = 1;//pour pouvoir relancer le son aprs avoir recharg la partie sauve pendant le dplacement de l'ascenseur
	SetMoveDone(CallStopMove);
	LinearMove(vecNewPosition, movespeed);
	
}

void CLift::StopMove()
{

	STOP_SOUND(ENT(pev), CHAN_STATIC, STRING(m_movesound));

	MoveSound = 0;

	EMIT_SOUND(ENT(pev), CHAN_VOICE, STRING(m_stopsound), m_volume , ATTN_NORM);
	
	Moving = 0;

	Bloque = 0;

	if (pev->spawnflags = LIFT_RETURN_START)
	{
		SetThink( ReturnHome );

		pev->nextthink = pev->ltime + 5; //on attent 5 seconde avant de retourner au dpart
	}
}

void CLift::ReturnHome( )
{
	if ( vecNewPosition == vecStartPosition )
		return;

	EMIT_SOUND(ENT(pev), CHAN_VOICE, STRING(m_startsound), m_volume , ATTN_NORM);

	vecNewPosition = vecStartPosition;

	if ( !Moving )
	{
		Moving = 1;

		SetThink(LaunchLift);

		pev->nextthink = pev->ltime + 0.3;
	}

}

void CLift::Blocked(CBaseEntity *pOther)
{

	pOther->TakeDamage(pev, pev, 1, DMG_CRUSH);

	//STOP_SOUND(ENT(pev), CHAN_STATIC, STRING(m_movesound));

	EMIT_SOUND(ENT(pev), CHAN_VOICE, STRING(m_stopsound), m_volume , ATTN_NORM);

	if (!Bloque)
	{
		vecNewPosition = vecStartPosition;
		Bloque++;//bloqu une fois
	}

	else
	{
		//on remet la valeur normale de vecNewPosition au cas ou le chemin serait bloqu des deux cts
		vecNewPosition = vecBloqueRet;
		Bloque--;
	}
	
	StartMove();
}
