/////////////////////////////////////////////////////////////////////////////
//
// Prise en charge des sons au format MP3
//
// Prise en charge des musiques au format:  MOD, S3M, XM, IT, MID, RMI, SGT
//
// Ce code va utiliser la DLL fmod pour jouer ces sons
//
// Par Jean-Franois Vuagniaux
//
// vuagniaux.jf@com.mcnet.ch
//
// Utilise Fmod version 3.4
//
//////////////////////////////////////////////////////////////////////////////

#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"player.h"
#include	"../apifmod/fmod.h"





//=============================================================
// Fonctions pour prendre en charge les sons au format MP3
// Utilise la DLL fmod.dll
// Lecture en fonction stream sans positionnement 3D
// Utilis pour jouer une bande son mp3 comme musique de fond
//=============================================================

FSOUND_STREAM *stream;

BOOL FsoundRunning;

int channel;

int pausedPlayPos;




void RestartPlay(int volume, entvars_t* pev);//pour relancer aprs une pause vers les menus


int UTIL_PlayMP3(const char *filename, int volume, int loop, int priority, entvars_t* pev)//volume entre 0 et 255, loop 1 ou 0
{															//filename : adresse  partir du dossier de Half-Life o se trouve fmod	
															//priority : dfini si le mp3 courant doit tre stopp et remplac
	//vrification de la position de lecture
	if ( stream )
	{
		int taillemp3 = FSOUND_Stream_GetLength( stream );
		int positionmp3 = FSOUND_Stream_GetPosition( stream );

		//ALERT(at_console, "Taille MP3 %d\nPosition MP3 %d\n", taillemp3, positionmp3);

		if ( positionmp3 >= taillemp3 - 256 || positionmp3 == 0 || priority ==1)//on est vers la fin du mp3 ou echec du test
		{
			FSOUND_Stream_Close(stream);

			stream = NULL;
		}
		else 
		{
			if(pausedPlayPos == positionmp3)//on n'a pas avanc dans la lecture
			{
				RestartPlay(volume, pev);

			}
			else
			{
				pausedPlayPos = positionmp3;

				FSOUND_SetVolume(channel, volume);//met  jour le volume

			}
			return 2;// on continue de jouer le mp3 en cours
		}
	}
	
	//rcupration de la longueur du fichier mp3
	int longueur = 0;
	FILE *	mp3;

	mp3 = fopen(filename, "rb");//ouverture du fichier

	if (!mp3)
		{
			return 0;
		}
	
	fseek(mp3, 0, SEEK_END);
	
	longueur = ftell(mp3);//longueur du mp3

	//ALERT(at_console, "LONGUEUR %d\n", longueur);
	
	fclose(mp3);
	
	//s'il faut jouer en boucle
	int LoopType = 0;

	if( loop )
	{
		LoopType = FSOUND_LOOP_NORMAL;
	}
	else
	{
		LoopType = FSOUND_LOOP_OFF;
	}
	
	if ( stream )
	{
		FSOUND_Stream_Close(stream);
	}
	
	stream = NULL;

	UTIL_InitFsound();

	stream = FSOUND_Stream_OpenFile(filename, FSOUND_NORMAL | LoopType, longueur);  // Ouvre le fichier
	
	
							
	channel = FSOUND_Stream_Play(FSOUND_FREE, stream); // Joue le .mp3 
	
	FSOUND_SetVolume(channel, volume);
	
	return 1;
}

void UTIL_StopMP3(float fadetime)
{
	if ( stream )
	{
		FSOUND_Stream_Close(stream);
	}

	stream = NULL;

	
}

void RestartPlay(int volume, entvars_t* pev)
{
	channel = FSOUND_Stream_Play(FSOUND_FREE, stream); // Joue le .mp3 

	FSOUND_SetVolume(channel, volume);

	
}

//=========================================================================================
// Fonctions pour prendre en charge les musiques au format MOD, S3M, XM, IT, MID, RMI, SGT
// Utilise la DLL fmod.dll
//=========================================================================================	


FMUSIC_MODULE *mod;

BOOL FmusicStarted;

int UTIL_PlaySong(const char *filename, int volume, int loop, int priority)
{

	//vrification de la position de lecture

	if ( mod )
	{
		if ( FMUSIC_IsFinished( mod ) || priority == 1)
		{

			FMUSIC_StopSong( mod );
			FMUSIC_FreeSong( mod );//libre la mmoire

			mod = NULL;
		}
		else
		{
			return 2; //on continue  jouer
		}
	}

	if ( mod )
	{

		FMUSIC_FreeSong( mod );//libre la mmoire

		mod = NULL;
	}


	UTIL_InitFsound( );

	mod = FMUSIC_LoadSong(filename);

	FMUSIC_PlaySong( mod );

	FmusicStarted = 1;

	return 1;
}

void UTIL_StopSong( void )
{
	if ( mod )
	{

		FMUSIC_StopSong( mod );
	}

	mod = NULL;

	FmusicStarted = 0;
}

int UTIL_InitFsound( void )
{

	if ( !FsoundRunning)
	{
		//ALERT(at_console, "INIT FSOUND \n");
		FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND); // Output par DirectSound
		FSOUND_SetBufferSize(200);
		FSOUND_SetDriver(0); // Slectionne le prifrique DirectSound par dfaut
		FSOUND_SetSFXMasterVolume(255);

		FSOUND_Init(22050, 32, 0); // Initialise le playback stream  22khz 32 canaux

		FSOUND_SetSFXMasterVolume(255);

		FsoundRunning = 1;
		return 1;
	}
	else 
		return 0;

}

// utilis par trigger_changevel avec : ChangeLevelNow( CBaseEntity *pActivator )
// utilis dans client.cpp avec : ClientDisconnect( edict_t *pEntity )
void UTIL_CloseFsound(void)
{

	FSOUND_Close();

	stream = NULL;

	pausedPlayPos = 0;

	FsoundRunning = 0;

}

void UTIL_FsoundSilent(int vol)
{

	if ( FsoundRunning )
		FSOUND_SetSFXMasterVolume(vol);
}


//================================================================================
//
//	Classe d'entit ponctuelle grant le son en 3 dimensions
//	Lecture en fonction sample des sons au format mp3
//	Utilis pour diverses applications comme une musique localise dans l'espace 
//	ou un objet mobile mettant un son
//
//=================================================================================



class C3Dmp3 : public CBaseEntity
{
public:
	void KeyValue( KeyValueData* pkvd);
	void Spawn( void );
	void Precache( void );
	void EXPORT ToggleUse ( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );

	void EXPORT Think3D( void );

	int Playmp3(const char *filename, int volume, int loop, int StartStop);

	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];
	virtual int	ObjectCaps( void ) { return (CBaseEntity :: ObjectCaps() & ~FCAP_ACROSS_TRANSITION); }

	BOOL	m_fActive;	
	BOOL	m_fLooping;
	BOOL	m_Update3d;
	BOOL	m_Paused;
	int		volume;
	int		attenuation;
	int		channel;
	float   m_flSpawnTime;

	FSOUND_SAMPLE *sample;

	CBasePlayer * Player;

	char* filename;

	Vector VecOrigine;
	Vector VecVelocity;

	FSOUND_DSPUNIT * DSP1;

};

LINK_ENTITY_TO_CLASS( ambient_mp3, C3Dmp3 );
TYPEDESCRIPTION	C3Dmp3::m_SaveData[] = 
{
	
	DEFINE_FIELD( C3Dmp3, m_fActive, FIELD_BOOLEAN ),
	DEFINE_FIELD( C3Dmp3, m_fLooping, FIELD_BOOLEAN ),
	DEFINE_FIELD( C3Dmp3, m_Update3d, FIELD_BOOLEAN ),
	DEFINE_FIELD( C3Dmp3, attenuation, FIELD_INTEGER ),
	DEFINE_FIELD( C3Dmp3, volume, FIELD_INTEGER ),
	DEFINE_FIELD( C3Dmp3, channel, FIELD_INTEGER ),

};

//IMPLEMENT_SAVERESTORE( C3Dmp3, CBaseEntity );


int C3Dmp3::Save( CSave &save )
{
	if ( !CBaseEntity::Save(save) )
		return 0;
	return save.WriteFields( "C3Dmp3", this, m_SaveData, ARRAYSIZE(m_SaveData) );
}

int C3Dmp3 ::Restore( CRestore &restore )
{

	if ( !CBaseEntity::Restore(restore) )
		return 0;

	int status = restore.ReadFields( "C3Dmp3", this, m_SaveData, ARRAYSIZE(m_SaveData) );

	
	FSOUND_Close();

	FsoundRunning = 0;

	filename = (char*) STRING(pev->message);

	UTIL_InitFsound();

	return status;
}

void C3Dmp3 :: Spawn( void )
{
	
    pev->solid		= SOLID_NOT;
    pev->movetype	= MOVETYPE_NONE;

	//inizialisation de FMOD si ce n'est pas dj le cas
	UTIL_InitFsound();
	//FSOUND_3D_Listener_SetDistanceFactor(40);

	SetThink(Think3D);//initialise mais ne dmarre pas
	pev->nextthink = 0;

	SetUse ( ToggleUse );

	UTIL_SetOrigin( pev, pev->origin );
	
	m_fActive = FALSE;

	m_Update3d = 0;//pas de code 3D pour le moment
	
	
	if ( FBitSet ( pev->spawnflags, AMBIENT_SOUND_NOT_LOOPING ) )
		m_fLooping = FALSE;
	else
		m_fLooping = TRUE;
	Precache( );


}


void C3Dmp3 :: Precache( void )
{
	filename = (char*) STRING(pev->message);

	

	if ( !FStringNull( pev->message ) && strlen( filename ) > 1 )
	{
		//if (*szSoundFile != '!')
		
	}

	if ( !FBitSet (pev->spawnflags, AMBIENT_SOUND_START_SILENT ) )
	{

		if (m_fLooping)
			m_fActive = TRUE;
	}
	if ( m_fActive )
	{
		//code pour continuer  jouer

		m_flSpawnTime = gpGlobals->time;
		//temporisation de 1 seconde pour laisser le temps au joueur de prendre place dans l'espace 3d

		Playmp3(filename, volume, m_fLooping, 1);

		pev->nextthink = gpGlobals->time + 0.1;
		
	}
}


void C3Dmp3 :: KeyValue( KeyValueData *pkvd )
{

	if (FStrEq(pkvd->szKeyName, "volume"))
	{
		volume = atoi(pkvd->szValue);

		if ( volume > 255 )
			volume = 255;

		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "attenuation"))
	{
		attenuation = atoi(pkvd->szValue);

		if ( attenuation == 0 )
			attenuation = 1;

		pkvd->fHandled = TRUE;
	}
	else

		CBaseEntity::KeyValue( pkvd );
}


void C3Dmp3 :: ToggleUse ( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{


	if ( useType != USE_TOGGLE )
	{
		if ( (m_fActive && useType == USE_ON) || (!m_fActive && useType == USE_OFF) )
			return;
	}
	

	// Toggle

	// m_fActive est TRUE seulement si un son est en train de jouer.
	
	if ( m_fActive )
	{// on arrte le mp3

			
		m_fActive = FALSE;
			
		// bidouillage pour que le code dans Precache fonctionne (m_fActive) aprs un save/restore
		pev->spawnflags |= AMBIENT_SOUND_START_SILENT;

		//code d'arrt du son mp3
		Playmp3(filename, volume, m_fLooping, 0);

		pev->nextthink = gpGlobals->time + 0.1;
	}
	else 
	{// on va lancer le son mp3

		// on commute seulement si c'est un son en boucle. Si ce n'est pas un son en boucle, 
		// chaque dclenchement va dmarrer le son.  Si le son est toujours en train de jouer, 
		// il va tre arrt et redmarr.
		

		if (m_fLooping)
			m_fActive = TRUE;
		
		else // code pour couper le son mp3
			
			Playmp3(filename, volume, m_fLooping, 0);
		
		//code pour relancer le son mp3
		Playmp3(filename, volume, m_fLooping, 1);
		
		pev->nextthink = gpGlobals->time + 0.1;

	} 
}

int C3Dmp3 :: Playmp3(const char *filename, int volume, int loop, int StartStop)
{
	if (attenuation == 0 )
		attenuation = 1;

	VecOrigine = pev->origin;
	VecVelocity = pev->velocity;
	VecOrigine = VecOrigine / (40 / attenuation);

	int playvolume = 0;

	int result = 0;

	//s'il faut jouer en boucle
	int LoopType = 0;

	if( m_fLooping )
	{
		LoopType = FSOUND_LOOP_NORMAL;
	}
	else
	{
		LoopType = FSOUND_LOOP_OFF;
	}

	if( StartStop ) //on dmarre le son
	{

		sample = FSOUND_Sample_Load(FSOUND_FREE, filename, FSOUND_HW3D | LoopType, 0);

		FSOUND_Sample_SetMinMaxDistance(sample, 5, 100);

		channel = FSOUND_PlaySoundEx(FSOUND_FREE, sample, DSP1, 1);//dmarre le son en mode pause

		FSOUND_SetVolume(channel, 0);

		FSOUND_3D_SetAttributes(channel, VecOrigine, VecVelocity);

		//FSOUND_3D_Listener_SetDistanceFactor(40.0);

		FSOUND_3D_Update();
		

		m_Paused = 1;//le son est en pause au dpart

		m_flSpawnTime = gpGlobals->time; //moment de cration du son

		m_Update3d = 1;

	}
	else //on arrte le son
	{

		if ( sample )
		{
			FSOUND_Sample_Free(sample);
		}
		
		m_Update3d = 0;//on arrte le code 3D

	}

return result;

}

void C3Dmp3::Think3D( void )
{


	if ( !m_Update3d )
		return;
	
	
	if(!FSOUND_IsPlaying(channel) && !m_Paused)
	{
		//ALERT(at_console, "RESTART\n");
		if ( sample )
		{
			FSOUND_Sample_Free(sample);
		}

		Playmp3(filename, volume, m_fLooping, 1);

	}

	//ALERT(at_console, "CHANNEL %d PLAYING %d VOLUME %d\n", channel, FSOUND_IsPlaying(channel), FSOUND_GetVolume(channel));
		
	if (attenuation == 0 )
		attenuation = 1;

	CBaseEntity * pEntity = NULL;

	pEntity = UTIL_PlayerByIndex( 1 );

	if ( pEntity == NULL )
	{
		pev->nextthink = gpGlobals->time + 0.1;
		
		return;
	}
	
	
	//dfinition de la position et de l'orientation du joueur 
	

	Vector VecForward = gpGlobals->v_forward;
	Vector VecUpward = gpGlobals->v_up;
	Vector VecPlayerpos = pEntity->pev->origin;
	VecPlayerpos = VecPlayerpos / (40 / attenuation);

	//ALERT(at_console, "Player %f %f %f\n Source %f %f %f\n", VecPlayerpos.x, VecPlayerpos.y, VecPlayerpos.z, pev->origin.x, pev->origin.y, pev->origin.z);

	//ALERT(at_console, "Player %f %f\n", VecForward.x, VecForward.y);

	//ATTENTION fmod n'utilise pas le systme de coordonnes de HL, il faut rajuster les angles dans la composante z
	
	FSOUND_3D_Listener_SetAttributes(VecPlayerpos, NULL, VecForward.x, VecForward.y, -VecForward.z, VecUpward.x, VecUpward.y, -VecUpward.z); 
	
	FSOUND_3D_SetAttributes(channel, VecOrigine, VecVelocity);

	//FSOUND_3D_Listener_SetDistanceFactor(40.0);// l'unit de FMOD fait 1 mtre dans Half-Life l'unit fait 1 pouce
	
	FSOUND_3D_Update();//met  jour les paramtres 3D du son

	FSOUND_SetVolume(channel, volume);

	pev->nextthink = gpGlobals->time + 0.1;

	if ( gpGlobals->time > m_flSpawnTime + 1 )//on retire la pause aprs 1 seconde le temps de mettre les attributs au son
	{

		FSOUND_SetPaused(channel, 0);

		m_Paused = 0;
	}

}