/////////////////////////////////////////////////////////////////////////////
//
// Prise en charge des sons au format MP3
//
// Prise en charge des musiques au format:  MOD, S3M, XM, IT, MID, RMI, SGT
//
// Ce code va utiliser la DLL fmod pour jouer ces sons
//
// Par Jean-Franois Vuagniaux
//
// vuagniaux.jf@com.mcnet.ch
//
//////////////////////////////////////////////////////////////////////////////

#include	"extdll.h"
#include	"util.h"
#include	"../apifmod/fmod.h"





//=============================================================
// Fonctions pour prendre en charge les sons au format MP3
// Utilise la DLL fmod.dll
//=============================================================

FSOUND_STREAM *stream;

BOOL FsoundRunning;

int channel;

int pausedPlayPos;

//extern int gmsgMp3Stream;

void RestartPlay(int volume, entvars_t* pev);//pour relancer aprs une pause vers les menus


int UTIL_PlayMP3(const char *filename, int volume, int loop, int priority, entvars_t* pev)//volume entre 0 et 255, loop 1 ou 0
{															//filename : adresse  partir du dossier de Half-Life o se trouve fmod	
															//priority : dfini si le mp3 courant doit tre stopp et remplac
	//vrification de la position de lecture
	if ( stream )
	{
		int taillemp3 = FSOUND_Stream_GetLength( stream );
		int positionmp3 = FSOUND_Stream_GetPosition( stream );

		ALERT(at_console, "Taille MP3 %d\nPosition MP3 %d\n", taillemp3, positionmp3);

		if ( positionmp3 >= taillemp3 - 256 || positionmp3 == 0 || priority ==1)//on est vers la fin du mp3 ou echec du test
		{
			FSOUND_Stream_Close(stream);

			stream = NULL;
		}
		else 
		{
			if(pausedPlayPos == positionmp3)//on n'a pas avanc dans la lecture
			{
				RestartPlay(volume, pev);

			}
			else
			{
				pausedPlayPos = positionmp3;

			}
			return 2;// on continue de jouer le mp3 en cours
		}
	}
	//rcupration de la longueur du fichier mp3
	int longueur = 0;
	FILE *	mp3;

	mp3 = fopen(filename, "rb");//ouverture du fichier

	if (!mp3)
		{
			return 0;
		}
	
	fseek(mp3, 0, SEEK_END);
	
	longueur = ftell(mp3);//longueur du mp3

	//ALERT(at_console, "LONGUEUR %d\n", longueur);
	
	fclose(mp3);
	
	//s'il faut jouer en boucle
	int LoopType = 0;

	if( loop )
	{
		LoopType = FSOUND_LOOP_NORMAL;
	}
	else
	{
		LoopType = FSOUND_LOOP_OFF;
	}
	
	if ( stream )
	{
		FSOUND_Stream_Close(stream);
	}
	
	stream = NULL;

	UTIL_InitFsound();

	stream = FSOUND_Stream_OpenFile(filename, FSOUND_NORMAL | LoopType, longueur);  // Ouvre le fichier
	
	
	channel = FSOUND_Stream_Play3DAttrib(FSOUND_FREE, stream, -1, volume, -1, NULL, NULL); // Joue le .mp3 
	
	//on envoie un message la dll client pour pouvoir baisser le volume pendant une pause.
	//pas en fonction car le son est coup de toute faon par FSOUND_StopSound(FSOUND_ALL) dans input32.cpp
	//MESSAGE_BEGIN(MSG_ONE, gmsgMp3Stream, NULL, pev);
	
	//WRITE_SHORT(channel);
	//WRITE_SHORT(volume);

	//MESSAGE_END();
	
	return 1;
}

void UTIL_StopMP3(float fadetime)
{
	if ( stream )
	{
		FSOUND_Stream_Close(stream);
	}

	stream = NULL;

	
}

void RestartPlay(int volume, entvars_t* pev)
{
	channel = FSOUND_Stream_Play3DAttrib(FSOUND_FREE, stream, -1, volume, -1, NULL, NULL); // Joue le .mp3 

	//on envoie un message la dll client pour pouvoir baisser le volume pendant une pause.
	//pas en fonction car le son est coup de toute faon par FSOUND_StopSound(FSOUND_ALL) dans input32.cpp
	//MESSAGE_BEGIN(MSG_ONE, gmsgMp3Stream, NULL, pev);
	
	//WRITE_SHORT(channel);
	//WRITE_SHORT(volume);

	//MESSAGE_END();
}

//=========================================================================================
// Fonctions pour prendre en charge les musiques au format MOD, S3M, XM, IT, MID, RMI, SGT
// Utilise la DLL fmod.dll
//=========================================================================================	


FMUSIC_MODULE *mod;

BOOL FmusicStarted;

int UTIL_PlaySong(const char *filename, int volume, int loop, int priority)
{

	//vrification de la position de lecture

	if ( mod )
	{
		if ( FMUSIC_IsFinished( mod ) || priority == 1)
		{

			FMUSIC_StopSong( mod );
			FMUSIC_FreeSong( mod );//libre la mmoire

			mod = NULL;
		}
		else
		{
			return 2; //on continue  jouer
		}
	}

	if ( mod )
	{

		FMUSIC_FreeSong( mod );//libre la mmoire

		mod = NULL;
	}


	UTIL_InitFsound( );

	mod = FMUSIC_LoadSong(filename);

	FMUSIC_PlaySong( mod );

	FmusicStarted = 1;

	return 1;
}

void UTIL_StopSong( void )
{
	if ( mod )
	{

		FMUSIC_StopSong( mod );
	}

	mod = NULL;

	FmusicStarted = 0;
}

int UTIL_InitFsound( void )
{

	if ( !FsoundRunning)
	{
		ALERT(at_console, "INIT FSOUND \n");
		FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND); // Output par DirectSound
		FSOUND_SetBufferSize(200);
		FSOUND_SetDriver(0); // Slectionne le prifrique DirectSound par dfaut
		FSOUND_SetSFXMasterVolume(255);

		FSOUND_Init(22050, 32, 0); // Initialise le playback stream  22khz 32 canaux

		FSOUND_SetSFXMasterVolume(255);

		FsoundRunning = 1;
		return 1;
	}
	else 
		return 0;

}

// utilis par trigger_changevel avec : ChangeLevelNow( CBaseEntity *pActivator )
// utilis dans client.cpp avec : ClientDisconnect( edict_t *pEntity )
void UTIL_CloseFsound(void)
{

	FSOUND_Close();

	stream = NULL;

	pausedPlayPos = 0;

	FsoundRunning = 0;

}

void UTIL_FsoundSilent(int vol)
{

	if ( FsoundRunning )
		FSOUND_SetSFXMasterVolume(vol);
}