/***
*
*	Code pour le mynok, norme insecte trs emmerdant
*
****/


//=========================================================
// Mynok
//=========================================================

#include	"extdll.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"schedule.h"
#include    "flyingmonster.h"
#include	"nodes.h"
#include	"soundent.h"
#include	"animation.h"
#include	"effects.h"
#include	"weapons.h"

#define SEARCH_RETRY	16

#define MYNOK_SPEED 150

extern CGraph WorldGraph;

//=========================================================
// Evnements d'animation
//=========================================================
#define	MYNOK_AE_PIQUE		1
#define	MYNOK_AE_FLAPS		2

// dfinitions spcifiques

#define MYNOK_MELEE_DIST	100

// Tches spcifiques

enum
{
	TASK_MYNOK_CIRCLE_ENEMY = LAST_COMMON_TASK + 1,
	TASK_MYNOK_FLY,
};


class CMynok : public CFlyingMonster
{
public:
	

	void Spawn( void );
	void Precache( void );
	void SetYawSpeed( void );
	int  Classify ( void );
	void HandleAnimEvent( MonsterEvent_t *pEvent );
	CUSTOM_SCHEDULES;
	
	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	Schedule_t* GetSchedule ( void );
	Schedule_t* GetScheduleOfType ( int Type );
	
	void Killed( entvars_t *pevAttacker, int iGib );

	void EXPORT CombatUse( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	void EXPORT PiqueTouch( CBaseEntity *pOther );
	
	void StartTask ( Task_t *pTask );
	void RunTask ( Task_t *pTask );
	
	BOOL CheckRangeAttack1 ( float flDot, float flDist );	// rien pour le moment
	BOOL CheckMeleeAttack1 ( float flDot, float flDist );	// pique
	
	float ChangeYaw( int speed );
	Activity GetStoppedActivity( void );
	
	//void BecomeDead( void );

	void Move ( float flInterval );
	void MoveExecute( CBaseEntity *pTargetEnt, const Vector &vecDir, float flInterval );
	void MonsterThink( void );
	void Stop( void );
	void Fly( void );
	Vector DoProbe(const Vector &Probe);
	
	float VectorToPitch( const Vector &vec);
	float FlPitchDiff( void );
	float ChangePitch( int speed );

	
	void PainSound( void );
	void AlertSound( void );
	void IdleSound( void );
	void AttackSound( void );
	void DeathSound( void );

	static const char *pAttackSounds[];
	static const char *pAttackHitSounds[];
	static const char *pAttackMissSounds[];
	static const char *pIdleSounds[];
	static const char *pAlertSounds[];
	static const char *pPainSounds[];
	static const char *pDeathSounds[];

	int TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType );
	void TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType);
	
	void GibMonster( void );

	Vector m_SaveVelocity;
	float m_idealDist;
	float m_flEnemyTouched;
	BOOL  m_bOnAttack;
	float m_flNextAlert;
	float m_nextRangeCheck; // pour viter qu'il se colle au joueur

	float m_flMaxSpeed;
	float m_flMinSpeed;
	float m_flMaxDist;

	BOOL	m_fCanAttack;
	float	m_flNextAttackCheck;
	float   m_flNextAttack;
	float   m_timetoplay;  // pour le bruit des ailles
	
};

LINK_ENTITY_TO_CLASS( monster_mynok, CMynok );

TYPEDESCRIPTION	CMynok::m_SaveData[] = 
{
	
	DEFINE_FIELD( CMynok, m_SaveVelocity, FIELD_VECTOR ),
	DEFINE_FIELD( CMynok, m_idealDist, FIELD_FLOAT ),
	DEFINE_FIELD( CMynok, m_flEnemyTouched, FIELD_FLOAT ),
	DEFINE_FIELD( CMynok, m_bOnAttack, FIELD_BOOLEAN ),
	DEFINE_FIELD( CMynok, m_flMaxSpeed, FIELD_FLOAT ),
	DEFINE_FIELD( CMynok, m_flMinSpeed, FIELD_FLOAT ),
	DEFINE_FIELD( CMynok, m_flMaxDist, FIELD_FLOAT ),
	DEFINE_FIELD( CMynok, m_flNextAlert, FIELD_TIME ),
	DEFINE_FIELD( CMynok, m_nextRangeCheck, FIELD_TIME ),
	DEFINE_FIELD( CMynok, m_flNextAttack, FIELD_TIME ),
};
IMPLEMENT_SAVERESTORE( CMynok, CFlyingMonster );


const char *CMynok::pAttackSounds[] = 
{
	"mynok/attack1.wav",
	"mynok/attack2.wav",
	
};

const char *CMynok::pAttackHitSounds[] = 
{
	"mynok/hit1.wav",
	"mynok/hit2.wav",
	
};

const char *CMynok::pAttackMissSounds[] = 
{
	"mynok/miss1.wav",
	"mynok/miss2.wav",
	
};

const char *CMynok::pIdleSounds[] = 
{
	"mynok/idle1.wav",
	"mynok/idle2.wav",
	
};

const char *CMynok::pAlertSounds[] = 
{
	"mynok/alert1.wav",
	
};

const char *CMynok::pPainSounds[] = 
{
	"mynok/pain1.wav",
	"mynok/pain2.wav",
	
};

const char *CMynok::pDeathSounds[] = 
{
	"mynok/die1.wav",
	"mynok/die2.wav",
};


//=========================================================
// Classification
//=========================================================
int	CMynok :: Classify ( void )
{
	return	CLASS_ALIEN_MONSTER;
}

//=========================================================
// SetYawSpeed - allows each sequence to have a different
// turn rate associated with it.
//=========================================================
void CMynok :: SetYawSpeed ( void )
{
	int ys;

	ys = 120;

	pev->yaw_speed = ys;
}


void CMynok :: TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType)
{
	
	
		m_LastHitGroup = ptr->iHitgroup;

		switch ( ptr->iHitgroup )
		{
		case HITGROUP_GENERIC:
			break;
		case HITGROUP_HEAD:
			flDamage *= gSkillData.monHead;
			break;
		case HITGROUP_CHEST:
			flDamage *= gSkillData.monChest;
			break;
		case HITGROUP_STOMACH:
			flDamage *= gSkillData.monStomach;
			break;
		case HITGROUP_LEFTARM:
		case HITGROUP_RIGHTARM:
			flDamage *= gSkillData.monArm;
			break;
		case HITGROUP_LEFTLEG:
		case HITGROUP_RIGHTLEG:
			flDamage *= gSkillData.monLeg;
			break;
		default:
			break;
		}


		SpawnBlood(ptr->vecEndPos, BloodColor(), flDamage);
		TraceBleed( flDamage, vecDir, ptr, bitsDamageType );
		
		AddMultiDamage( pevAttacker, this, flDamage, bitsDamageType );

}


int CMynok :: TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType )
{
	
	if ( IsAlive() )
		PainSound();
	return CBaseMonster::TakeDamage( pevInflictor, pevAttacker, flDamage, bitsDamageType );
}


void CMynok::Killed( entvars_t *pevAttacker, int iGib )
{
	
		CBaseMonster::Killed( pevAttacker, iGib );
		pev->velocity = Vector( 0, 0, 0 );
}


void CMynok::GibMonster( void )
{
	
	CFlyingMonster::GibMonster( );
}


void CMynok :: PainSound( void )
{
	
	if (RANDOM_LONG(0,5) < 2)
		EMIT_SOUND_ARRAY_DYN( CHAN_BODY, pPainSounds ); 

}	

void CMynok :: AlertSound( void )
{
	
	EMIT_SOUND_ARRAY_DYN( CHAN_VOICE, pAlertSounds ); 
	
}

void CMynok :: IdleSound( void )
{
	
	EMIT_SOUND_ARRAY_DYN( CHAN_VOICE, pIdleSounds ); 

}

void CMynok :: AttackSound( void )
{
	
	EMIT_SOUND_ARRAY_DYN( CHAN_WEAPON, pAttackSounds ); 

}

void CMynok :: DeathSound( void )
{
	

	EMIT_SOUND_ARRAY_DYN( CHAN_VOICE, pDeathSounds ); 

}

//=========================================================
// Evnement des animations
//=========================================================
void CMynok :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
	switch( pEvent->event )
	{
		case MYNOK_AE_PIQUE:
		{
			CBaseEntity *pHurt = CheckTraceHullAttack( MYNOK_MELEE_DIST, gSkillData.mynokDmgPique, DMG_POISON );
			
			if ( pHurt )//touch
			{
				m_bOnAttack = FALSE;

				pHurt->pev->punchangle.y = -15;
				pHurt->pev->punchangle.x = -10;
				pHurt->pev->velocity = pHurt->pev->velocity + gpGlobals->v_forward * 20 + gpGlobals->v_up * 40;


				EMIT_SOUND_DYN ( ENT(pev), CHAN_WEAPON, pAttackHitSounds[ RANDOM_LONG(0,ARRAYSIZE(pAttackHitSounds)-1) ], 1.0, ATTN_NORM, 0, 100 + RANDOM_LONG(-5,5) );

				Vector vecArmPos, vecArmAng;
				GetAttachment( 0, vecArmPos, vecArmAng ); //attachement 0 vers la gueule du dragon
				SpawnBlood(vecArmPos, pHurt->BloodColor(), 25);// une gicle de sang
			}

			else
			{
				// c'est rat
				EMIT_SOUND_DYN ( ENT(pev), CHAN_WEAPON, pAttackMissSounds[ RANDOM_LONG(0,ARRAYSIZE(pAttackMissSounds)-1) ], 1.0, ATTN_NORM, 0, 100 + RANDOM_LONG(-5,5) );
			}
	

		}
		break;

		case MYNOK_AE_FLAPS:
		{

			if ( m_timetoplay < gpGlobals->time )
			{
				switch (RANDOM_LONG(0,1))
				{
				// pied gauche
				case 0:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "mynok/fly1.wav", 1, 0.3, 0, 100 );	break;
				case 1:	EMIT_SOUND_DYN ( ENT(pev), CHAN_BODY, "mynok/fly2.wav", 1, 0.3, 0, 100 );	break;
				}

				m_timetoplay = gpGlobals->time + 3,8;
			}
		}
		break;
		default:
			CBaseMonster::HandleAnimEvent( pEvent );
			break;
		
	}
}

//=========================================================
// Spawn
//=========================================================
void CMynok :: Spawn()
{
	Precache( );

	SET_MODEL(ENT(pev), "models/mynok.mdl");
	UTIL_SetSize( pev, Vector( -32, -32, 0 ), Vector( 32, 32, 64 ));

	pev->solid			= SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_FLY;
	pev->flags			|= FL_MONSTER;
	m_bloodColor		= BLOOD_COLOR_GREEN;
	pev->health			= gSkillData.mynokHealth;
	pev->view_ofs		= Vector( 0, 0, 20 );// position of the eyes relative to monster's origin.
	m_flFieldOfView		= VIEW_FIELD_WIDE;// indicates the width of this monster's forward view cone ( as a dotproduct result )
	m_MonsterState		= MONSTERSTATE_NONE;
	pev->iuser3			= 1;//masse
	pev->iuser4			= 1;

	SetFlyingSpeed( MYNOK_SPEED );
	SetFlyingMomentum( 2.5 );	// Set momentum constant
	
	m_afCapability		= bits_CAP_RANGE_ATTACK1 | bits_CAP_FLY;
	
	MonsterInit();

	SetTouch( PiqueTouch );
	SetUse( CombatUse );

	m_idealDist = 384;
	m_flMinSpeed = 80;
	m_flMaxSpeed = 300;
	m_flMaxDist = 384;

	Vector Forward;
	UTIL_MakeVectorsPrivate(pev->angles, Forward, 0, 0);
	pev->velocity = m_flightSpeed * Forward.Normalize();
	m_SaveVelocity = pev->velocity;
}

//=========================================================
// Precache - precaches all resources this monster needs
//=========================================================
void CMynok :: Precache()
{
	PRECACHE_MODEL("models/mynok.mdl");

	PRECACHE_SOUND_ARRAY( pAttackSounds );
	PRECACHE_SOUND_ARRAY( pAttackHitSounds );
	PRECACHE_SOUND_ARRAY( pAttackMissSounds );
	PRECACHE_SOUND_ARRAY( pIdleSounds );
	PRECACHE_SOUND_ARRAY( pAlertSounds );
	PRECACHE_SOUND_ARRAY( pPainSounds );
	PRECACHE_SOUND_ARRAY( pDeathSounds );
	PRECACHE_SOUND( "mynok/fly1.wav" );
	PRECACHE_SOUND( "mynok/fly2.wav" );

}	

//=========================================================
// AI Schedules Specific to this monster
//=========================================================


static Task_t	tlFlyAround[] =
{
	{ TASK_SET_ACTIVITY,			(float)ACT_WALK },
	{ TASK_MYNOK_FLY,		0.0 },
};

static Schedule_t	slFlyAround[] =
{
	{ 
		tlFlyAround,
		ARRAYSIZE(tlFlyAround), 
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE	|
		bits_COND_SEE_ENEMY		|
		bits_COND_NEW_ENEMY		|
		bits_COND_HEAR_SOUND,
		bits_SOUND_PLAYER |
		bits_SOUND_COMBAT,
		"FlyAround"
	},
};

static Task_t	tlFlyAgitated[] =
{
	{ TASK_STOP_MOVING,				(float) 0 },
	{ TASK_SET_ACTIVITY,			(float)ACT_RUN },
	{ TASK_WAIT,					(float)2.0 },
};

static Schedule_t	slFlyAgitated[] =
{
	{ 
		tlFlyAgitated,
		ARRAYSIZE(tlFlyAgitated), 
		0, 
		0, 
		"FlyAgitated"
	},
};


static Task_t	tlCircleEnemy[] =
{
	{ TASK_SET_ACTIVITY,			(float)ACT_WALK },
	{ TASK_MYNOK_CIRCLE_ENEMY, 0.0 },
};

static Schedule_t	slCircleEnemy[] =
{
	{ 
		tlCircleEnemy,
		ARRAYSIZE(tlCircleEnemy), 
		bits_COND_NEW_ENEMY		|
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE	|
		bits_COND_CAN_MELEE_ATTACK1 |
		bits_COND_CAN_RANGE_ATTACK1,
		0,
		"CircleEnemy"
	},
};


DEFINE_CUSTOM_SCHEDULES(CMynok)
{
    slFlyAround,
	slFlyAgitated,
	slCircleEnemy,
	
};
IMPLEMENT_CUSTOM_SCHEDULES(CMynok, CFlyingMonster);


/////////////////////////
// fonctions spcifiques
/////////////////////////

void CMynok::PiqueTouch( CBaseEntity *pOther )
{
	// pique si on touche l'ennemi
	if ( pOther == m_hEnemy ) 
	{
		m_flEnemyTouched = gpGlobals->time;
		m_bOnAttack = TRUE;
	}
}

void CMynok::CombatUse( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	if ( !ShouldToggle( useType, m_bOnAttack ) )
		return;

	if (m_bOnAttack)
	{
		m_bOnAttack = 0;
	}
	else
	{
		m_bOnAttack = 1;
	}
}


//=========================================================
// StartTask
//=========================================================
void CMynok :: StartTask ( Task_t *pTask )
{
	
	switch (pTask->iTask)
	{
	case TASK_MYNOK_CIRCLE_ENEMY:
		break;
	case TASK_MYNOK_FLY:
		break;
	case TASK_SMALL_FLINCH:
		if (m_idealDist > 128)
		{
			m_flMaxDist = 512;
			m_idealDist = 512;
		}
		else
		{
			m_bOnAttack = TRUE;
		}
		CFlyingMonster::StartTask(pTask);
		break;


	default:
		CFlyingMonster::StartTask(pTask);
		break;
	}
}


//=========================================================
// RunTask 
//=========================================================
void CMynok :: RunTask ( Task_t *pTask )
{

	
	switch ( pTask->iTask )
	{
	
		case TASK_MYNOK_CIRCLE_ENEMY:
		if (m_hEnemy == NULL)
		{
			TaskComplete( );
		}
		else if (FVisible( m_hEnemy ))
		{
			Vector vecFrom = m_hEnemy->EyePosition( );

			Vector vecDelta = (pev->origin - vecFrom).Normalize( );
			Vector vecFly = CrossProduct( vecDelta, Vector( 0, 0, 1 ) ).Normalize( );
			
			if (DotProduct( vecFly, m_SaveVelocity ) < 0)
				vecFly = vecFly * -1.0;

			Vector vecPos = vecFrom + vecDelta * m_idealDist + vecFly * 32;

			TraceResult tr;
		
			UTIL_TraceHull( vecFrom, vecPos, ignore_monsters, large_hull, m_hEnemy->edict(), &tr );

			if (tr.flFraction > 0.5)
				vecPos = tr.vecEndPos;

			m_SaveVelocity = m_SaveVelocity * 0.8 + 0.2 * (vecPos - pev->origin).Normalize() * m_flightSpeed;

			
			if (HasConditions( bits_COND_ENEMY_FACING_ME ) && m_hEnemy->FVisible( this ))
			{
				m_flNextAlert -= 0.1;

				if (m_idealDist < m_flMaxDist)
				{
					m_idealDist += 4;
				}
				if (m_flightSpeed > m_flMinSpeed)
				{
					m_flightSpeed -= 2;
				}
				else if (m_flightSpeed < m_flMinSpeed)
				{
					m_flightSpeed += 2;
				}
				if (m_flMinSpeed < m_flMaxSpeed)
				{
					m_flMinSpeed += 0.5;
				}
			}
			else 
			{
				m_flNextAlert += 0.1;

				if (m_idealDist > 128)
				{
					m_idealDist -= 4;
				}
				if (m_flightSpeed < m_flMaxSpeed)
				{
					m_flightSpeed += 4;
				}
			}
		
		}
		else
		{
			m_flNextAlert = gpGlobals->time + 0.2;
		}

		if (m_flNextAlert < gpGlobals->time)
		{
			AlertSound( );
			m_flNextAlert = gpGlobals->time + RANDOM_FLOAT( 3, 5 );
		}

		break;
	case TASK_MYNOK_FLY:
		if (m_fSequenceFinished)
		{
			TaskComplete( );
		}
		break;
	

	default: 
		CFlyingMonster :: RunTask ( pTask );
		break;
	}
}


//=========================================================
// GetSchedule - Decides which type of schedule best suits
// the monster's current state and conditions. Then calls
// monster's member function to get a pointer to a schedule
// of the proper type.
//=========================================================
Schedule_t *CMynok :: GetSchedule ( void )
{
	switch	( m_MonsterState )
	{
	case MONSTERSTATE_IDLE:
		m_flightSpeed = 80;
		return GetScheduleOfType( SCHED_IDLE_WALK );

	case MONSTERSTATE_ALERT:
		m_flightSpeed = 150;
		return GetScheduleOfType( SCHED_IDLE_WALK );

	case MONSTERSTATE_COMBAT:
		
		if ( pev->iuser4 == 2 )//on fuit si on brle
				{
					return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN );
				}
		
		m_flMaxSpeed = 400;
		{
			if ( HasConditions ( bits_COND_CAN_MELEE_ATTACK1 ) )
			{
				//AttackSound();// this is a total hack. Should be part of the schedule
				return GetScheduleOfType ( SCHED_MELEE_ATTACK1 );
			}

			if ( HasConditions( bits_COND_CAN_RANGE_ATTACK1 ) )
			{
			//ALERT( at_console, "Range Attack\n" );

			return GetScheduleOfType( SCHED_CHASE_ENEMY );
			}

			if ( HasConditions( bits_COND_HEAVY_DAMAGE ) )
			{
			m_bOnAttack = TRUE;
			}

			if ( pev->health < pev->max_health - 20 )
			{
			m_bOnAttack = TRUE;
			}
		}
		return GetScheduleOfType( SCHED_STANDOFF );
	}

	return CFlyingMonster :: GetSchedule();
}



//=========================================================
//=========================================================
Schedule_t* CMynok :: GetScheduleOfType ( int Type ) 
{
	
	switch	( Type )
	{
	case SCHED_IDLE_WALK:
		return slFlyAround;

	case SCHED_STANDOFF:
		return slCircleEnemy;
	
	case SCHED_FAIL:
		return slFlyAgitated;

	
		
	
	case SCHED_CHASE_ENEMY:
		AttackSound( );

	}

	return CBaseMonster :: GetScheduleOfType( Type );
}





//=========================================================
// CheckRangeAttack1  
//
//=========================================================
BOOL CMynok :: CheckRangeAttack1 ( float flDot, float flDist )
{

	if ( gpGlobals->time < m_nextRangeCheck )
	{
		return FALSE;
	}


	if ( flDot > -0.7 && (m_bOnAttack || ( flDist <= 192 && m_idealDist <= 192)))
	{
		
		m_nextRangeCheck = gpGlobals->time + 8;

		return TRUE;
	}

	return FALSE;
}


BOOL CMynok :: CheckMeleeAttack1 ( float flDot, float flDist )
{

	if ( gpGlobals->time < m_flNextAttack )
	{
		return FALSE;
	}
	
	if ( gpGlobals->time < m_flNextAttackCheck )
	{
		return m_fCanAttack;
	}
	
	/*if ( flDot >= 0.7 && m_flEnemyTouched > gpGlobals->time - 0.2 )
	{
		return TRUE;
	}
	*/
	if ( HasConditions ( bits_COND_SEE_ENEMY ) && flDist <= MYNOK_MELEE_DIST && flDot >= 0.7 && m_hEnemy != NULL )
	{
		//ALERT ( at_console, "c'est la melee\n" ) ;
		m_flNextAttackCheck = gpGlobals->time + RANDOM_FLOAT( 2, 5 );
		m_fCanAttack = TRUE;
		m_flNextAttack = gpGlobals->time + 4;
		return m_fCanAttack;

	}
	m_flNextAttackCheck = gpGlobals->time + 0.1;// don't check for half second if this check wasn't successful
	m_fCanAttack = FALSE;
	return m_fCanAttack;
	
}


float CMynok::VectorToPitch( const Vector &vec )
{
	float pitch;
	if (vec.z == 0 && vec.x == 0)
		pitch = 0;
	else
	{
		pitch = (int) (atan2(vec.z, sqrt(vec.x*vec.x+vec.y*vec.y)) * 180 / M_PI);
		if (pitch < 0)
			pitch += 360;
	}
	return pitch;
}


void CMynok :: Move ( float flInterval ) 
{

	CFlyingMonster::Move( flInterval );

}

float CMynok::FlPitchDiff( void )
{
	float	flPitchDiff;
	float	flCurrentPitch;

	flCurrentPitch = UTIL_AngleMod( pev->angles.z );

	if ( flCurrentPitch == pev->idealpitch )
	{
		return 0;
	}

	flPitchDiff = pev->idealpitch - flCurrentPitch;

	if ( pev->idealpitch > flCurrentPitch )
	{
		if (flPitchDiff >= 180)
			flPitchDiff = flPitchDiff - 360;
	}
	else 
	{
		if (flPitchDiff <= -180)
			flPitchDiff = flPitchDiff + 360;
	}
	return flPitchDiff;
}

float CMynok :: ChangePitch( int speed )
{
	if ( pev->movetype == MOVETYPE_FLY )
	{
		float diff = FlPitchDiff();
		float target = 0;
		if ( m_IdealActivity != GetStoppedActivity() )
		{
			if (diff < -20)
				target = 45;
			else if (diff > 20)
				target = -45;
		}
		pev->angles.x = UTIL_Approach(target, pev->angles.x, 220.0 * 0.1 );
	}
	return 0;
}


float CMynok::ChangeYaw( int speed )
{
	if ( pev->movetype == MOVETYPE_FLY )
	{
		float diff = FlYawDiff();
		float target = 0;

		if ( m_IdealActivity != GetStoppedActivity() )
		{
			if ( diff < -20 )
				target = 20;
			else if ( diff > 20 )
				target = -20;
		}
		pev->angles.z = UTIL_Approach( target, pev->angles.z, 220.0 * 0.1 );
	}
	return CFlyingMonster::ChangeYaw( speed );
}


Activity CMynok:: GetStoppedActivity( void )
{ 
	if ( pev->movetype != MOVETYPE_FLY )		
		return ACT_IDLE;
	return ACT_WALK;
}


void CMynok::MoveExecute( CBaseEntity *pTargetEnt, const Vector &vecDir, float flInterval )
{
	m_SaveVelocity = vecDir * m_flightSpeed;
}


void CMynok::MonsterThink ( void )
{
	CFlyingMonster::MonsterThink( );

	if (pev->deadflag == DEAD_NO)
	{
		if (m_MonsterState != MONSTERSTATE_SCRIPT)
		{
			Fly( );

		}
	}
}


void CMynok :: Stop( void ) 
{
	if (!m_bOnAttack)
		m_flightSpeed = 80.0;
}


void CMynok::Fly( )
{
	int retValue = 0;

	Vector start = pev->origin;

	Vector Angles;
	Vector Forward, Right, Up;

	if (FBitSet( pev->flags, FL_ONGROUND))
	{
		pev->angles.x = 0;
		pev->angles.y += RANDOM_FLOAT( -45, 45 );
		ClearBits( pev->flags, FL_ONGROUND );

		Angles = Vector( -pev->angles.x, pev->angles.y, pev->angles.z );
		UTIL_MakeVectorsPrivate(Angles, Forward, Right, Up);

		pev->velocity = Forward * 200 + Up * 200;

		return;
	}

	if (m_bOnAttack && m_flightSpeed < m_flMaxSpeed)
	{
		m_flightSpeed += 40;
	}
	if (m_flightSpeed < 180)
	{
		if (m_IdealActivity == ACT_RUN)
			SetActivity( ACT_WALK );
		if (m_IdealActivity == ACT_WALK)
			pev->framerate = m_flightSpeed / 150.0;
		if ( pev->framerate < 1 )
		{
			pev->framerate = 1;
		}
		 //ALERT( at_console, "walk %.2f\n", pev->framerate );
	}
	else
	{
		if (m_IdealActivity == ACT_WALK)
			SetActivity( ACT_RUN );
		if (m_IdealActivity == ACT_RUN)
			pev->framerate = m_flightSpeed / 150.0;
		if ( pev->framerate < 1 )
		{
			pev->framerate = 1;
		}
		 //ALERT( at_console, "run  %.2f\n", pev->framerate );
	}

#define PROBE_LENGTH 150
	Angles = UTIL_VecToAngles( m_SaveVelocity );
	Angles.x = -Angles.x;
	UTIL_MakeVectorsPrivate(Angles, Forward, Right, Up);

	Vector f, u, l, r, d;
	f = DoProbe(start + PROBE_LENGTH   * Forward);
	r = DoProbe(start + PROBE_LENGTH/3 * Forward+Right);
	l = DoProbe(start + PROBE_LENGTH/3 * Forward-Right);
	u = DoProbe(start + PROBE_LENGTH/3 * Forward+Up);
	d = DoProbe(start + PROBE_LENGTH/3 * Forward-Up);

	Vector SteeringVector = f+r+l+u+d;
	m_SaveVelocity = (m_SaveVelocity + SteeringVector/2).Normalize();

	Angles = Vector( -pev->angles.x, pev->angles.y, pev->angles.z );
	UTIL_MakeVectorsPrivate(Angles, Forward, Right, Up);
	// ALERT( at_console, "%f : %f\n", Angles.x, Forward.z );

	float flDot = DotProduct( Forward, m_SaveVelocity );
	if (flDot > 0.5)
		pev->velocity = m_SaveVelocity = m_SaveVelocity * m_flightSpeed;
	else if (flDot > 0)
		pev->velocity = m_SaveVelocity = m_SaveVelocity * m_flightSpeed * (flDot + 0.5);
	else
		pev->velocity = m_SaveVelocity = m_SaveVelocity * 80;

	// ALERT( at_console, "%.0f %.0f\n", m_flightSpeed, pev->velocity.Length() );
	// ALERT( at_console, "Steer %f %f %f\n", SteeringVector.x, SteeringVector.y, SteeringVector.z );
	// ALERT( at_console, "speed %f\n", m_flightSpeed );
	
	Angles = UTIL_VecToAngles( m_SaveVelocity );

	// Smooth Pitch
	//
	if (Angles.x > 180)
		Angles.x = Angles.x - 360;
	pev->angles.x = UTIL_Approach(Angles.x, pev->angles.x, 50 * 0.1 );
	if (pev->angles.x < -80) pev->angles.x = -80;
	if (pev->angles.x >  80) pev->angles.x =  80;

	// Smooth Yaw and generate Roll
	//
	float turn = 360;
	// ALERT( at_console, "Y %.0f %.0f\n", Angles.y, pev->angles.y );

	if (fabs(Angles.y - pev->angles.y) < fabs(turn))
	{
		turn = Angles.y - pev->angles.y;
	}
	if (fabs(Angles.y - pev->angles.y + 360) < fabs(turn))
	{
		turn = Angles.y - pev->angles.y + 360;
	}
	if (fabs(Angles.y - pev->angles.y - 360) < fabs(turn))
	{
		turn = Angles.y - pev->angles.y - 360;
	}

	float speed = m_flightSpeed * 0.1;

	// ALERT( at_console, "speed %.0f %f\n", turn, speed );
	if (fabs(turn) > speed)
	{
		if (turn < 0.0)
		{
			turn = -speed;
		}
		else
		{
			turn = speed;
		}
	}
	pev->angles.y += turn;
	pev->angles.z -= turn;
	pev->angles.y = fmod((pev->angles.y + 360.0), 360.0);

	// Roll Smoothing
	//
	turn = 360;
	if (fabs(Angles.z - pev->angles.z) < fabs(turn))
	{
		turn = Angles.z - pev->angles.z;
	}
	if (fabs(Angles.z - pev->angles.z + 360) < fabs(turn))
	{
		turn = Angles.z - pev->angles.z + 360;
	}
	if (fabs(Angles.z - pev->angles.z - 360) < fabs(turn))
	{
		turn = Angles.z - pev->angles.z - 360;
	}
	speed = m_flightSpeed/2 * 0.1;
	if (fabs(turn) < speed)
	{
		pev->angles.z += turn;
	}
	else
	{
		if (turn < 0.0)
		{
			pev->angles.z -= speed;
		}
		else
		{
			pev->angles.z += speed;
		}
	}
	if (pev->angles.z < -20) pev->angles.z = -20;
	if (pev->angles.z >  20) pev->angles.z =  20;

	UTIL_MakeVectorsPrivate( Vector( -Angles.x, Angles.y, Angles.z ), Forward, Right, Up);
	
}

Vector CMynok::DoProbe(const Vector &Probe)
{
	Vector WallNormal = Vector(0,0,-1); 
	float frac;
	BOOL bBumpedSomething = ProbeZ(pev->origin, Probe, &frac);

	TraceResult tr;
	TRACE_MONSTER_HULL(edict(), pev->origin, Probe, dont_ignore_monsters, edict(), &tr);
	if ( tr.fAllSolid || tr.flFraction < 0.99 )
	{
		if (tr.flFraction < 0.0) tr.flFraction = 0.0;
		if (tr.flFraction > 1.0) tr.flFraction = 1.0;
		if (tr.flFraction < frac)
		{
			frac = tr.flFraction;
			bBumpedSomething = TRUE;
			WallNormal = tr.vecPlaneNormal;
		}
	}

	if (bBumpedSomething && (m_hEnemy == NULL || tr.pHit != m_hEnemy->edict()))
	{
		Vector ProbeDir = Probe - pev->origin;

		Vector NormalToProbeAndWallNormal = CrossProduct(ProbeDir, WallNormal);
		Vector SteeringVector = CrossProduct( NormalToProbeAndWallNormal, ProbeDir);

		float SteeringForce = m_flightSpeed * (1-frac) * (DotProduct(WallNormal.Normalize(), m_SaveVelocity.Normalize()));
		if (SteeringForce < 0.0)
		{
			SteeringForce = -SteeringForce;
		}
		SteeringVector = SteeringForce * SteeringVector.Normalize();
		
		return SteeringVector;
	}
	return Vector(0, 0, 0);
}
