/***
*
*	Code pour l'arme poings du joueur
*	Par Jean-Franois Vuagniaux
*	vuagniaux.jf@com.mcnet.ch
*
****/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"


#define	HANDS_BODYHIT_VOLUME 128
#define	HANDS_WALLHIT_VOLUME 512

class CHands : public CBasePlayerWeapon
{
public:
	void Spawn( void );
	void Precache( void );
	int iItemSlot( void ) { return 1; }
	void EXPORT SwingAgain( void );
	void EXPORT Smack( void );
	int GetItemInfo(ItemInfo *p);

	void PrimaryAttack( void );
	int Swing( int fFirst );
	BOOL Deploy( void );
	void Holster( int skiplocal = 0 );
	int m_iSwing;
	
};
LINK_ENTITY_TO_CLASS( weapon_hands, CHands );



enum gauss_e {
	HANDS_IDLE = 0,
	HANDS_DRAW,
	HANDS_HOLSTER,
	HANDS_ATTACK1HIT,
	HANDS_ATTACK1MISS,
	HANDS_ATTACK2HIT,
	HANDS_ATTACK2MISS,
	HANDS_ATTACK3HIT,
	HANDS_ATTACK3MISS
	
};


void CHands::Spawn( )
{
	Precache( );
	m_iId = WEAPON_HANDS;
	SET_MODEL(ENT(pev), "models/w_hands.mdl");
	m_iClip = -1;

	FallInit();// get ready to fall down.
}


void CHands::Precache( void )
{
	PRECACHE_MODEL("models/v_hands.mdl");
	PRECACHE_MODEL("models/w_hands.mdl");
	PRECACHE_MODEL("models/p_hands.mdl");
	PRECACHE_SOUND("weapons/punchwall1.wav");
	PRECACHE_SOUND("weapons/punchwall2.wav");
	PRECACHE_SOUND("weapons/punch1.wav");
	PRECACHE_SOUND("weapons/punch2.wav");
	PRECACHE_SOUND("weapons/punch3.wav");
	PRECACHE_SOUND("weapons/punch_miss1.wav");
	PRECACHE_SOUND("weapons/punch_miss2.wav");
	
}

int CHands::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = NULL;
	p->iMaxAmmo1 = -1;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 0;
	p->iPosition = 1;
	p->iId = WEAPON_HANDS;
	p->iWeight = HANDS_WEIGHT;
	return 1;
}



BOOL CHands::Deploy( )
{
	return DefaultDeploy( "models/v_hands.mdl", "models/p_hands.mdl", HANDS_DRAW, "hands" );
}

void CHands::Holster( int skiplocal /* = 0 */ )
{
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;
	SendWeaponAnim( HANDS_HOLSTER );
}

// calcul de la collision ventuelle  l'aide d'un volume virtuel
void PoingsHullIntersection( const Vector &vecSrc, TraceResult &tr, float *mins, float *maxs, edict_t *pEntity )
{
	int			i, j, k;
	float		distance;
	float		*minmaxs[2] = {mins, maxs};
	TraceResult tmpTrace;
	Vector		vecHullEnd = tr.vecEndPos;
	Vector		vecEnd;

	distance = 1e6f;

	vecHullEnd = vecSrc + ((vecHullEnd - vecSrc)*2);
	UTIL_TraceLine( vecSrc, vecHullEnd, dont_ignore_monsters, pEntity, &tmpTrace );
	if ( tmpTrace.flFraction < 1.0 )
	{
		tr = tmpTrace;
		return;
	}

	for ( i = 0; i < 2; i++ )
	{
		for ( j = 0; j < 2; j++ )
		{
			for ( k = 0; k < 2; k++ )
			{
				vecEnd.x = vecHullEnd.x + minmaxs[i][0];
				vecEnd.y = vecHullEnd.y + minmaxs[j][1];
				vecEnd.z = vecHullEnd.z + minmaxs[k][2];

				UTIL_TraceLine( vecSrc, vecEnd, dont_ignore_monsters, pEntity, &tmpTrace );
				if ( tmpTrace.flFraction < 1.0 )
				{
					float thisDistance = (tmpTrace.vecEndPos - vecSrc).Length();
					if ( thisDistance < distance )
					{
						tr = tmpTrace;
						distance = thisDistance;
					}
				}
			}
		}
	}
}


void CHands::PrimaryAttack()
{
	if (! Swing( 1 ))
	{
		SetThink( SwingAgain );
		pev->nextthink = gpGlobals->time + 0.1;
	}
}

/*
void CHands::Smack( )
{
	DecalGunshot( &m_trHit, BULLET_PLAYER_CROWBAR );
}

*/

void CHands::SwingAgain( void )
{
	Swing( 0 );
}


int CHands::Swing( int fFirst )
{
	int fDidHit = FALSE;

	TraceResult tr;

	UTIL_MakeVectors (m_pPlayer->pev->v_angle);
	Vector vecSrc	= m_pPlayer->GetGunPosition( );
	Vector vecEnd	= vecSrc + gpGlobals->v_forward * 24;

	UTIL_TraceLine( vecSrc, vecEnd, dont_ignore_monsters, ENT( m_pPlayer->pev ), &tr );

	if ( tr.flFraction >= 1.0 )
	{
		UTIL_TraceHull( vecSrc, vecEnd, dont_ignore_monsters, head_hull, ENT( m_pPlayer->pev ), &tr );
		if ( tr.flFraction < 1.0 )
		{
			// Calculate the point of intersection of the line (or hull) and the object we hit
			// This is and approximation of the "best" intersection
			CBaseEntity *pHit = CBaseEntity::Instance( tr.pHit );
			if ( !pHit || pHit->IsBSPModel() )
				PoingsHullIntersection( vecSrc, tr, VEC_DUCK_HULL_MIN, VEC_DUCK_HULL_MAX, m_pPlayer->edict() );
			vecEnd = tr.vecEndPos;	// This is the point on the actual surface (the hull could have hit space)
		}
	}

	if ( tr.flFraction >= 1.0 )
	{
		if (fFirst)
		{
			// miss
			switch( (m_iSwing++) % 3 )
			{
			case 0:
				SendWeaponAnim( HANDS_ATTACK1MISS ); break;
			case 1:
				SendWeaponAnim( HANDS_ATTACK2MISS ); break;
			case 2:
				SendWeaponAnim( HANDS_ATTACK3MISS ); break;
			}
			m_flNextPrimaryAttack = gpGlobals->time + 1.0;
			// play wiff or swish sound
			//EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_WEAPON, "zombie/claw_miss1.wav", 1, ATTN_NORM, 0, 94 + RANDOM_LONG(0,0xF));

			switch( RANDOM_LONG(0,1) )
			{
			case 0:
				
				EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/punch_miss1.wav", 1, ATTN_NORM, 0, 94 + RANDOM_LONG(0,0xF)); 
				break;
			case 1:
				
				EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/punch_miss2.wav", 1, ATTN_NORM, 0, 94 + RANDOM_LONG(0,0xF)); 
				break;
			}
			// player "shoot" animation
			m_pPlayer->SetAnimation( PLAYER_ATTACK1 );
		}
	}
	else
	{
		// hit
		fDidHit = TRUE;

		CBaseEntity *pEntity = CBaseEntity::Instance(tr.pHit);

		switch( ((m_iSwing++) % 2) + 1 )
		{
		case 0:
			SendWeaponAnim( HANDS_ATTACK1HIT ); break;
		case 1:
			SendWeaponAnim( HANDS_ATTACK2HIT ); break;
		case 2:
			SendWeaponAnim( HANDS_ATTACK3HIT ); break;
		}

		// player "shoot" animation
		m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

		ClearMultiDamage( );
		if ( (m_flNextPrimaryAttack + 1 < gpGlobals->time) || g_pGameRules->IsMultiplayer() )
		{
			// first swing does full damage
			pEntity->TakeDamage(m_pPlayer->pev, m_pPlayer->pev, (gSkillData.officerDmgPunch * 1.5), DMG_CLUB ); 
		}
		else
		{
			// subsequent swings do half
			pEntity->TakeDamage(m_pPlayer->pev, m_pPlayer->pev, gSkillData.officerDmgPunch, DMG_CLUB ); 
		}	
		ApplyMultiDamage( m_pPlayer->pev, m_pPlayer->pev );

		m_flNextPrimaryAttack = gpGlobals->time + 1;

		// play thwack, smack, or dong sound
		float flVol = 1.0;
		int fHitWorld = TRUE;

		if (pEntity)
		{
			if (pEntity->Classify() != CLASS_NONE && pEntity->Classify() != CLASS_MACHINE)
			{
				// play thwack or smack sound
				switch( RANDOM_LONG(0,2) )
				{
				case 0:
					EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/punch1.wav", 1, ATTN_NORM); break;
				case 1:
					EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/punch2.wav", 1, ATTN_NORM); break;
				case 2:
					EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/punch3.wav", 1, ATTN_NORM); break;
				}
				m_pPlayer->m_iWeaponVolume = HANDS_BODYHIT_VOLUME;
				if (!pEntity->IsAlive() )
					return TRUE;
				else
					flVol = 0.1;

				fHitWorld = FALSE;
			}
		}

		// play texture hit sound
		// UNDONE: Calculate the correct point of intersection when we hit with the hull instead of the line

		if (fHitWorld)
		{
			float fvolbar = TEXTURETYPE_PlaySound(&tr, vecSrc, vecSrc + (vecEnd-vecSrc)*2, BULLET_PLAYER_CROWBAR);

			if ( g_pGameRules->IsMultiplayer() )
			{
				// override the volume here, cause we don't play texture sounds in multiplayer, 
				// and fvolbar is going to be 0 from the above call.

				fvolbar = 1;
			}

			
			switch( RANDOM_LONG(0,1) )
			{
			case 0:
				
				EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_ITEM, "weapons/punchwall1.wav", fvolbar, ATTN_NORM, 0, 98 + RANDOM_LONG(0,3)); 
				break;
			case 1:
				
				EMIT_SOUND_DYN(ENT(m_pPlayer->pev), CHAN_ITEM, "weapons/punchwall2.wav", fvolbar, ATTN_NORM, 0, 98 + RANDOM_LONG(0,3)); 
				break;
			}
		}


		m_pPlayer->m_iWeaponVolume = flVol * HANDS_WALLHIT_VOLUME;
	}
	return fDidHit;
}



