/*
Fichier de code pour le Railgun

Par Jean-Franois Vuagniaux

vuagniaux.jf@com.mcnet.ch

*/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"


enum rail_e {  // animations
	RAIL_IDLE = 0,
	RAIL_FIDGET,
	RAIL_RELOAD,	
	RAIL_FIRE,		
	RAIL_HOLSTER,	
	RAIL_DRAW,		
};


class CRLaserSpot : public CBaseEntity // point rouge de guidage laser
{
	void Spawn( void );
	void Precache( void );

	int	ObjectCaps( void ) { return FCAP_DONT_SAVE; }

public:
	void Suspend( float flSuspendTime ); // pour un temps de pause
	void EXPORT Revive( void );			// ravive le point
	
	static CRLaserSpot *CreateSpot( void );
};
LINK_ENTITY_TO_CLASS( rlaser_spot, CRLaserSpot );


class CRail : public CBasePlayerWeapon  // railgun lui-mme
{
public:
	int		Save( CSave &save );
	int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	void Spawn( void );
	void Precache( void );
	void Reload( void );
	int iItemSlot( void ) { return 4; }
	int GetItemInfo(ItemInfo *p);
	int AddToPlayer( CBasePlayer *pPlayer );

	BOOL Deploy( void );
	BOOL CanHolster( void );
	void Holster( int skiplocal = 0  );

	void PrimaryAttack( void );
	void SecondaryAttack( void );
	void WeaponIdle( void );

	void UpdateSpot( void );
	BOOL ShouldWeaponIdle( void ) { return TRUE; };
	Vector GetGunPosition( void );

	CRLaserSpot *m_RSpot;
	int m_RSpotActive; // si le point rouge est activ
	int m_RActiveRockets;// combien de projectiles en vol actuellement

};
LINK_ENTITY_TO_CLASS( weapon_railgun, CRail );

TYPEDESCRIPTION	CRail::m_SaveData[] = 
{
	DEFINE_FIELD( CRail, m_RSpotActive, FIELD_INTEGER ),
	DEFINE_FIELD( CRail, m_RActiveRockets, FIELD_INTEGER ),
};
IMPLEMENT_SAVERESTORE( CRail, CBasePlayerWeapon );

//=========================================================
//=========================================================
CRLaserSpot *CRLaserSpot::CreateSpot( void )  // gnre un point rouge  partir de la dfinition de classe
{
	CRLaserSpot *RSpot = GetClassPtr( (CRLaserSpot *)NULL );
	RSpot->Spawn();

	RSpot->pev->classname = MAKE_STRING("rlaser_spot");

	return RSpot;
}

//=========================================================
//=========================================================
void CRLaserSpot::Spawn( void ) // cration du point
{
	Precache( );
	pev->movetype = MOVETYPE_NONE;
	pev->solid = SOLID_NOT;

	pev->rendermode = kRenderGlow;
	pev->renderfx = kRenderFxNoDissipation;
	pev->renderamt = 255;

	SET_MODEL(ENT(pev), "sprites/laserdot.spr");
	UTIL_SetOrigin( pev, pev->origin );
};

//=========================================================
// Suspend- rend le point invisible pour un moment 
//=========================================================
void CRLaserSpot::Suspend( float flSuspendTime )
{
	pev->effects |= EF_NODRAW;
	
	SetThink( Revive );
	pev->nextthink = gpGlobals->time + flSuspendTime;
}

//=========================================================
// Revive - ravive un point rendu invisible par suspend.
//=========================================================
void CRLaserSpot::Revive( void )
{
	pev->effects &= ~EF_NODRAW;

	SetThink( NULL );
}

void CRLaserSpot::Precache( void )
{
	PRECACHE_MODEL("sprites/laserdot.spr");
};





class CRailRocket : public CGrenade // le projectile explosif du railgun
{
public:
	int		Save( CSave &save );
	int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];
	void Spawn( void );
	void Precache( void );
	void EXPORT FollowThink( void );
	void EXPORT IgniteThink( void );
	void EXPORT RocketTouch( CBaseEntity *pOther );
	static CRailRocket *CreateRailRocket( Vector vecOrigin, Vector vecAngles, CBaseEntity *pOwner, CRail *pLauncher );

	int m_iRTrail; // trace
	float m_fRIgniteTime; // temps d'allumage
	CRail *m_pLauncher;// pointe en arrire vers qui a envoy le projectile 
};
LINK_ENTITY_TO_CLASS( rail_rocket, CRailRocket );

TYPEDESCRIPTION	CRailRocket::m_SaveData[] = 
{
	DEFINE_FIELD( CRailRocket, m_fRIgniteTime, FIELD_TIME ),
	DEFINE_FIELD( CRailRocket, m_pLauncher, FIELD_CLASSPTR ),
};
IMPLEMENT_SAVERESTORE( CRailRocket, CGrenade );


//=========================================================
// Gnre un projectile  partir de sa dfinition de classe
//=========================================================
CRailRocket *CRailRocket::CreateRailRocket( Vector vecOrigin, Vector vecAngles, CBaseEntity *pOwner, CRail *pLauncher )
{
	CRailRocket *pRRocket = GetClassPtr( (CRailRocket *)NULL );

	UTIL_SetOrigin( pRRocket->pev, vecOrigin );
	pRRocket->pev->angles = vecAngles;
	pRRocket->Spawn();
	pRRocket->SetTouch( CRailRocket::RocketTouch );
	pRRocket->m_pLauncher = pLauncher;// pour se souvenir de quel railgun est partit le coup.
	pRRocket->m_pLauncher->m_RActiveRockets++;// enregistre le projectile comme actif pour ce railgun
	pRRocket->pev->owner = pOwner->edict();

	return pRRocket;
}

//=========================================================
// Cre un projectile
//=========================================================
void CRailRocket :: Spawn( void ) 
{
	Precache( );

	pev->movetype = MOVETYPE_BOUNCE;
	pev->solid = SOLID_BBOX;

	SET_MODEL(ENT(pev), "models/railrocket.mdl");
	UTIL_SetSize(pev, Vector( 0, 0, 0), Vector(0, 0, 0));
	UTIL_SetOrigin( pev, pev->origin );

	pev->classname = MAKE_STRING("rail_rocket");

	SetThink( IgniteThink );
	SetTouch( ExplodeTouch );  // vrifie si une impacte a lieu.

	pev->angles.x -= 5;
	UTIL_MakeVectors( pev->angles );
	pev->angles.x = -(pev->angles.x + 5);

	pev->velocity = gpGlobals->v_forward * 500; // vitesse du projectile
	pev->gravity = 0.2;

	pev->nextthink = gpGlobals->time + 0.4;

	pev->dmg = gSkillData.plrDmgRAIL;  // dommages engendrs par l'explosion
}

//=========================================================
// Gre l'impacte du projectile
//=========================================================
void CRailRocket :: RocketTouch ( CBaseEntity *pOther )
{
	TraceResult tr;

	if ( m_pLauncher )
	{
		// averti le railgun lanceur de l'impacte.
		m_pLauncher->m_RActiveRockets--;
	}
	
	// stope le son de propulsion
	STOP_SOUND( edict(), CHAN_VOICE, "weapons/railfly.wav" );

	UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );

	//pas d'effet si on touche le ciel

	if ( UTIL_IsTextureSky( &tr, pev->origin, pev->origin + pev->velocity * 10 ) )
	{
		SetThink ( SUB_Remove );
		pev->nextthink = gpGlobals->time;

		return;
	}
	// cre l'explosion  l'impacte
	ExplodeTouch( pOther );
}

//=========================================================
//=========================================================
void CRailRocket :: Precache( void )
{
	PRECACHE_MODEL("models/railrocket.mdl");
	m_iRTrail = PRECACHE_MODEL("sprites/smoke.spr"); // trace de la propulsion
	PRECACHE_SOUND ("weapons/railfly.wav");
}

// =================================================
// dmarre le moteur de fuse, cre le sillage et la poursuite du point rouge
// =================================================
void CRailRocket :: IgniteThink( void  )
{
	
	pev->movetype = MOVETYPE_FLY;	// dfini un objet volant
	pev->effects |= EF_LIGHT;		// cre un effet de lumire

	// dmarre le son du moteur de fuse
	EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/railfly.wav", 1, 0.5 );

	// cre un sillage de fume pour le projectile
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );

		WRITE_BYTE( TE_BEAMFOLLOW ); // le sillage suit le projectile
		WRITE_SHORT(entindex());	// entit
		WRITE_SHORT(m_iRTrail );	// modle
		WRITE_BYTE( 40 ); // dure
		WRITE_BYTE( 5 );  // largeur
		WRITE_BYTE( 224 );   // r, g, b
		WRITE_BYTE( 224 );   // r, g, b
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );	// luminosit

	MESSAGE_END();  

	m_fRIgniteTime = gpGlobals->time; // dfini le moment du dpart

	// dmarre la poursuite du point rouge laser
	SetThink( FollowThink );
	pev->nextthink = gpGlobals->time + 0.1; // raligne la course sur le point tout les 0.1 sec
}

// ===================================================
// Gre le suivi du point rouge
// ===================================================
void CRailRocket :: FollowThink( void  ) 
{
	CBaseEntity *pOther = NULL;
	Vector vecTarget;
	Vector vecDir;
	float flDist, flMax, flDot;
	TraceResult tr;

	UTIL_MakeAimVectors( pev->angles );

	vecTarget = gpGlobals->v_forward; // on dfini la cible droit devant
	flMax = 4096;
	
	// Recherche du point rouge  dans un rayon donn.
	while ((pOther = UTIL_FindEntityByClassname( pOther, "rlaser_spot" )) != NULL)
	{
		UTIL_TraceLine ( pev->origin, pOther->pev->origin, dont_ignore_monsters, ENT(pev), &tr );
		// ALERT( at_console, "%f\n", tr.flFraction );
		if (tr.flFraction >= 0.90)
		{
			vecDir = pOther->pev->origin - pev->origin;
			flDist = vecDir.Length( );		// distance
			vecDir = vecDir.Normalize( );	// direction

		// compare la direction avant avec la direction de la cible
			flDot = DotProduct( gpGlobals->v_forward, vecDir );
			if ((flDot > 0) && (flDist * (1 - flDot) < flMax))
			{
				flMax = flDist * (1 - flDot);
				vecTarget = vecDir; // si le point n'est pas trop loign 
			}
		}
	}

	pev->angles = UTIL_VecToAngles( vecTarget );

	// approximation du calcul de l'acclration et du changement de direction
	float fRSpeed = pev->velocity.Length();
	
	if (gpGlobals->time - m_fRIgniteTime < 1.0) // si il y a moins d'une seconde depuis le dpart
	{
		// le projectile acclre
		pev->velocity = pev->velocity * 0.2 + vecTarget * (fRSpeed * 0.8 + 400);
		if (pev->waterlevel == 3)
		{
			// la vitesse est plus lente sous l'eau
			if (pev->velocity.Length() > 300)
			{
				pev->velocity = pev->velocity.Normalize() * 300;
			}

			// cre des bulles sous l'eau
			UTIL_BubbleTrail( pev->origin - pev->velocity * 0.1, pev->origin, 4 );
		} 
		else 
		{
			// limite la vitesse  2000
			if (pev->velocity.Length() > 1750)
			{
				pev->velocity = pev->velocity.Normalize() * 1750;//pour eviter que le sabre la renvoie
			}
		}
	}
	else // si il y a plus d'une seconde depuis le dpart
	{
		if (pev->effects & EF_LIGHT) // si l'effet lumineux est prsent
		{
			pev->effects = 0;	// arrte la luminosit
			STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/railfly.wav" ); // stope le son du moteur
		}
		pev->velocity = pev->velocity * 0.2 + vecTarget * fRSpeed * 0.798;// rduit la vitesse
		
		// si la vitesse tombe en dessous de 1500 le projectile explose
		if (pev->waterlevel == 0 && pev->velocity.Length() < 1500)
		{
			Detonate( );
		}
	}

	pev->nextthink = gpGlobals->time + 0.1;
}


// ==================================================
// Fonctions du railgun lui-mme
// ==================================================



void CRail::Reload( void )
{
	int iRResult;

	if ( m_iClip == 3 )
	{
		// pas de recharge si le chargeur est plein.
		return;
	}
	
	m_flNextPrimaryAttack = gpGlobals->time + 1;

	
	if ( m_RActiveRockets && m_RSpotActive )
	{
		// pas de recharge pendant qu'un projectile est en vol
		
		return;
	}

	if (m_RSpot && m_RSpotActive) // s'il y a un point rouge actif, on le coupe pendant 6 sec
	{
		m_RSpot->Suspend( 6 );
		m_flNextSecondaryAttack = gpGlobals->time + 6;
	}

	//if (m_iClip == 0)	// si le chargeur est vide on recharge
	//{					// ne pas oublier de mettre en prcache les sons de recharge
						// ces sons sont dfinis dans le fichier railgun.qc 
		iRResult = DefaultReload( RAIL_MAX_CLIP, RAIL_RELOAD, 5.5 );
//	}

	if (iRResult)		// si on recharge on repousse l'animation de pause de 3  6 sec
	{
		m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 6, 12 );
	}
}

Vector CRail :: GetGunPosition( )
{
	Vector vecArmPos, vecArmDir;
	GetAttachment( 0, vecArmPos, vecArmDir ); 
	return vecArmPos;
}

void CRail::Spawn( )
{
	Precache( );
	m_iId = WEAPON_RAIL;

	SET_MODEL(ENT(pev), "models/w_railgun.mdl");
	m_RSpotActive = 1;

	pev->iuser3 = 1;//masse

	if ( g_pGameRules->IsMultiplayer() )
	{
		// plus de munitions en multiplayer 
		m_iDefaultAmmo = RAIL_DEFAULT_GIVE * 2;
	}
	else
	{
		m_iDefaultAmmo = RAIL_DEFAULT_GIVE;
	}

	FallInit();// prt  tomber au sol
}


void CRail::Precache( void )
{
	PRECACHE_MODEL("models/w_railgun.mdl");
	PRECACHE_MODEL("models/v_railgun.mdl");
	PRECACHE_MODEL("models/p_railgun.mdl");

	PRECACHE_SOUND("items/railclip.wav");
	PRECACHE_SOUND("weapons/deverouille.wav");
	PRECACHE_SOUND("weapons/clonk.wav");
	PRECACHE_SOUND("weapons/sortie.wav");
	PRECACHE_SOUND("weapons/chute.wav");
	PRECACHE_SOUND("weapons/insertion.wav");
	PRECACHE_SOUND("weapons/verrou.wav");
	PRECACHE_SOUND("weapons/clic.wav");
	PRECACHE_SOUND("weapons/rotclic.wav");

	UTIL_PrecacheOther( "rlaser_spot" );
	UTIL_PrecacheOther( "rail_rocket" );

	PRECACHE_SOUND("weapons/railstart.wav");
	PRECACHE_SOUND("weapons/raillaunch.wav"); // autre son de lancement
}


int CRail::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "railcharge";
	p->iMaxAmmo1 = RAILCHARGE_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = RAIL_MAX_CLIP;
	p->iSlot = 3;
	p->iPosition = 1;
	p->iId = m_iId = WEAPON_RAIL;
	p->iFlags = 0;
	p->iWeight = RAIL_WEIGHT;

	return 1;
}

int CRail::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}



BOOL CRail::Deploy( )	// dploiement de l'arme
{

	return DefaultDeploy( "models/v_railgun.mdl", "models/p_railgun.mdl", RAIL_DRAW, "railgun" );
}




BOOL CRail::CanHolster( void )
{
	if ( m_RSpotActive && m_RActiveRockets )
	{
		// pas possible de ranger l'arme pendant le guidage
		return FALSE;
	}

	return TRUE;
}



void CRail::Holster( int skiplocal )
{
	m_fInReload = FALSE;	// arrte l'animation de recharge

	m_pPlayer->m_flNextAttack = gpGlobals->time + 0.5;

	SendWeaponAnim( RAIL_HOLSTER );
	if (m_RSpot)
	{
		m_RSpot->Killed( NULL, GIB_NEVER );		//supprime le point rouge
		m_RSpot = NULL;
	}
}



void CRail::PrimaryAttack()
{
	if (m_iClip > 0)
	{
		m_pPlayer->m_iWeaponVolume = LOUD_GUN_VOLUME;
		m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;

		SendWeaponAnim( RAIL_FIRE );

		// animation de tir pour le joueur
		m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

		UTIL_MakeVectors( m_pPlayer->pev->v_angle );

		//modifier pour prendre en compte le dsalignement de la man par rapport  la ligne de vise 
		//car le coup porte trop bas  droite:
		//
		//tracer une ligne avec le vecteur: vector aiming et trouver le point d'impacte
		//corriger le vecteur de tir en consquence.
	
		//Vector vecSrc	 = m_pPlayer->GetGunPosition( ) ; //point de vise du HUD
		//Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );
	
		//TraceResult tr;
		//UTIL_TraceLine ( vecSrc, vecSrc + vecAiming * 8192, dont_ignore_monsters, ENT(m_pPlayer->pev), &tr ); //ligne de vise

		//vecSrc = vecSrc + gpGlobals->v_forward * 3 + gpGlobals->v_right * 5 + gpGlobals->v_up * -13;//position du canon
	
		//vecAiming = tr.vecEndPos - vecSrc;
	
		//vecAiming = vecAiming.Normalize();

		// fin de correction
		
		// position de dpart du coup
		Vector vecSrc = m_pPlayer->GetGunPosition( ) + gpGlobals->v_forward * 20 + gpGlobals->v_right * 4 + gpGlobals->v_up * -13;
		
		// cration d'un projectile au dpart du coup
		CRailRocket *pRRocket = CRailRocket::CreateRailRocket( vecSrc, m_pPlayer->pev->v_angle, m_pPlayer, this );

		UTIL_MakeVectors( m_pPlayer->pev->v_angle );// RailRocket::Create agit sur les globals, alors on le recalcule.
		pRRocket->pev->velocity = pRRocket->pev->velocity + gpGlobals->v_forward * DotProduct( m_pPlayer->pev->velocity, gpGlobals->v_forward );

		// sons combins du dpart du coup
		EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/railstart.wav", 0.9, ATTN_NORM );
		EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_ITEM, "weapons/raillaunch.wav", 0.7, ATTN_NORM );

		m_iClip--; 
		// retire une unit au chargeur
		//m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;
		
		m_flNextPrimaryAttack = gpGlobals->time + 0.5;
		m_flTimeWeaponIdle = gpGlobals->time + 1.5;
		
		// bouscule le joueur de 3
		m_pPlayer->pev->punchangle.x -= 3;
	}
	else
	{
		PlayEmptySound( );
		m_flNextPrimaryAttack = gpGlobals->time + 0.5;
	}
	UpdateSpot( );
}

// ============================================================
// sert  afficher ou retirer le point rouge de guidage et 
//  activer/dsactiver le guidage
// ============================================================

void CRail::SecondaryAttack() 
{
	m_RSpotActive = ! m_RSpotActive;

	if (!m_RSpotActive && m_RSpot)
	{
		m_RSpot->Killed( NULL, GIB_NORMAL );
		m_RSpot = NULL;
	}

	m_flNextSecondaryAttack = gpGlobals->time + 0.2;
}


void CRail::WeaponIdle( void )
{
	UpdateSpot( );

	ResetEmptySound( );

	if (m_flTimeWeaponIdle > gpGlobals->time)
		return;

	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		int iAnim;
		float flRand = RANDOM_FLOAT(0, 1);
		if (flRand <= 0.5 )
		{
			iAnim = RAIL_IDLE;

			m_flTimeWeaponIdle = gpGlobals->time + 15.0;
		}
		else
		{
			
			iAnim = RAIL_FIDGET;

			m_flTimeWeaponIdle = gpGlobals->time + 5.0;
		}

		SendWeaponAnim( iAnim );
	}
	else
	{
		m_flTimeWeaponIdle = gpGlobals->time + 1;
	}
}



void CRail::UpdateSpot( void )  // met  jour le point rouge sur le point de vise
{
	if (m_RSpotActive)
	{
		if (!m_RSpot)
		{
			m_RSpot = CRLaserSpot::CreateSpot();
		}

		UTIL_MakeVectors( m_pPlayer->pev->v_angle );
		Vector vecSrc = m_pPlayer->GetGunPosition( );;
		Vector vecAiming = gpGlobals->v_forward;

		TraceResult tr;
		UTIL_TraceLine ( vecSrc, vecSrc + vecAiming * 8192, dont_ignore_monsters, ENT(m_pPlayer->pev), &tr );
		
		UTIL_SetOrigin( m_RSpot->pev, tr.vecEndPos );
	}
}


class CRailAmmo : public CBasePlayerAmmo  // Railcharges, munitions pour le Railgun
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_railcharge.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_railcharge.mdl");
		PRECACHE_SOUND("items/railclip.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		int iGive;

		if ( g_pGameRules->IsMultiplayer() )
		{
			// double la munitions en multiplayer
			iGive = AMMO_RAILCLIP_GIVE * 2;
		}
		else
		{
			iGive = AMMO_RAILCLIP_GIVE;
		}

		if (pOther->GiveAmmo( iGive, "railcharge", RAILCHARGE_MAX_CARRY ) != -1)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/railclip.wav", 1, ATTN_NORM);
			return TRUE;
		}
		return FALSE;
	}
};
LINK_ENTITY_TO_CLASS( ammo_railcharges, CRailAmmo );

