///////////////////////////////////////////////////////////////////
//
// Code pour les seqcharges 
//
// Charges explosives que l'on peut placer sur les murs et sur le sol
// Elles peuvent tre dsactive et rcupres
//
// Par Jean-Franois Vuagniaux
// vuagniaux.jf@com.mcnet.ch
//
//////////////////////////////////////////////////////////////////

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "effects.h"
#include "gamerules.h"
#include "soundent.h"

#define	SEQCHARGE_PRIMARY_VOLUME		450
	

BOOL	m_radical;

enum tripmine_e {
	SEQCHARGE_IDLE1 = 0,
	SEQCHARGE_IDLE2,
	SEQCHARGE_FIDGET,
	SEQCHARGE_HOLSTER,
	SEQCHARGE_DRAW,
};



class CSeqchargeGrenade : public CGrenade
{
	void Spawn( void );
	void Precache( void );

	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	void	KeyValue( KeyValueData *pkvd );

	static	TYPEDESCRIPTION m_SaveData[];

	int TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType );
	
	void EXPORT ScanThink( void );
	void EXPORT PowerupThink( void );
	void EXPORT DelayDeathThink( void );
	void Killed( entvars_t *pevAttacker, int iGib );

	float		m_flPowerUp;
	Vector		m_vecDir;
	Vector		m_vecEnd;
	BOOL		m_ramasse;
	float		m_flNextSound;
	float		m_flNextProp;
	float		m_flCountDown;
	BOOL		m_FromPlayer;
	BOOL		m_MapRadical;

	BOOL		m_PasDeDecal;
	
	EHANDLE		m_hOwner;
	Vector		m_posOwner;
	Vector		m_angleOwner;
	edict_t		*m_pRealOwner;// tracelines don't hit PEV->OWNER, which means a player couldn't detonate his own trip mine, so we store the owner here.
};

LINK_ENTITY_TO_CLASS( monster_seqcharge, CSeqchargeGrenade );

TYPEDESCRIPTION	CSeqchargeGrenade::m_SaveData[] = 
{
	DEFINE_FIELD( CSeqchargeGrenade, m_flPowerUp, FIELD_TIME ),
	DEFINE_FIELD( CSeqchargeGrenade, m_vecDir, FIELD_VECTOR ),
	DEFINE_FIELD( CSeqchargeGrenade, m_vecEnd, FIELD_POSITION_VECTOR ),
	DEFINE_FIELD( CSeqchargeGrenade, m_hOwner, FIELD_EHANDLE ),
	DEFINE_FIELD( CSeqchargeGrenade, m_posOwner, FIELD_POSITION_VECTOR ),
	DEFINE_FIELD( CSeqchargeGrenade, m_angleOwner, FIELD_VECTOR ),
	DEFINE_FIELD( CSeqchargeGrenade, m_pRealOwner, FIELD_EDICT ),
	DEFINE_FIELD( CSeqchargeGrenade, m_flNextSound, FIELD_TIME ),
	DEFINE_FIELD( CSeqchargeGrenade, m_flNextProp, FIELD_TIME ),
	DEFINE_FIELD( CSeqchargeGrenade, m_flCountDown, FIELD_TIME ),
	DEFINE_FIELD( CSeqchargeGrenade, m_ramasse, FIELD_BOOLEAN ),
	DEFINE_FIELD( CSeqchargeGrenade, m_FromPlayer, FIELD_BOOLEAN ),
	DEFINE_FIELD( CSeqchargeGrenade, m_MapRadical, FIELD_BOOLEAN ),

};

IMPLEMENT_SAVERESTORE(CSeqchargeGrenade,CGrenade);

void CSeqchargeGrenade::KeyValue( KeyValueData *pkvd )
{
	
	if (FStrEq(pkvd->szKeyName, "radical"))
	{
		m_MapRadical = atof(pkvd->szValue);
		pkvd->fHandled = TRUE;

	}
	else 
		CBaseMonster::KeyValue( pkvd );
}


void CSeqchargeGrenade :: Spawn( void )
{
	Precache( );

	pev->movetype = MOVETYPE_FLY;
	pev->solid = SOLID_NOT;

	SET_MODEL(ENT(pev), "models/w_seqcharge.mdl");
	pev->skin = 0;
	pev->frame = 0;

	pev->iuser2 = 0;
	pev->sequence = 0;
	ResetSequenceInfo( );
	pev->framerate = 0;

	
	
	UTIL_SetSize(pev, Vector( -8, -8, -8), Vector(8, 8, 8));
	UTIL_SetOrigin( pev, pev->origin );


	CBaseEntity *pOwnerMine = NULL;

	if (pev->spawnflags & 1)
	{
		// temps de mise en marche rapide(dangereux)
		m_flPowerUp = gpGlobals->time + 1.0;
	}
	else
	{
		// temps de mise en marche normal
		m_flPowerUp = gpGlobals->time + 2.5;
	}

	SetThink( PowerupThink );//dmarre la minuterie de mise en marche
	pev->nextthink = gpGlobals->time + 0.2;
	
	pev->takedamage = DAMAGE_YES;//peut tre dtruite
	pev->dmg = gSkillData.plrDmgTripmine;
	pev->health = 1; // sant minimale


	m_FromPlayer = 0;

	pev->iuser1 = 0;

	if (pev->owner != NULL)//s'il existe un propritaire de la mine
	{
		// son de mise en marche
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/activation.wav", 1.0, ATTN_NORM );
		EMIT_SOUND( ENT(pev), CHAN_BODY, "weapons/alertemine.wav", 0.2, ATTN_NORM ); 

		m_pRealOwner = pev->owner;	// dfini le vrai dtenteur pour viter que si on utilise TraceLine() 
									// le poseur ne dtonne pas la mine
	
		pOwnerMine = CBaseEntity::Instance( pev->owner );//va chercher l'entit propritaire

		if ( pOwnerMine->IsPlayer() )
		{
			m_FromPlayer = 1;
			
			pev->iuser1 = 1;
		}
	
	}

	UTIL_MakeAimVectors( pev->angles );

	m_ramasse = 0;
	m_vecDir = gpGlobals->v_forward;
	m_vecEnd = pev->origin + m_vecDir * 2048; // dfini l'orientation de la mine

	m_flNextSound = 0;
	m_flNextProp = 0;
	m_flCountDown = 0;

	
}


void CSeqchargeGrenade :: Precache( void )
{
	PRECACHE_MODEL("models/w_seqcharge.mdl");
	PRECACHE_SOUND("weapons/activation.wav");
	PRECACHE_SOUND("weapons/mineprete.wav");
	PRECACHE_SOUND("weapons/alertemine.wav");
	PRECACHE_SOUND("weapons/friendmine.wav");
}


/////////////////////////////////////////////////////////////////////////
//
//scanner qui va dtecter les entits dans un certain rayon de proximit
//
/////////////////////////////////////////////////////////////////////////

void CSeqchargeGrenade :: ScanThink( void  )
{
	float m_flRayon = 128;
	Vector vecStart = pev->origin;
	BOOL m_alert = 0;
	BOOL m_proprio = 0;
	int m_entite = 0;
	TraceResult tr;

	CBaseEntity *pEntity = NULL;
	

	//analyse toutes les entits prsentes, si le joueur qui a pos la mine est dans le rayon de dtection,
	//la mine n'explose pas et peut tre rcupre par son propritaire, sinon le compte  rebours est dclench,
	//le joueur a alors un temps minimal pour dsactiver la mine

	while ((pEntity = UTIL_FindEntityInSphere( pEntity, vecStart, m_flRayon)) != NULL)
	{
		if (  pEntity->IsPlayer() || FBitSet ( pEntity->pev->flags, FL_MONSTER ) )
		{			
			//on va tracer une ligne pour voir si l'entit n'est pas cache derrire une cloison
				
			m_entite++;

			UTIL_TraceLine( vecStart, pEntity->BodyTarget( vecStart ), dont_ignore_monsters, ENT( pev ), &tr );

			// si le rayon ne touche rien ou l'entit
			if ( tr.pHit == pEntity->edict() ) 
			{
				if ( m_radical )
				{
					m_alert = 1;
				}
				else
				{

					if ( pEntity->IsPlayer() )
					{
						if ( m_FromPlayer || !m_MapRadical )//si c'est le joueur ou si la mine a t pose comme amie du joueur
						{
							m_proprio = 1; //le propritaire est dans le rayon d'action
						}
						else
						{
							m_alert = 1;
						}
					}
					else
					{		
					
						if ( m_FromPlayer || !m_MapRadical )
						{
							m_alert = 1; //le propritaire est dans le rayon d'action
						}
						else
						{
							m_proprio = 1;
						}
					}
				}				
			}			
		}	
	}
	
	
	if ( m_proprio )
	{
		//ALERT( at_console, "PROPRIO\n");
		
		pev->skin = 2;// met la skin d'alerte

		if ( m_flNextProp < gpGlobals->time )
		{
		
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/friendmine.wav", 1.0, ATTN_NORM );

			m_flNextProp = gpGlobals->time + 0.5;
			
			pev->skin = 0;//met la skin de dsactivation

		}
		
		pev->nextthink = gpGlobals->time + 0.1;

		m_ramasse = 1;//on pourra rcuprer la mine en la faisant tomber d'un coup de poing (DAMAGE_CLUB) sinon BOUM!!!

		m_flCountDown = 0;

		

		return;
	}

	if ( m_alert )
	{
		//ALERT( at_console, "ALERTE, %d, TYPE, %d \n", m_entite, m_MapRadical);

		
		if ( m_flNextSound < gpGlobals->time )
		{

			// son d'avertissement si on approche trop prs
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/alertemine.wav", 1.0, ATTN_NORM );

			CSoundEnt::InsertSound ( bits_SOUND_DANGER, pev->origin, 400, 0.3 );

			m_flNextSound = gpGlobals->time + 1.0;
		}

		if ( m_flCountDown == 0)
		{
			m_flCountDown = gpGlobals->time + 2.0; //temps avant l'explosion ( 2secondes )
		}

		if ( m_flCountDown < gpGlobals->time )
		{
			
			pev->owner = m_pRealOwner;
			pev->health = 0;
			Killed( VARS( pev->owner ), GIB_NORMAL );
						
		}
		
		pev->skin = 2;// met la skin d'alerte

	}
	if ( !m_alert )
	{
		m_flCountDown = 0;
		pev->skin = 1;//remet la skin de marche

	}

	if ( pev->iuser2 == 1 )//explosion tlcommande
	{
		pev->owner = m_pRealOwner;
		pev->health = 0;
		Killed( VARS( pev->owner ), GIB_NORMAL );

	}

	m_ramasse = 0;//on ne peut plus prendre la mine

	pev->nextthink = gpGlobals->time + 0.1;
}



void CSeqchargeGrenade :: PowerupThink( void  )
{
	TraceResult tr;

	if (m_hOwner == NULL)
	{
		// trouve le propritaire de la mine
		edict_t *oldowner = pev->owner;
		pev->owner = NULL;
		// trace une ligne pour dtecter le poseur de mine
		UTIL_TraceLine( pev->origin + m_vecDir * 8, pev->origin - m_vecDir * 32, dont_ignore_monsters, ENT( pev ), &tr );
		if (tr.fStartSolid || (oldowner && tr.pHit == oldowner))// on dtecte un mur ou l'ancien propritaire
		{
			pev->owner = oldowner;
			m_flPowerUp += 0.1;
			pev->nextthink = gpGlobals->time + 0.1;
			return;
		}
		if (tr.flFraction < 1.0) //si un poseur est dtect
		{
			pev->owner = tr.pHit;
			m_hOwner = CBaseEntity::Instance( pev->owner );// dfini le propritaire
			m_posOwner = m_hOwner->pev->origin;
			m_angleOwner = m_hOwner->pev->angles;
		}
		else // si pas de propritaire trouv on l'limine
		{
			STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/activation.wav" );
			STOP_SOUND( ENT(pev), CHAN_BODY, "weapons/alertemine.wav" );
			SetThink( SUB_Remove );
			pev->nextthink = gpGlobals->time + 0.1;
			ALERT( at_console, "WARNING:Seqcharge at %.0f, %.0f, %.0f removed\n", pev->origin.x, pev->origin.y, pev->origin.z );
			
			return;
		}
	}
	// deuxime test sur le propritaire(position et orientation)
	else if (m_posOwner != m_hOwner->pev->origin || m_angleOwner != m_hOwner->pev->angles)
	{
	
		STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/activation.wav" );
		STOP_SOUND( ENT(pev), CHAN_BODY, "weapons/alertemine.wav" );

		// on rcupre la mine que l'on a pos
		CBaseEntity *pSequ = Create( "weapon_seqcharge", pev->origin + m_vecDir * 24, pev->angles );
		pSequ->pev->spawnflags |= SF_NORESPAWN;

		SetThink( SUB_Remove );
	
		pev->nextthink = gpGlobals->time + 0.1;
		return;
	}
	// ALERT( at_console, "%d %.0f %.0f %0.f\n", pev->owner, m_pOwner->pev->origin.x, m_pOwner->pev->origin.y, m_pOwner->pev->origin.z );
 
	if (gpGlobals->time > m_flPowerUp)
	{
		// make solid
		pev->solid = SOLID_BBOX;
		UTIL_SetOrigin( pev, pev->origin );

		// play enabled sound
        EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "weapons/mineprete.wav", 0.5, ATTN_NORM, 1.0, 75 );

		SetThink( ScanThink );// dmarre le scanner
		pev->nextthink = gpGlobals->time + 0.1;
		
		pev->skin = 1;//le bouton vert s'teint et la barre rouge s'allume

		/*
		
		if (m_radical)
		{
			ALERT( at_console, "RADICAL\n");
		}
		if (m_MapRadical)
		{
			ALERT( at_console, "MAP_RADICAL = 1\n");
		}

		if (!m_MapRadical)
		{
			ALERT( at_console, "MAP_RADICAL = 0\n");
		}
		
		*/
		
		return;

	}
	pev->nextthink = gpGlobals->time + 0.1;
}



int CSeqchargeGrenade :: TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType )

{
	//on dfini que si c'est un choc lectrique (champs de force) on ne fera pas de decal sur le champs

	if (bitsDamageType == DMG_SHOCK)
		m_PasDeDecal = 1;


	// si le temps de mise en marche n'est pas termin et si le dommage est trs faible
	if (bitsDamageType != DMG_CLUB && flDamage > pev->health)
	{
		SetThink( DelayDeathThink );
		pev->nextthink = gpGlobals->time + RANDOM_FLOAT( 0.1, 0.3 );

		
		EMIT_SOUND( ENT(pev), CHAN_BODY, "common/null.wav", 0.5, ATTN_NORM ); // coupe le son d'alertemine
	}
	else if (bitsDamageType == DMG_CLUB)
	{
		// dsactive la mine et gnre une mine que le joueur peut ramasser
		CBaseEntity *pSequ = Create( "weapon_seqcharge", pev->origin + m_vecDir * 24, pev->angles );
		pSequ->pev->spawnflags |= SF_NORESPAWN;
		
		SetThink( SUB_Remove );
		pev->nextthink = gpGlobals->time + 0.1;
		
		return FALSE;
	}
	return CGrenade::TakeDamage( pevInflictor, pevAttacker, flDamage, bitsDamageType );
}

void CSeqchargeGrenade::Killed( entvars_t *pevAttacker, int iGib )
{
	pev->takedamage = DAMAGE_NO;
	
	if ( pevAttacker && ( pevAttacker->flags & FL_CLIENT ) )
	{
		// renseigne sur qui a pos la mine pour le crditer du dommage en multijoueurs
		pev->owner = ENT( pevAttacker );
	}

	SetThink( DelayDeathThink );
	pev->nextthink = gpGlobals->time + RANDOM_FLOAT( 0.1, 0.3 );

	EMIT_SOUND( ENT(pev), CHAN_BODY, "common/null.wav", 0.5, ATTN_NORM ); // coupe le son d'alertemine
}


void CSeqchargeGrenade::DelayDeathThink( void )
{
	TraceResult tr;
	UTIL_TraceLine ( pev->origin + m_vecDir * 8, pev->origin - m_vecDir * 64,  dont_ignore_monsters, ENT(pev), & tr);
	Explode( &tr, DMG_BLAST, m_PasDeDecal);
}


////////////////////////////////////////////////////////////
//
// Arme du joueur
//
////////////////////////////////////////////////////////////

class CSeqcharge : public CBasePlayerWeapon
{
public:
	void Spawn( void );
	void Precache( void );
	int iItemSlot( void ) { return 5; }
	int GetItemInfo(ItemInfo *p);
	void SetObjectCollisionBox( void )
	{
		//!!!BUGBUG - fix the model!
		pev->absmin = pev->origin + Vector(-16, -16, -5);
		pev->absmax = pev->origin + Vector(16, 16, 28); 
	}
	void PrimaryAttack( void );
	void SecondaryAttack( void );
	BOOL Deploy( void );
	void Holster( int skiplocal = 0 );
	void WeaponIdle( void );
	
};
LINK_ENTITY_TO_CLASS( weapon_seqcharge, CSeqcharge );


void CSeqcharge::Spawn( )
{
	Precache( );
	m_iId = WEAPON_SEQCHARGE;
	SET_MODEL(ENT(pev), "models/w_seqcharge.mdl");
	pev->frame = 0;
	pev->body = 0;
	pev->skin = 0;
	pev->sequence = 1;
	
	pev->framerate = 0;

	pev->iuser3 = 1;//masse

	FallInit();

	m_iDefaultAmmo = SEQCHARGE_DEFAULT_GIVE;

	if ( !g_pGameRules->IsDeathmatch() )
	{
		UTIL_SetSize(pev, Vector(-16, -16, 0), Vector(16, 16, 28) ); 
	}

	m_radical = 0;
}

void CSeqcharge::Precache( void )
{
	PRECACHE_MODEL ("models/v_seqcharge.mdl");
	PRECACHE_MODEL ("models/w_seqcharge.mdl");
	PRECACHE_MODEL ("models/p_seqcharge.mdl");
	UTIL_PrecacheOther( "monster_seqcharge" );
}

int CSeqcharge::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "seqcharges";
	p->iMaxAmmo1 = SEQCHARGE_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 4;
	p->iPosition = 1;
	p->iId = m_iId = WEAPON_SEQCHARGE;
	p->iWeight = SEQCHARGE_WEIGHT;
	p->iFlags = ITEM_FLAG_LIMITINWORLD;//| ITEM_FLAG_EXHAUSTIBLE;//si plus de mines, plus d'arme

	return 1;
}

BOOL CSeqcharge::Deploy( )
{
	pev->body = 0;
	return DefaultDeploy( "models/v_seqcharge.mdl", "models/p_seqcharge.mdl", SEQCHARGE_DRAW, "seqcharge" );
}


void CSeqcharge::Holster( int skiplocal /* = 0 */ )
{
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;

	if (!m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType])
	{
		// plus de mines
		m_pPlayer->pev->weapons &= ~(1<<WEAPON_SEQCHARGE);
		SetThink( DestroyItem );
		pev->nextthink = gpGlobals->time + 0.1;
	}

	SendWeaponAnim( SEQCHARGE_HOLSTER );
	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "common/null.wav", 1.0, ATTN_NORM);
}

void CSeqcharge::PrimaryAttack( void )
{
	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		return;

	UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );
	Vector vecSrc	 = m_pPlayer->GetGunPosition( );
	Vector vecAiming = gpGlobals->v_forward;

	TraceResult tr;

	UTIL_TraceLine( vecSrc, vecSrc + vecAiming * 128, dont_ignore_monsters, ENT( m_pPlayer->pev ), &tr );

	if (tr.flFraction < 1.0)
	{
		// ALERT( at_console, "hit %f\n", tr.flFraction );

		CBaseEntity *pEntity = CBaseEntity::Instance( tr.pHit );
		if (pEntity && !(pEntity->pev->flags & FL_CONVEYOR))
		{
			Vector angles = UTIL_VecToAngles( tr.vecPlaneNormal );

			CBaseEntity *pEnt = CBaseEntity::Create( "monster_seqcharge", tr.vecEndPos + tr.vecPlaneNormal * 1, angles, m_pPlayer->edict() );

			CSeqchargeGrenade *pSequ = (CSeqchargeGrenade *)pEnt;

			m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

			
			m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

			if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0)
			{
				SendWeaponAnim( SEQCHARGE_DRAW );
			}
			else
			{				 
				m_pPlayer->pev->viewmodel = iStringNull;
				m_pPlayer->pev->weaponmodel = iStringNull;
				return;
			}
		}
		else
		{
			
		}
	}
	else
	{

		
	}

	m_flNextPrimaryAttack = gpGlobals->time + 0.3;
	m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );

	UTIL_MakeVectors( m_pPlayer->pev->v_angle );

	m_radical = 0;
}


void CSeqcharge::SecondaryAttack( void )
{
	float m_flRayon = 1024;
	
	CBaseEntity *pEntity = NULL;


	

	UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );
	Vector vecSrc	 = m_pPlayer->GetGunPosition( );
	Vector vecAiming = gpGlobals->v_forward;

	TraceResult tr;

	UTIL_TraceLine( vecSrc, vecSrc + vecAiming * 128, dont_ignore_monsters, ENT( m_pPlayer->pev ), &tr );

	if (tr.flFraction < 1.0)
	{
		// ALERT( at_console, "hit %f\n", tr.flFraction );

		if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		return;


		CBaseEntity *pEntity = CBaseEntity::Instance( tr.pHit );
		if (pEntity && !(pEntity->pev->flags & FL_CONVEYOR))
		{
			Vector angles = UTIL_VecToAngles( tr.vecPlaneNormal );

			CBaseEntity *pEnt = CBaseEntity::Create( "monster_seqcharge", tr.vecEndPos + tr.vecPlaneNormal * 1, angles, m_pPlayer->edict() );

			CSeqchargeGrenade *pSequ = (CSeqchargeGrenade *)pEnt;

			m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

			m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

			if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0)
			{
				SendWeaponAnim( SEQCHARGE_DRAW );
			}
			else
			{				 
				m_pPlayer->pev->viewmodel = iStringNull;
				m_pPlayer->pev->weaponmodel = iStringNull;
			}
		}
		else
		{
			// ALERT( at_console, "no deploy\n" );
		}
	}
	else
	{
		while ((pEntity = UTIL_FindEntityInSphere( pEntity, m_pPlayer->pev->origin, m_flRayon)) != NULL)
			{

		
				if ( FClassnameIs( pEntity->pev, "monster_seqcharge" ) )
				{

					if ( pEntity->pev->iuser1 == 1 )
					{
						//ALERT( at_console, "BOUM \n" );

						pEntity->pev->iuser2 = 1;// on fait exploser
					}
				}
			}

	}

	m_flNextSecondaryAttack = gpGlobals->time + 0.3;
	m_flTimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT ( 10, 15 );

	m_radical = 1;

	UTIL_MakeVectors( m_pPlayer->pev->v_angle );

}


void CSeqcharge::WeaponIdle( void )
{
	if (m_flTimeWeaponIdle > gpGlobals->time)
		return;

	if (m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0)
	{
		SendWeaponAnim( SEQCHARGE_DRAW );
	}
	else
	{
		m_pPlayer->pev->viewmodel = iStringNull;
		m_pPlayer->pev->weaponmodel = iStringNull;
		return;
	}

	int iAnim;
	float flRand = RANDOM_FLOAT(0, 1);
	if (flRand <= 0.25)
	{
		iAnim = SEQCHARGE_IDLE1;
		m_flTimeWeaponIdle = gpGlobals->time + 6;
	}
	else if (flRand <= 0.75)
	{
		iAnim = SEQCHARGE_IDLE2;
		m_flTimeWeaponIdle = gpGlobals->time + 6;
	}
	else
	{
		iAnim = SEQCHARGE_FIDGET;
		m_flTimeWeaponIdle = gpGlobals->time + 6;
	}

	SendWeaponAnim( iAnim );
	
}




