/***
*
*	Copyright (c) 1999, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   This source code contains proprietary and confidential information of
*   Valve LLC and its suppliers.  Access to this code is restricted to
*   persons who have executed a written SDK license with Valve.  Any access,
*   use or distribution of this code by or to any unlicensed person is illegal.
*
****/
//=========================================================
// stormt
//=========================================================

//=========================================================
// Hit groups!	
//=========================================================
/*

  1 - Head
  2 - Stomach
  3 - Gun

*/


#include	"extdll.h"
#include	"plane.h"
#include	"util.h"
#include	"cbase.h"
#include	"monsters.h"
#include	"schedule.h"
#include	"animation.h"
#include	"squadmonster.h"
#include	"weapons.h"
#include	"talkmonster.h"
#include	"soundent.h"
#include	"effects.h"
#include	"customentity.h"
//#include	"blast.h"

int g_fStormQuestion;				// true if an idle stormtrooper asked a question. Cleared when someone answers.
int m_Misprite;
int m_Mismoke;
int  m_iSTrail;
int m_StWallSmoke;

Vector direction;	// ajout pour stocker la direction du tir du railgun et la transmettre de la fonction Railgun()  
					// la fonction Spawn() de la classe CSRailRocket.


extern DLL_GLOBAL int		g_iSkillLevel;

//=========================================================
// monster-specific DEFINE's
//=========================================================
#define	STORM_CLIP_SIZE					36 // how many bullets in a clip? - NOTE: 3 round burst sound, so keep as 3 * x!
#define STORM_VOL						0.65		// volume of stormtrooper sounds
#define STORM_ATTN						ATTN_NORM	// attenutation of stormtrooper sentences
#define STORMT_LIMP_HEALTH				20
#define STORMT_DMG_HEADSHOT				( DMG_BULLET | DMG_CLUB )	// damage types that can kill a stormtrooper with a single headshot.
#define STORMT_NUM_HEADS				2 // how many stormtrooper heads are there? 
#define STORMT_MINIMUM_HEADSHOT_DAMAGE	15 // must do at least this much damage in one shot to head to score a headshot kill
#define	STORMT_SENTENCE_VOLUME			(float)0.65 // volume of stormtrooper sentences

#define STORMT_STRIFLE				( 1 << 0) // strifle a le numro : 1				
#define STORMT_HANDGRENADE			( 1 << 1) // grenade a le numro : 2		
#define STORMT_GRENADELAUNCHER		( 1 << 2) // lance grenade a le numro : 4	
#define STORMT_RAILGUN				( 1 << 3) // le railgun a le numro : 8 	

#define HEAD_GROUP					1
#define HEAD_STORM					0
#define HEAD_COMMANDER				1
#define HEAD_RAILGUN				2
#define HEAD_FIELD					3
#define GUN_GROUP					2
#define GUN_STRIFLE					0
#define GUN_RAILGUN					1
#define GUN_NONE					2

//=========================================================
// Monster's Anim Events Go Here
//=========================================================
#define		STORMT_AE_RELOAD		( 2 )
#define		STORMT_AE_KICK			( 3 )
#define		STORMT_AE_BURST1		( 4 )
#define		STORMT_AE_BURST2		( 5 ) 
#define		STORMT_AE_GREN_TOSS		( 6 )
#define		STORMT_AE_GREN_DROP		( 7 )
#define		STORMT_AE_CAUGHT_ENEMY	( 8) // stormtrooper established sight with an enemy (player only) that had previously eluded the squad.
#define		STORMT_AE_DROP_GUN		( 9) // stormtrooper (probably dead) is dropping his strifle.
#define		STORMT_AE_RELOAD2		( 10) // recharge railgun
//=========================================================
// monster-specific schedule types
//=========================================================
enum
{
	SCHED_STORM_SUPPRESS = LAST_COMMON_SCHEDULE + 1,
	SCHED_STORM_ESTABLISH_LINE_OF_FIRE,// move to a location to set up an attack against the enemy. (usually when a friendly is in the way).
	SCHED_STORM_COVER_AND_RELOAD,
	SCHED_STORM_SWEEP,
	SCHED_STORM_FOUND_ENEMY,
	SCHED_STORM_REPEL,
	SCHED_STORM_REPEL_ATTACK,
	SCHED_STORM_REPEL_LAND,
	SCHED_STORM_WAIT_FACE_ENEMY,
	SCHED_STORM_TAKECOVER_FAILED,// special schedule type that forces analysis of conditions and picks the best possible schedule to recover from this type of failure.
	SCHED_STORM_ELOF_FAIL,
	SCHED_STORM_RECHERCHE,
};

//=========================================================
// monster-specific tasks
//=========================================================
enum 
{
	TASK_STORM_FACE_TOSS_DIR = LAST_COMMON_TASK + 1,
	TASK_STORM_SPEAK_SENTENCE,
	TASK_STORM_CHECK_FIRE,
};

//=========================================================
// monster-specific conditions
//=========================================================
#define bits_COND_STORM_NOFIRE	( bits_COND_SPECIAL1 )


//=========================================================
//Point rouge de guidage
//=========================================================


class CSLaserSpot : public CBaseEntity // point rouge de guidage laser
{
	void Spawn( void );
	void Precache( void );

	int	ObjectCaps( void ) { return FCAP_DONT_SAVE; }

public:
	void Suspend( float flSuspendTime ); // pour un temps de pause
	void EXPORT Revive( void );			// ravive le point
	
	static CSLaserSpot *CreateSpot( void );
};
LINK_ENTITY_TO_CLASS( slaser_spot, CSLaserSpot );




//========================================================
// Dfinition de la classe Stormtrooper
//========================================================

class CStormt : public CSquadMonster
{
public:
	void Spawn( void );
	void Precache( void );
	void SetYawSpeed ( void );
	int  Classify ( void );
	int ISoundMask ( void );
	void HandleAnimEvent( MonsterEvent_t *pEvent );
	BOOL FCanCheckAttacks ( void );
	BOOL CheckMeleeAttack1 ( float flDot, float flDist );
	BOOL CheckRangeAttack1 ( float flDot, float flDist );
	BOOL CheckRangeAttack2 ( float flDot, float flDist );
	void CheckAmmo ( void );
	void SetActivity ( Activity NewActivity );
	void StartTask ( Task_t *pTask );
	void RunTask ( Task_t *pTask );
	void DeathSound( void );
	void PainSound( void );
	void IdleSound ( void );
	Vector GetGunPosition( void );
	void Shoot ( void );
	void Railgun ( void );
	void PrescheduleThink ( void );
	void GibMonster( void );
	void SpeakSentence( void );
	void RunAI(void);

	int	Save( CSave &save ); 
	int Restore( CRestore &restore );
	
	CBaseEntity	*Kick( void );
	Schedule_t	*GetSchedule( void );
	Schedule_t  *GetScheduleOfType ( int Type );
	void TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType);
	int TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType );

	int IRelationship ( CBaseEntity *pTarget );

	BOOL FOkToSpeak( void );
	void JustSpoke( void );

	CUSTOM_SCHEDULES;
	static TYPEDESCRIPTION m_SaveData[];

	// checking the feasibility of a grenade toss is kind of costly, so we do it every couple of seconds,
	// not every server frame.
	float m_flNextGrenadeCheck;
	float m_flNextPainTime;
	float m_flLastEnemySightTime;

	Vector	m_vecTossVelocity;

	BOOL	m_fThrowGrenade;
	BOOL	m_fStanding;
	BOOL	m_fFirstEncounter;// only put on the handsign show in the squad's first encounter.
	int		m_cClipSize;

	int m_voicePitch;

	int	m_iSentence;

	int m_RActiveRockets;

	static const char *pStormSentences[];
};

LINK_ENTITY_TO_CLASS( monster_stormtrooper, CStormt );

TYPEDESCRIPTION	CStormt::m_SaveData[] = 
{
	DEFINE_FIELD( CStormt, m_flNextGrenadeCheck, FIELD_TIME ),
	DEFINE_FIELD( CStormt, m_flNextPainTime, FIELD_TIME ),
	DEFINE_FIELD( CStormt, m_vecTossVelocity, FIELD_VECTOR ),
	DEFINE_FIELD( CStormt, m_fThrowGrenade, FIELD_BOOLEAN ),
	DEFINE_FIELD( CStormt, m_fStanding, FIELD_BOOLEAN ),
	DEFINE_FIELD( CStormt, m_fFirstEncounter, FIELD_BOOLEAN ),
	DEFINE_FIELD( CStormt, m_cClipSize, FIELD_INTEGER ),
	DEFINE_FIELD( CStormt, m_voicePitch, FIELD_INTEGER ),
	DEFINE_FIELD( CStormt, m_iSentence, FIELD_INTEGER ),
};

IMPLEMENT_SAVERESTORE( CStormt, CSquadMonster );

const char *CStormt::pStormSentences[] = 
{
	"ST_GREN", // grenade scared stormtrooper
	"ST_ALERT", // sees player
	"ST_MONST", // sees monster
	"ST_COVER", // running to cover
	"ST_THROW", // about to throw grenade
	"ST_CHARGE",  // running out to get the enemy
	"ST_TAUNT", // say rude things
};

enum
{
	STORMT_SENT_NONE = -1,
	STORMT_SENT_GREN = 0,
	STORMT_SENT_ALERT,
	STORMT_SENT_MONST,
	STORMT_SENT_COVER,
	STORMT_SENT_THROW,
	STORMT_SENT_CHARGE,
	STORMT_SENT_TAUNT,
} STORMT_SENTENCE_TYPES;



//=======================================================
//Fonctions pour le point rouge
//=======================================================

CSLaserSpot *CSLaserSpot::CreateSpot( void )  // gnre un point rouge  partir de la dfinition de classe
{
	CSLaserSpot *SSpot = GetClassPtr( (CSLaserSpot *)NULL );
	SSpot->Spawn();

	SSpot->pev->classname = MAKE_STRING("slaser_spot");

	return SSpot;
}

//=========================================================
//=========================================================
void CSLaserSpot::Spawn( void ) // cration du point
{
	Precache( );
	pev->movetype = MOVETYPE_NONE;
	pev->solid = SOLID_NOT;

	pev->rendermode = kRenderGlow;
	pev->renderfx = kRenderFxNoDissipation;
	pev->renderamt = 255;

	SET_MODEL(ENT(pev), "sprites/laserdot.spr");
	UTIL_SetOrigin( pev, pev->origin );
};

//=========================================================
// Suspend- rend le point invisible pour un moment 
//=========================================================
void CSLaserSpot::Suspend( float flSuspendTime )
{
	pev->effects |= EF_NODRAW;
	
	SetThink( Revive );
	pev->nextthink = gpGlobals->time + flSuspendTime;
}

//=========================================================
// Revive - ravive un point rendu invisible par suspend.
//=========================================================
void CSLaserSpot::Revive( void )
{
	pev->effects &= ~EF_NODRAW;

	SetThink( NULL );
}

void CSLaserSpot::Precache( void )
{
	PRECACHE_MODEL("sprites/laserdot.spr");
};



//=========================================================
// Projectile de blaster pour les Strifle des Stormtroopers de la companie BlasTech industries
//=========================================================

class CMBlaster : public CBaseEntity
{
public:
	void Spawn( void );

	static void Shoot( entvars_t *pevOwner, Vector MvecStart, Vector MvecVelocity );
	void Touch( CBaseEntity *pOther );
	void EXPORT Animate( void );

	virtual int		Save( CSave &save );
	virtual int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];

	int  m_maxFrame;
	int  m_Mblastballs;
	int  m_Mblastglow;
	float nextAnim;
	Vector vecNewVelocity;
	BOOL m_refelct;

	float m_flAttenuation;//facteur de perte d'efficacit avec le temps de vol
};

LINK_ENTITY_TO_CLASS( Mblaster, CMBlaster );

TYPEDESCRIPTION	CMBlaster::m_SaveData[] = 
{
	DEFINE_FIELD( CMBlaster, m_maxFrame, FIELD_INTEGER ),
};

IMPLEMENT_SAVERESTORE( CMBlaster, CBaseEntity );


void CMBlaster:: Spawn( void )
{
	pev->movetype = MOVETYPE_FLY;
	pev->classname = MAKE_STRING( "Mblaster" );
	
	pev->solid = SOLID_BBOX;
	pev->rendermode = kRenderTransAdd;
	pev->renderamt = 255;

	SET_MODEL(ENT(pev), "sprites/blaster.spr");
	pev->frame = 0;
	pev->scale = 0.2;

	UTIL_SetSize( pev, Vector( 0, 0, 0), Vector(0, 0, 0) );

	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );

		WRITE_BYTE( TE_BEAMFOLLOW ); // le sillage suit le projectile
		WRITE_SHORT(entindex());	// entit
		WRITE_SHORT(m_iSTrail );	// modle
		WRITE_BYTE( 1 ); // dure
		WRITE_BYTE( 1 );  // largeur
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 0 );   // r, g, b
		WRITE_BYTE( 0 );   // r, g, b
		WRITE_BYTE( 255 );	// luminosit

	MESSAGE_END();  

	m_maxFrame = (float) MODEL_FRAMES( pev->modelindex ) - 1;
	nextAnim = 0;
	m_flAttenuation = 1;

}

void CMBlaster::Animate( void )
{
	//ajout de lumire mise par le projectile blaster

	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
			WRITE_BYTE(TE_DLIGHT);
			WRITE_COORD(pev->origin.x);	// X
			WRITE_COORD(pev->origin.y);	// Y
			WRITE_COORD(pev->origin.z);	// Z
			WRITE_BYTE( 15 );		// radius * 0.1
			WRITE_BYTE( 255 );		// r
			WRITE_BYTE( 100 );		// g
			WRITE_BYTE( 100 );		// b
			WRITE_BYTE( 0.2 );		// time * 10   1 doit tre 0.1 seconde
			WRITE_BYTE( 0 );		// decay * 0.1
		MESSAGE_END( );

	//le reste on ne le fait que chaque dixime de seconde
		
	pev->nextthink = gpGlobals->time;

	if ( nextAnim < gpGlobals->time )
	{

		m_flAttenuation -= 0.1;//on diminue de 0.1 chaque 1/10me de seconde

		if (m_flAttenuation < 0.5)//on limite  un minimum
			m_flAttenuation = 0.5;

		nextAnim = gpGlobals->time + 0.1;
		
		Vector vecVelocity;
		int iPitch = RANDOM_FLOAT( 90, 110 );

		if ( pev->frame++ )
		{
			if ( pev->frame > m_maxFrame )
			{
				pev->frame = 0;
			}
		}

		if ( m_refelct )
		{
			pev->velocity = vecNewVelocity;

			m_refelct = 0;
		}

		vecVelocity = pev->velocity;

		if ( vecVelocity.Length() < 1000 )
		{
			
			EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "weapons/grill.wav", 1, ATTN_NORM, 0, iPitch );	

			switch ( RANDOM_LONG( 0, 1 ) )
			{
			case 0:
				EMIT_SOUND_DYN( ENT(pev), CHAN_WEAPON, "weapons/choc3.wav", 1, ATTN_NORM, 0, iPitch );	
				break;
			case 1:
				EMIT_SOUND_DYN( ENT(pev), CHAN_WEAPON, "weapons/choc4.wav", 1, ATTN_NORM, 0, iPitch );	
				break;
			}
				Vector centre = pev->origin;

					MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, centre );
						WRITE_BYTE( TE_SPRITE );
						WRITE_COORD( centre.x );	// position
						WRITE_COORD( centre.y );	
						WRITE_COORD( centre.z + 6 );	
						WRITE_SHORT( m_Mismoke );		// modle
						WRITE_BYTE( 3 );				//taille * 10
						WRITE_BYTE( 255 );			//luminosit
					MESSAGE_END();

			SetThink ( SUB_Remove );
			pev->nextthink = gpGlobals->time;
		}
	}

}

void CMBlaster::Shoot( entvars_t *pevOwner, Vector MvecStart, Vector MvecVelocity )
{
	CMBlaster *pMBlast = GetClassPtr( (CMBlaster *)NULL );
	pMBlast->Spawn();
	
	UTIL_SetOrigin( pMBlast->pev, MvecStart );
	pMBlast->pev->velocity = MvecVelocity;
	pMBlast->pev->euser1 = ENT(pevOwner);
	pMBlast->pev->owner = NULL;// modifi pour que le ricochet du sabre l'atteigne
	pMBlast->SetThink ( Animate );
	pMBlast->pev->nextthink = gpGlobals->time + 0.1;
}

void CMBlaster :: Touch ( CBaseEntity *pOther )
{
	TraceResult tr;
	int		iMPitch;
	m_Mblastglow = m_Misprite;
	m_Mblastballs = m_Misprite;
	Vector vecDirImpact;
	
	Vector vecTrOrigin;


	// son de choc
	iMPitch = RANDOM_FLOAT( 90, 110 );

	entvars_t *pevAttacker = NULL;
	if ( pev->euser1 )
	{
		pevAttacker = VARS(pev->euser1);
	}

	if ( pev->euser1 == NULL ) // attention si pevAttacker est null on plante 
												   
	{
		pevAttacker = pev;
	}
	
	if ( pOther->pev->iuser3 == 1000 )//c'est une surface blinde ou un champs de force
	{
		UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );

		float fl_vitesse = pev->velocity.Length();
		
		vecNewVelocity = pev->velocity.Normalize();
		

		float n = -DotProduct(tr.vecPlaneNormal, vecNewVelocity);

		vecNewVelocity = ( 2 * tr.vecPlaneNormal * n ) + vecNewVelocity;
		vecNewVelocity.Normalize();
		vecNewVelocity = vecNewVelocity * fl_vitesse * 0.8;
		
		
		UTIL_SetOrigin( pev, (pev->origin) + (tr.vecPlaneNormal * 3) );//on sort le blaster du mur
		pev->velocity = vecNewVelocity;

		m_refelct = 1;

		EMIT_SOUND_DYN(ENT(pev), CHAN_WEAPON, "weapons/lsaber_ric.wav", 1, ATTN_NORM, 0, 94 + RANDOM_LONG(0,0xF));

		return;
	}


	if ( pOther->pev->takedamage && pOther->pev->iuser3 == 100 )//ATST blind rflchit les blasters
	{
		//on va crer un ricochet

		vecNewVelocity = pev->velocity;

		vecNewVelocity = vecNewVelocity.Normalize();
				
		vecTrOrigin = pOther->pev->origin - pev->origin;
		vecTrOrigin = vecTrOrigin.Normalize();
		
			
		vecNewVelocity = vecNewVelocity - vecTrOrigin;

		if ( vecNewVelocity.Length() == 0 )
		{
			ALERT( at_console, "VELOCITY NULLE\n");
			
			vecNewVelocity = -1 * vecTrOrigin;
		}

		vecNewVelocity = vecNewVelocity.Normalize();

		pev->velocity = vecNewVelocity * 1500;

		vecNewVelocity = pev->velocity;

		//m_flVitesse = vecNewVelocity.Length() ;
		
		//ALERT( at_console, "VELOCITY %f\n", m_flVitesse);


	}
	else//sinon c'est normal
	{

		if ( !pOther->pev->takedamage )
		{
			
			UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );
			
			//pas d'effet si on touche le ciel

			if ( UTIL_IsTextureSky( &tr, pev->origin, pev->origin + pev->velocity * 10 ) )
			{
				SetThink ( SUB_Remove );
				pev->nextthink = gpGlobals->time;

				return;

			}

			
			// sons d'impacte sur les murs
			EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "weapons/grill.wav", 1, ATTN_NORM, 0, iMPitch );	

				switch ( RANDOM_LONG( 0, 1 ) )
				{
				case 0:
					EMIT_SOUND_DYN( ENT(pev), CHAN_WEAPON, "weapons/choc1.wav", 1, ATTN_NORM, 0, iMPitch );	
					break;
				case 1:
					EMIT_SOUND_DYN( ENT(pev), CHAN_WEAPON, "weapons/choc2.wav", 1, ATTN_NORM, 0, iMPitch );	
					break;
				}

			// placer un decal sur les murs
			
			UTIL_DecalTrace(&tr, 31);
			
			// incandescence sur les murs
					MESSAGE_BEGIN( MSG_PAS, SVC_TEMPENTITY, tr.vecEndPos );
						WRITE_BYTE( TE_GLOWSPRITE );
						WRITE_COORD( tr.vecEndPos.x);		// pos
						WRITE_COORD( tr.vecEndPos.y);
						WRITE_COORD( tr.vecEndPos.z);
						WRITE_SHORT( m_Mblastglow );		// modle
						WRITE_BYTE( 30 );	// dure
						WRITE_BYTE( 2 );					// taille
						WRITE_BYTE( 3000 );			// luminosit
					MESSAGE_END();

				

					// billes incandescentes projetes
					MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, tr.vecEndPos );
						WRITE_BYTE( TE_SPRITETRAIL );// TE_RAILTRAIL);
						WRITE_COORD( tr.vecEndPos.x );
						WRITE_COORD( tr.vecEndPos.y );
						WRITE_COORD( tr.vecEndPos.z );
						WRITE_COORD( tr.vecEndPos.x + tr.vecPlaneNormal.x );
						WRITE_COORD( tr.vecEndPos.y + tr.vecPlaneNormal.y );
						WRITE_COORD( tr.vecEndPos.z + tr.vecPlaneNormal.z );
						WRITE_SHORT( m_Mblastballs );		// modle
						WRITE_BYTE( 5 );				// nombre
						WRITE_BYTE( 5 );				// dure
						WRITE_BYTE( RANDOM_LONG( 1, 2 ) );				// taille
						WRITE_BYTE( 10 );				// amplitude * 0.1
						WRITE_BYTE( 5 );				// vitesse * 100
					MESSAGE_END();

					Vector vecSmokePos = tr.vecEndPos + ( tr.vecPlaneNormal * 10 );//loigne la sprite lgrement du mur
					
					MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, vecSmokePos );//nuage de fume
						WRITE_BYTE( TE_SPRITE );
						WRITE_COORD( vecSmokePos.x );	// position
						WRITE_COORD( vecSmokePos.y );	
						WRITE_COORD( vecSmokePos.z + 16 );	
						WRITE_SHORT( m_StWallSmoke );		// modle
						WRITE_BYTE( 10 );				//taille * 10
						WRITE_BYTE( 255 );			//luminosit
					MESSAGE_END();
		
		}
		else
		{
			//sons d'impacte sur les corps
			EMIT_SOUND_DYN( ENT(pev), CHAN_VOICE, "weapons/grill.wav", 1, ATTN_NORM, 0, iMPitch );	

				switch ( RANDOM_LONG( 0, 1 ) )
				{
				case 0:
					EMIT_SOUND_DYN( ENT(pev), CHAN_WEAPON, "weapons/choc3.wav", 1, ATTN_NORM, 0, iMPitch );	
					break;
				case 1:
					EMIT_SOUND_DYN( ENT(pev), CHAN_WEAPON, "weapons/choc4.wav", 1, ATTN_NORM, 0, iMPitch );	
					break;
				}

				Vector Mcentre = pev->origin;//pOther->Center();

				MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, Mcentre );
					WRITE_BYTE( TE_SPRITE );
					WRITE_COORD( Mcentre.x );	// pos
					WRITE_COORD( Mcentre.y );	
					WRITE_COORD( Mcentre.z + 6 );	
					WRITE_SHORT( m_Mismoke );		// model
					WRITE_BYTE( 3 );				// size * 10
					WRITE_BYTE( 255 );			// brightness
				MESSAGE_END();
			
				//pOther->TakeDamage ( pev, pev, gSkillData.blasterDmg, DMG_ENERGYBEAM );
				
				UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );

				vecDirImpact = pev->velocity;
				vecDirImpact.Normalize();

				ClearMultiDamage( );

				pOther->TraceAttack( pevAttacker, gSkillData.blasterDmg * m_flAttenuation, vecDirImpact, &tr, DMG_ENERGYBEAM );
				
				ApplyMultiDamage( pev, pevAttacker );
			
		}

		SetThink ( SUB_Remove );
		pev->nextthink = gpGlobals->time;
	}
}


//=========================================================
// Projectile du type rocket pour le railgun
//=========================================================


class CSRailRocket : public CGrenade // le projectile explosif du railgun pour Stormtrooper
{
public:
	int		Save( CSave &save );
	int		Restore( CRestore &restore );
	static	TYPEDESCRIPTION m_SaveData[];
	void Spawn( void );
	void Precache( void );
	void EXPORT FollowThink( void );
	void EXPORT IgniteThink( void );
	void EXPORT RocketTouch( CBaseEntity *pOther );
	static CSRailRocket *CreateSRailRocket( Vector vecOrigin, Vector vecAngles, entvars_t *pevOwner, CStormt *pLauncher );

	int m_iSRTrail; // trace
	float m_fRIgniteTime; // temps d'allumage
	CStormt *m_pLauncher;// pointe en arrire vers qui a envoy le projectile 
};
LINK_ENTITY_TO_CLASS( srail_rocket, CSRailRocket );

TYPEDESCRIPTION	CSRailRocket::m_SaveData[] = 
{
	DEFINE_FIELD( CSRailRocket, m_fRIgniteTime, FIELD_TIME ),
	DEFINE_FIELD( CSRailRocket, m_pLauncher, FIELD_CLASSPTR ),
};
IMPLEMENT_SAVERESTORE( CSRailRocket, CGrenade );


//=========================================================
// Gnre un projectile  partir de sa dfinition de classe
//=========================================================
CSRailRocket *CSRailRocket::CreateSRailRocket( Vector vecOrigin, Vector vecAngles, entvars_t *pevOwner, CStormt *pLauncher )
{
	CSRailRocket *pSRRocket = GetClassPtr( (CSRailRocket *)NULL );

	UTIL_SetOrigin( pSRRocket->pev, vecOrigin );
	pSRRocket->pev->angles = vecAngles;
	pSRRocket->Spawn();
	pSRRocket->SetTouch( CSRailRocket::RocketTouch );
	pSRRocket->m_pLauncher = pLauncher;// pour se souvenir de quel railgun est partit le coup.
	pSRRocket->m_pLauncher->m_RActiveRockets++;// enregistre le projectile comme actif pour ce railgun
	pSRRocket->pev->owner = ENT(pevOwner);

	return pSRRocket;
}

//=========================================================
// Cre un projectile
//=========================================================
void CSRailRocket :: Spawn( void ) 
{
	Precache( );

	pev->movetype = MOVETYPE_BOUNCE;
	pev->solid = SOLID_BBOX;

	SET_MODEL(ENT(pev), "models/railrocket.mdl");
	UTIL_SetSize(pev, Vector( 0, 0, 0), Vector(0, 0, 0));
	UTIL_SetOrigin( pev, pev->origin );

	pev->classname = MAKE_STRING("srail_rocket");

	SetThink( IgniteThink );
	SetTouch( ExplodeTouch );  // vrifie si une impacte a lieu.

	//pev->angles.x -= 5;
	//UTIL_MakeVectors( pev->angles );
	//pev->angles.x = -(pev->angles.x + 5);

	pev->velocity = direction * 500; // vitesse du projectile
	pev->gravity = 0.2;

	pev->nextthink = gpGlobals->time + 0.4;

	pev->dmg = gSkillData.plrDmgRAIL;  // dommages engendrs par l'explosion
}

//=========================================================
// Gre l'impacte du projectile
//=========================================================
void CSRailRocket :: RocketTouch ( CBaseEntity *pOther )
{
	TraceResult tr;

	if ( m_pLauncher )
	{
		// averti le railgun lanceur de l'impacte.
		m_pLauncher->m_RActiveRockets--;
	}
	
	// stope le son de propulsion
	STOP_SOUND( edict(), CHAN_VOICE, "weapons/railfly.wav" );

	UTIL_TraceLine( pev->origin, pev->origin + pev->velocity * 10, dont_ignore_monsters, ENT( pev ), &tr );

	//pas d'effet si on touche le ciel

	if ( UTIL_IsTextureSky( &tr, pev->origin, pev->origin + pev->velocity * 10 ) )
	{
		SetThink ( SUB_Remove );
		pev->nextthink = gpGlobals->time;

		return;
	}
	// cre l'explosion  l'impacte
	ExplodeTouch( pOther );
}

//=========================================================
//=========================================================
void CSRailRocket :: Precache( void )
{
	PRECACHE_MODEL("models/railrocket.mdl");
	m_iSRTrail = PRECACHE_MODEL("sprites/smoke.spr"); // trace de la propulsion
	PRECACHE_SOUND ("weapons/railfly.wav");
}

// =================================================
// dmarre le moteur de fuse, cre le sillage et la poursuite du point rouge
// =================================================
void CSRailRocket :: IgniteThink( void  )
{
	
	pev->movetype = MOVETYPE_FLY;	// dfini un objet volant
	pev->effects |= EF_LIGHT;		// cre un effet de lumire

	// dmarre le son du moteur de fuse
	EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/railfly.wav", 1, 0.5 );

	// cre un sillage de fume pour le projectile
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );

		WRITE_BYTE( TE_BEAMFOLLOW ); // le sillage suit le projectile
		WRITE_SHORT(entindex());	// entit
		WRITE_SHORT(m_iSRTrail );	// modle
		WRITE_BYTE( 40 ); // dure
		WRITE_BYTE( 5 );  // largeur
		WRITE_BYTE( 224 );   // r, g, b
		WRITE_BYTE( 224 );   // r, g, b
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );	// luminosit

	MESSAGE_END();  

	m_fRIgniteTime = gpGlobals->time; // dfini le moment du dpart

	// dmarre la poursuite du point rouge laser
	SetThink( FollowThink );
	pev->nextthink = gpGlobals->time + 0.1; // raligne la course sur le point tout les 0.1 sec
}

// ===================================================
// Gre le suivi du point rouge
// ===================================================
void CSRailRocket :: FollowThink( void  ) 
{
	CBaseEntity *pOther = NULL;
	Vector vecTarget;
	Vector vecDir;
	float flDist, flMax, flDot;
	TraceResult tr;

	UTIL_MakeAimVectors( pev->angles );

	vecTarget = direction; // on dfini la cible droit devant
	flMax = 4096;
	
	// Recherche du point rouge  dans un rayon donn.
	
	
	while ((pOther = UTIL_FindEntityByClassname( pOther, "rlaser_spot" )) != NULL)
	{
		UTIL_TraceLine ( pev->origin, pOther->pev->origin, dont_ignore_monsters, ENT(pev), &tr );
		// ALERT( at_console, "%f\n", tr.flFraction );
		if (tr.flFraction >= 0.90)
		{
			vecDir = pOther->pev->origin - pev->origin;
			flDist = vecDir.Length( );		// distance
			vecDir = vecDir.Normalize( );	// direction

		// compare la direction avant avec la direction de la cible
			flDot = DotProduct( gpGlobals->v_forward, vecDir );
			if ((flDot > 0) && (flDist * (1 - flDot) < flMax))
			{
				flMax = flDist * (1 - flDot);
				vecTarget = vecDir; // si le point n'est pas trop loign 
			}
		}
	}


	pev->angles = UTIL_VecToAngles( vecTarget );

	// approximation du calcul de l'acclration et du changement de direction
	float fRSpeed = pev->velocity.Length();
	
	if (gpGlobals->time - m_fRIgniteTime < 1.0) // si il y a moins d'une seconde depuis le dpart
	{
		// le projectile acclre
		pev->velocity = pev->velocity * 0.2 + vecTarget * (fRSpeed * 0.8 + 400);
		if (pev->waterlevel == 3)
		{
			// la vitesse est plus lente sous l'eau
			if (pev->velocity.Length() > 300)
			{
				pev->velocity = pev->velocity.Normalize() * 300;
			}

			// cre des bulles sous l'eau
			UTIL_BubbleTrail( pev->origin - pev->velocity * 0.1, pev->origin, 4 );
		} 
		else 
		{
			// limite la vitesse  2000
			if (pev->velocity.Length() > 1750)
			{
				pev->velocity = pev->velocity.Normalize() * 1750;//pour eviter que la sabre la renvoie
			}
		}
	}
	else // si il y a plus d'une seconde depuis le dpart
	{
		if (pev->effects & EF_LIGHT) // si l'effet lumineux est prsent
		{
			pev->effects = 0;	// arrte la luminosit
			STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/railfly.wav" ); // stope le son du moteur
		}
		pev->velocity = pev->velocity * 0.2 + vecTarget * fRSpeed * 0.798;// rduit la vitesse
		
		// si la vitesse tombe en dessous de 1500 le projectile explose
		if (pev->waterlevel == 0 && pev->velocity.Length() < 1500)
		{
			Detonate( );
		}
	}

	pev->nextthink = gpGlobals->time + 0.1;
}


//===================================
//RunAI
//===================================
void CStormt ::RunAI( void )
{

	CBaseMonster::RunAI();
/*
	pev->rendermode = kRenderNormal;

	pev->renderamt = 255;

	pev->rendercolor.x = 0;

	pev->rendercolor.y = 0;

	pev->rendercolor.z = 255;

	pev->renderamt = 10;

	pev->renderfx = kRenderFxGlowShell;
*/
}


//=========================================================
// Speak Sentence - say your cued up sentence.
//
// Some stormtrooper sentences (take cover and charge) rely on actually
// being able to execute the intended action. It's really lame
// when a stormtrooper says 'COVER ME' and then doesn't move. The problem
// is that the sentences were played when the decision to TRY
// to move to cover was made. Now the sentence is played after 
// we know for sure that there is a valid path. The schedule
// may still fail but in most cases, well after the stormtrooper has 
// started moving.
//=========================================================
void CStormt :: SpeakSentence( void )
{
	if ( m_iSentence == STORMT_SENT_NONE )
	{
		// no sentence cued up.
		return; 
	}

	if (FOkToSpeak())
	{
		SENTENCEG_PlayRndSz( ENT(pev), pStormSentences[ m_iSentence ], STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
		JustSpoke();
	}
}

//=========================================================
// IRelationship - overridden because Alien Grunts are 
// stormtroopers's nemesis.
//=========================================================
int CStormt::IRelationship ( CBaseEntity *pTarget )
{
	if ( FClassnameIs( pTarget->pev, "monster_alien_grunt" ) || ( FClassnameIs( pTarget->pev,  "monster_gargantua" ) ) )
	{
		return R_NM;
	}

	return CSquadMonster::IRelationship( pTarget );
}

//=========================================================
// GibMonster - make gun fly through the air.
//=========================================================
void CStormt :: GibMonster ( void )
{
	Vector	vecGunPos;
	Vector	vecGunAngles;

	if ( GetBodygroup( 2 ) != 2 )
	{// throw a gun if the stormtrooper has one
		GetAttachment( 0, vecGunPos, vecGunAngles );
		
		CBaseEntity *pGun;
		if (FBitSet( pev->weapons, STORMT_RAILGUN ))
		{
			pGun = DropItem( "weapon_railgun", vecGunPos, vecGunAngles );
		}
		else
		{
			pGun = DropItem( "weapon_strifle", vecGunPos, vecGunAngles );
		}
		if ( pGun )
		{
			pGun->pev->velocity = Vector (RANDOM_FLOAT(-100,100), RANDOM_FLOAT(-100,100), RANDOM_FLOAT(200,300));
			pGun->pev->avelocity = Vector ( 0, RANDOM_FLOAT( 200, 400 ), 0 );
		}
	
		
	}

	CBaseMonster :: GibMonster();
}

//=========================================================
// ISoundMask - Overidden for stormtroopers because they 
// hear the DANGER sound that is made by hand grenades and
// other dangerous items.
//=========================================================
int CStormt :: ISoundMask ( void )
{
	return	bits_SOUND_WORLD	|
			bits_SOUND_COMBAT	|
			bits_SOUND_PLAYER	|
			bits_SOUND_DANGER;
}

//=========================================================
// someone else is talking - don't speak
//=========================================================
BOOL CStormt :: FOkToSpeak( void )
{
// if someone else is talking, don't speak
	if (gpGlobals->time <= CTalkMonster::g_talkWaitTime)
		return FALSE;

	if ( pev->spawnflags & SF_MONSTER_GAG )
	{
		if ( m_MonsterState != MONSTERSTATE_COMBAT )
		{
			// no talking outside of combat if gagged.
			return FALSE;
		}
	}

	// if player is not in pvs, don't speak
//	if (FNullEnt(FIND_CLIENT_IN_PVS(edict())))
//		return FALSE;
	
	return TRUE;
}

//=========================================================
//=========================================================
void CStormt :: JustSpoke( void )
{
	CTalkMonster::g_talkWaitTime = gpGlobals->time + RANDOM_FLOAT(1.5, 2.0);
	m_iSentence = STORMT_SENT_NONE;
}

//=========================================================
// PrescheduleThink - this function runs after conditions
// are collected and before scheduling code is run.
//=========================================================
void CStormt :: PrescheduleThink ( void )
{
	if ( InSquad() && m_hEnemy != NULL )
	{
		if ( HasConditions ( bits_COND_SEE_ENEMY ) )
		{
			// update the squad's last enemy sighting time.
			MySquadLeader()->m_flLastEnemySightTime = gpGlobals->time;
		}
		else
		{
			if ( gpGlobals->time - MySquadLeader()->m_flLastEnemySightTime > 5 )
			{
				// been a while since we've seen the enemy
				MySquadLeader()->m_fEnemyEluded = TRUE;
			}
		}
	}
}

//=========================================================
// FCanCheckAttacks - this is overridden for stprmtroopers
// because they can throw/shoot grenades when they can't see their
// target and the base class doesn't check attacks if the monster
// cannot see its enemy.
//
// !!!BUGBUG - this gets called before a 3-round burst is fired
// which means that a friendly can still be hit with up to 2 rounds. 
// ALSO, grenades will not be tossed if there is a friendly in front,
// this is a bad bug. Friendly machine gun fire avoidance
// will unecessarily prevent the throwing of a grenade as well.
//=========================================================
BOOL CStormt :: FCanCheckAttacks ( void )
{
	if ( !HasConditions( bits_COND_ENEMY_TOOFAR ) )
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


//=========================================================
// CheckMeleeAttack1
//=========================================================
BOOL CStormt :: CheckMeleeAttack1 ( float flDot, float flDist )
{
	CBaseMonster *pEnemy;

	if ( m_hEnemy != NULL )
	{
		pEnemy = m_hEnemy->MyMonsterPointer();

		if ( !pEnemy )
		{
			return FALSE;
		}
	}

	if ( flDist <= 64 && flDot >= 0.7	&& 
		 pEnemy->Classify() != CLASS_ALIEN_BIOWEAPON &&
		 pEnemy->Classify() != CLASS_PLAYER_BIOWEAPON )
	{
		return TRUE;
	}
	return FALSE;
}

//=========================================================
// CheckRangeAttack1 - overridden for stormtroopers, cause 
// FCanCheckAttacks() doesn't disqualify all attacks based
// on whether or not the enemy is occluded because unlike
// the base class, the stormtrooper can attack when the enemy is
// occluded (throw grenade over wall, etc). We must 
// disqualify the machine gun attack if the enemy is occluded.
//=========================================================
BOOL CStormt :: CheckRangeAttack1 ( float flDot, float flDist )
{
	Vector vecSrc;
	
	if ( !HasConditions( bits_COND_ENEMY_OCCLUDED ) && flDist <= 2048 && flDot >= 0.5 && NoFriendlyFire() )
	{
		TraceResult	tr;

		if (FBitSet( pev->weapons, STORMT_RAILGUN))
		{
			
			if (gpGlobals->time < m_flNextGrenadeCheck )
			{
				return m_fThrowGrenade;
			}
			

			if ( InSquad() )
			{
				if (SquadMemberInRange( m_hEnemy->pev->origin, 384 ))//pas de tir si un membre de l'esquade est trop prs
				{
				 
					m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
					m_fThrowGrenade = FALSE;
				}
			}

			if ( ( m_hEnemy->pev->origin - pev->origin ).Length2D() <= 384 )
			{
				// crap, I don't want to blow myself up
				m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
				m_fThrowGrenade = FALSE;
				return m_fThrowGrenade;
			}

			vecSrc = GetGunPosition();

			// verify that a bullet fired from the gun will hit the enemy before the world.
			UTIL_TraceLine( vecSrc, m_hEnemy->BodyTarget(vecSrc), ignore_monsters, dont_ignore_glass, ENT(pev), &tr);

			if ( tr.flFraction == 1.0 ) //si gal  1 cel veut dire que la voie est libre
			{

				m_fThrowGrenade = TRUE;
				// don't check again for a while.
				m_flNextGrenadeCheck = gpGlobals->time; // 1/3 second.
			}
			else
			{
				//ALERT( at_console, "Dans le mur\n");
				// don't throw
				m_fThrowGrenade = FALSE;
				// don't check again for a while.
				m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
			}
			
			return m_fThrowGrenade;
		
		}
		
		
		if ( !m_hEnemy->IsPlayer() && flDist <= 64 )
		{
			// kick nonclients, but don't shoot at them.
			return FALSE;
		}

		vecSrc = GetGunPosition();

		// verify that a bullet fired from the gun will hit the enemy before the world.
		UTIL_TraceLine( vecSrc, m_hEnemy->BodyTarget(vecSrc), ignore_monsters, ignore_glass, ENT(pev), &tr);

		if ( tr.flFraction == 1.0 ) //si gal  1 cel veut dire que la voie est libre
		{

			return TRUE;
		}
		else
		{
			//ALERT( at_console, "Dans le mur\n");
			
		}
	}

	return FALSE;
}

//=========================================================
// CheckRangeAttack2 - this checks the stormtrooper's grenade
// attack. 
//=========================================================
BOOL CStormt :: CheckRangeAttack2 ( float flDot, float flDist )
{
	if (! FBitSet(pev->weapons, (STORMT_HANDGRENADE )))
	{
		return FALSE;
	}
	
	// if the stormtrooper isn't moving, it's ok to check.
	if ( m_flGroundSpeed != 0 )
	{
		m_fThrowGrenade = FALSE;
		return m_fThrowGrenade;
	}

	// assume things haven't changed too much since last time
	if (gpGlobals->time < m_flNextGrenadeCheck )
	{
		return m_fThrowGrenade;
	}

	if ( !FBitSet ( m_hEnemy->pev->flags, FL_ONGROUND ) && m_hEnemy->pev->waterlevel == 0 && m_vecEnemyLKP.z > pev->absmax.z  )
	{
		//!!!BUGBUG - we should make this check movetype and make sure it isn't FLY? Players who jump a lot are unlikely to 
		// be grenaded.
		// don't throw grenades at anything that isn't on the ground!
		m_fThrowGrenade = FALSE;
		return m_fThrowGrenade;
	}
	
	Vector vecTarget;

	if (FBitSet( pev->weapons, STORMT_HANDGRENADE))
	{
		// find feet
		if (RANDOM_LONG(0,1))
		{
			// magically know where they are
			vecTarget = Vector( m_hEnemy->pev->origin.x, m_hEnemy->pev->origin.y, m_hEnemy->pev->absmin.z );
		}
		else
		{
			// toss it to where you last saw them
			vecTarget = m_vecEnemyLKP;
		}
		// vecTarget = m_vecEnemyLKP + (m_hEnemy->BodyTarget( pev->origin ) - m_hEnemy->pev->origin);
		// estimate position
		// vecTarget = vecTarget + m_hEnemy->pev->velocity * 2;
	}
	else
	{
		// find target
		// vecTarget = m_hEnemy->BodyTarget( pev->origin );
		vecTarget = m_vecEnemyLKP + (m_hEnemy->BodyTarget( pev->origin ) - m_hEnemy->pev->origin);
		// estimate position
		if (HasConditions( bits_COND_SEE_ENEMY))
			vecTarget = vecTarget + ((vecTarget - pev->origin).Length() / gSkillData.stormtGrenadeSpeed) * m_hEnemy->pev->velocity;
	}

	// are any of my squad members near the intended grenade impact area?
	if ( InSquad() )
	{
		if (SquadMemberInRange( vecTarget, 384 ))
		{
			// crap, I might blow my own guy up. Don't throw a grenade and don't check again for a while.
			m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
			m_fThrowGrenade = FALSE;
		}
	}
	
	if ( ( vecTarget - pev->origin ).Length2D() <= 384 )
	{
		// crap, I don't want to blow myself up
		m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
		m_fThrowGrenade = FALSE;
		return m_fThrowGrenade;
	}

		
	if (FBitSet( pev->weapons, STORMT_HANDGRENADE))
	{
		Vector vecToss = VecCheckToss( pev, GetGunPosition(), vecTarget, 0.5 );

		if ( vecToss != g_vecZero )
		{
			m_vecTossVelocity = vecToss;

			// throw a hand grenade
			m_fThrowGrenade = TRUE;
			// don't check again for a while.
			m_flNextGrenadeCheck = gpGlobals->time; // 1/3 second.
		}
		else
		{
			// don't throw
			m_fThrowGrenade = FALSE;
			// don't check again for a while.
			m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
		}
	}
	else
	{
		Vector vecToss = VecCheckThrow( pev, GetGunPosition(), vecTarget, gSkillData.stormtGrenadeSpeed, 0.5 );

		if ( vecToss != g_vecZero )
		{
			m_vecTossVelocity = vecToss;

			// throw a hand grenade
			m_fThrowGrenade = TRUE;
			// don't check again for a while.
			m_flNextGrenadeCheck = gpGlobals->time + 0.3; // 1/3 second.
		}
		else
		{
			// don't throw
			m_fThrowGrenade = FALSE;
			// don't check again for a while.
			m_flNextGrenadeCheck = gpGlobals->time + 1; // one full second.
		}
	}

	

	return m_fThrowGrenade;
}


//=========================================================
// TraceAttack - make sure we're not taking it in the helmet
//=========================================================
void CStormt :: TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType)
{

	m_LastHitGroup = ptr->iHitgroup;

		switch ( ptr->iHitgroup )
		{
		case HITGROUP_GENERIC:
			break;
		case HITGROUP_HEAD:
			flDamage *= gSkillData.monHead;
			break;
		case HITGROUP_CHEST:
			flDamage *= gSkillData.monChest;
			break;
		case HITGROUP_STOMACH:
			flDamage *= gSkillData.monStomach;
			break;
		case HITGROUP_LEFTARM:
		case HITGROUP_RIGHTARM:
			flDamage *= gSkillData.monArm;
			break;
		case HITGROUP_LEFTLEG:
		case HITGROUP_RIGHTLEG:
			flDamage *= gSkillData.monLeg;
			break;
		default:
			break;
		}

	if ( bitsDamageType == DMG_SABER ) // pour comprendre, il faut couper la tte!
		{
			if ( ptr->iHitgroup == HITGROUP_HEAD )
			{
				if ( pev->health <= flDamage )
				{
					// code pour changer le modle (sans tte), 
					// crer une tte qui va rouler et jouer l'animation de mort correspondante.	
				}
			}
		}



	// check for helmet shot
	if (ptr->iHitgroup == 11)
	{
		// make sure we're wearing one
		if (GetBodygroup( 1 ) == HEAD_STORM && (bitsDamageType & (DMG_BULLET | DMG_SLASH | DMG_BLAST | DMG_CLUB)))
		{
			// absorb damage
			flDamage -= 20;
			if (flDamage <= 0)
			{
				UTIL_Ricochet( ptr->vecEndPos, 1.0 );
				flDamage = 0.01;
			}
		}
		// it's head shot anyways
		ptr->iHitgroup = HITGROUP_HEAD;
	}
		
	
	SpawnBlood(ptr->vecEndPos, BloodColor(), flDamage);
	TraceBleed( flDamage, vecDir, ptr, bitsDamageType );
		
	AddMultiDamage( pevAttacker, this, flDamage, bitsDamageType );

}


//=========================================================
// TakeDamage - overridden for the stormtrooper because the stormtrooper
// needs to forget that he is in cover if he's hurt. (Obviously
// not in a safe place anymore).
//=========================================================
int CStormt :: TakeDamage( entvars_t *pevInflictor, entvars_t *pevAttacker, float flDamage, int bitsDamageType )
{
	Forget( bits_MEMORY_INCOVER );

	
	return CSquadMonster :: TakeDamage ( pevInflictor, pevAttacker, flDamage, bitsDamageType );
}

//=========================================================
// SetYawSpeed - allows each sequence to have a different
// turn rate associated with it.
//=========================================================
void CStormt :: SetYawSpeed ( void )
{
	int ys;

	switch ( m_Activity )
	{
	case ACT_IDLE:	
		ys = 150;		
		break;
	case ACT_RUN:	
		ys = 150;	
		break;
	case ACT_WALK:	
		ys = 180;		
		break;
	case ACT_RANGE_ATTACK1:	
		ys = 120;	
		break;
	case ACT_RANGE_ATTACK2:	
		ys = 120;	
		break;
	case ACT_MELEE_ATTACK1:	
		ys = 120;	
		break;
	case ACT_MELEE_ATTACK2:	
		ys = 120;	
		break;
	case ACT_TURN_LEFT:
	case ACT_TURN_RIGHT:	
		ys = 180;
		break;
	case ACT_GLIDE:
	case ACT_FLY:
		ys = 30;
		break;
	default:
		ys = 90;
		break;
	}

	pev->yaw_speed = ys;
}

void CStormt :: IdleSound( void )
{
	if (FOkToSpeak() && (g_fStormQuestion || RANDOM_LONG(0,1)))
	{
		if (!g_fStormQuestion)
		{
			// ask question or make statement
			switch (RANDOM_LONG(0,2))
			{
			case 0: // check in
				SENTENCEG_PlayRndSz(ENT(pev), "ST_CHECK", STORMT_SENTENCE_VOLUME, ATTN_NORM, 0, m_voicePitch);
				g_fStormQuestion = 1;
				break;
			case 1: // question
				SENTENCEG_PlayRndSz(ENT(pev), "ST_QUEST", STORMT_SENTENCE_VOLUME, ATTN_NORM, 0, m_voicePitch);
				g_fStormQuestion = 2;
				break;
			case 2: // statement
				SENTENCEG_PlayRndSz(ENT(pev), "ST_IDLE", STORMT_SENTENCE_VOLUME, ATTN_NORM, 0, m_voicePitch);
				break;
			}
		}
		else
		{
			switch (g_fStormQuestion)
			{
			case 1: // check in
				SENTENCEG_PlayRndSz(ENT(pev), "ST_CLEAR", STORMT_SENTENCE_VOLUME, ATTN_NORM, 0, m_voicePitch);
				break;
			case 2: // question 
				SENTENCEG_PlayRndSz(ENT(pev), "ST_ANSWER", STORMT_SENTENCE_VOLUME, ATTN_NORM, 0, m_voicePitch);
				break;
			}
			g_fStormQuestion = 0;
		}
		JustSpoke();
	}
}

//=========================================================
// CheckAmmo - overridden for the stormtrooper because he actually
// uses ammo! (base class doesn't)
//=========================================================
void CStormt :: CheckAmmo ( void )
{
	if ( m_cAmmoLoaded <= 0 )
	{
		SetConditions(bits_COND_NO_AMMO_LOADED);
	}
}

//=========================================================
// Classify - donne le type de classification du stormtrooper
// dans la table des relations.
//=========================================================
int	CStormt :: Classify ( void )
{
	return	CLASS_HUMAN_MILITARY;
}

//=========================================================
//=========================================================
CBaseEntity *CStormt :: Kick( void )
{
	TraceResult tr;

	UTIL_MakeVectors( pev->angles );
	Vector vecStart = pev->origin;
	vecStart.z += pev->size.z * 0.5;
	Vector vecEnd = vecStart + (gpGlobals->v_forward * 70);

	UTIL_TraceHull( vecStart, vecEnd, dont_ignore_monsters, head_hull, ENT(pev), &tr );
	
	if ( tr.pHit )
	{
		CBaseEntity *pEntity = CBaseEntity::Instance( tr.pHit );
		return pEntity;
	}

	return NULL;
}

//=========================================================
// GetGunPosition	donne l'extrmit du canon
//=========================================================

Vector CStormt :: GetGunPosition( )
{

	Vector vecArmPos, vecArmDir;

		
	GetAttachment( 0, vecArmPos, vecArmDir ); //point d'attache numro 1 du canon
		
	return vecArmPos;
/*	
	if (m_fStanding )
	{
		return pev->origin + Vector( 0, 0, 60 );
	}
	else
	{
		return pev->origin + Vector( 0, 0, 48 );
	}
*/
}

//=========================================================
// Shoot Strifle
//=========================================================
void CStormt :: Shoot ( void )
{
	if (m_hEnemy == NULL)
	{
		return;
	}

	Vector vecShootOrigin = GetGunPosition();
	Vector vecShootDir = ShootAtEnemy( vecShootOrigin );

	UTIL_MakeVectors ( pev->angles );
	
	// code pour tirer une balle normale
	//FireBullets(1, vecShootOrigin, vecShootDir, VECTOR_CONE_10DEGREES, 2048, BULLET_MONSTER_STRF ); // shoot +-5 degrs

	// code pour tirer un projectile de blaster
	// ajout de la dispersion du tir automatique rapide

	Vector vecRight = gpGlobals->v_right;
	Vector vecUp = gpGlobals->v_up;

	float x, y, z;
	
			x = RANDOM_FLOAT(-0.5,0.5) + RANDOM_FLOAT(-0.5,0.5);
			y = RANDOM_FLOAT(-0.5,0.5) + RANDOM_FLOAT(-0.5,0.5);
			z = x*x+y*y;
	
	Vector vecSpread = VECTOR_CONE_5DEGREES ;
	vecShootDir = vecShootDir + x * vecSpread.x * vecRight + y * vecSpread.y * vecUp ;


	CMBlaster::Shoot(pev,vecShootOrigin,vecShootDir * 2000);

	pev->effects |= EF_MUZZLEFLASH;
	
	m_cAmmoLoaded--;// retire un coup du chargeur!

	Vector angDir = UTIL_VecToAngles( vecShootDir );
	SetBlending( 0, angDir.x ); // donne l'inclinaison de tir du stormtrooper pour l'animation
}

//=========================================================
// Shoot Railgun
//=========================================================
void CStormt :: Railgun ( void )
{
	TraceResult tr;

	if (m_hEnemy == NULL)
	{
		return;
	}
	
	Vector vecShootOrigin = GetGunPosition();
	Vector vecShootDir = ShootAtEnemy( vecShootOrigin ) ;
	
	
	
	direction = vecShootDir; // pour que le stormtrooper tire bien dans la bonne direction
	
	
	// cration d'un projectile au dpart du coup
	CSRailRocket *pSRRocket = CSRailRocket::CreateSRailRocket( vecShootOrigin, vecShootDir, pev, this );

		
	// code pour tirer une balle
	//FireBullets(1, vecShootOrigin, vecShootDir, VECTOR_CONE_1DEGREES, 2048, BULLET_PLAYER_RAILCHARGE, 0 ); // shoot +-7.5 degrees

	pev->effects |= EF_MUZZLEFLASH;
	
	m_cAmmoLoaded--;// take away a bullet!

	Vector angDir = UTIL_VecToAngles( vecShootDir );
	SetBlending( 0, angDir.x );
}

//=========================================================
// HandleAnimEvent - catches the monster-specific messages
// that occur when tagged animation frames are played.
//=========================================================
void CStormt :: HandleAnimEvent( MonsterEvent_t *pEvent )
{
	Vector	vecShootDir;
	Vector	vecShootOrigin;

	switch( pEvent->event )
	{
		case STORMT_AE_DROP_GUN:
			{
			Vector	vecGunPos;
			Vector	vecGunAngles;

			GetAttachment( 0, vecGunPos, vecGunAngles );

			// switch to body group with no gun.
			SetBodygroup( GUN_GROUP, GUN_NONE );

			// now spawn a gun.
			if (FBitSet( pev->weapons, STORMT_RAILGUN ))
			{
				 DropItem( "weapon_railgun", vecGunPos, vecGunAngles );
			}
			else
			{
				 DropItem( "weapon_strifle", vecGunPos, vecGunAngles );
			}

			}
			break;

		case STORMT_AE_RELOAD:
		
			if (FBitSet( pev->weapons, STORMT_RAILGUN ))
			{
				EMIT_SOUND( ENT(pev), CHAN_WEAPON, "stormt/rail_reload.wav", 1, ATTN_NORM );
				m_cAmmoLoaded = m_cClipSize;
				m_fThrowGrenade = FALSE;
				m_flNextGrenadeCheck = gpGlobals->time + 1;
			}

			else
			{
				EMIT_SOUND( ENT(pev), CHAN_WEAPON, "stormt/st_reload1.wav", 1, ATTN_NORM );
				m_cAmmoLoaded = m_cClipSize;
			}

			ClearConditions(bits_COND_NO_AMMO_LOADED);
			break;

		case STORMT_AE_GREN_TOSS:
		{
			UTIL_MakeVectors( pev->angles );
			// CGrenade::ShootTimed( pev, pev->origin + gpGlobals->v_forward * 34 + Vector (0, 0, 32), m_vecTossVelocity, 3.5 );
			CThermal::ShootTimed( pev, GetGunPosition(), m_vecTossVelocity, 3.5 );

			m_fThrowGrenade = FALSE;
			m_flNextGrenadeCheck = gpGlobals->time + 6;// wait six seconds before even looking again to see if a grenade can be thrown.
			// !!!LATER - when in a group, only try to throw grenade if ordered.
		}
		break;



		case STORMT_AE_GREN_DROP:
		{
			UTIL_MakeVectors( pev->angles );
			CThermal::ShootTimed( pev, pev->origin + gpGlobals->v_forward * 17 - gpGlobals->v_right * 27 + gpGlobals->v_up * 6, g_vecZero, 3 );
		}
		break;

		case STORMT_AE_BURST1:
		{
			if ( FBitSet( pev->weapons, STORMT_STRIFLE ))
			{
				Shoot();

				
				if ( RANDOM_LONG(0,1) )
				{
					EMIT_SOUND( ENT(pev), CHAN_WEAPON, "weapons/strf1.wav", 1, ATTN_NORM );
				}
				else
				{
					EMIT_SOUND( ENT(pev), CHAN_WEAPON, "weapons/strf2.wav", 1, ATTN_NORM );
				}
			}
			else
			{
				Railgun( );

				EMIT_SOUND(ENT(pev), CHAN_WEAPON, "weapons/railstart.wav", 1, ATTN_NORM );
				EMIT_SOUND(ENT(pev), CHAN_ITEM, "weapons/raillaunch.wav", 1, ATTN_NORM );

				m_fThrowGrenade = FALSE;
				m_flNextGrenadeCheck = gpGlobals->time + 1;
			}
		
			CSoundEnt::InsertSound ( bits_SOUND_COMBAT, pev->origin, 384, 0.3 );
		}
		break;

		case STORMT_AE_BURST2:
		
			Shoot();

			if ( RANDOM_LONG(0,1) )
				{
					EMIT_SOUND( ENT(pev), CHAN_WEAPON, "weapons/strf1.wav", 1, ATTN_NORM );
				}
				else
				{
					EMIT_SOUND( ENT(pev), CHAN_WEAPON, "weapons/strf2.wav", 1, ATTN_NORM );
				}

			break;

		case STORMT_AE_KICK:
		{
			CBaseEntity *pHurt = Kick();

			if ( pHurt )
			{
				// SOUND HERE!
				UTIL_MakeVectors( pev->angles );
				pHurt->pev->punchangle.x = 15;
				pHurt->pev->velocity = pHurt->pev->velocity + gpGlobals->v_forward * 100 + gpGlobals->v_up * 50;
				pHurt->TakeDamage( pev, pev, gSkillData.stormtDmgKick, DMG_CLUB );
			}
		}
		break;

		case STORMT_AE_CAUGHT_ENEMY:
		{
			if ( FOkToSpeak() )
			{
				SENTENCEG_PlayRndSz(ENT(pev), "ST_ALERT", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
				 JustSpoke();
			}

		}

		default:
			CSquadMonster::HandleAnimEvent( pEvent );
			break;
	}
}

//=========================================================
// Spawn
//=========================================================
void CStormt :: Spawn()
{
	Precache( );

	SET_MODEL(ENT(pev), "models/stormt.mdl");
	UTIL_SetSize(pev, VEC_HUMAN_HULL_MIN, VEC_HUMAN_HULL_MAX);

	pev->solid			= SOLID_SLIDEBOX;
	pev->movetype		= MOVETYPE_STEP;
	m_bloodColor		= BLOOD_COLOR_RED;
	pev->effects		= 0;
	pev->health			= gSkillData.stormtHealth;
	m_flFieldOfView		= 0.2;// indicates the width of this monster's forward view cone ( as a dotproduct result )
	m_MonsterState		= MONSTERSTATE_NONE;
	m_flNextGrenadeCheck = gpGlobals->time + 1;
	m_flNextPainTime	= gpGlobals->time;
	m_iSentence			= STORMT_SENT_NONE;

	m_afCapability		= bits_CAP_SQUAD | bits_CAP_TURN_HEAD | bits_CAP_DOORS_GROUP;

	m_fEnemyEluded		= FALSE;
	m_fFirstEncounter	= TRUE;// this is true when the stormtrooper spawns, because he hasn't encountered an enemy yet.

	m_HackedGunPos = Vector ( 0, 0, 55 );

	pev->iuser3 = 1;//masse
	pev->iuser4	= 1;//incendie

	

	if (pev->weapons == 0)
	{
		// initialize to original values
		pev->weapons = STORMT_STRIFLE | STORMT_HANDGRENADE;
		// pev->weapons = STORMT_RAILGUN;
		// pev->weapons = STORMT_STRIFLE;
	}

	if (FBitSet( pev->weapons, STORMT_RAILGUN ))
	{
		SetBodygroup( GUN_GROUP, GUN_RAILGUN );
		m_cClipSize		= 3;
		STORMT_AE_RELOAD == STORMT_AE_RELOAD2;
	}
	else
	{
		m_cClipSize		= STORM_CLIP_SIZE;
	}
	m_cAmmoLoaded		= m_cClipSize;

	if (RANDOM_LONG( 0, 99 ) < 80)
		pev->skin = 0;	// light skin
	else
		pev->skin = 1;	// dark skin

	if (FBitSet( pev->weapons, STORMT_RAILGUN ))
	{
		SetBodygroup( HEAD_GROUP, HEAD_RAILGUN);
	}
	

	CTalkMonster::g_talkWaitTime = 0;

	MonsterInit();
}

//=========================================================
// Precache - precaches all resources this monster needs
//=========================================================
void CStormt :: Precache()
{
	PRECACHE_MODEL("models/stormt.mdl");

	PRECACHE_SOUND( "weapons/strf1.wav" );
	PRECACHE_SOUND( "weapons/strf2.wav" );
	
	PRECACHE_SOUND( "stormt/st_die1.wav" );
	PRECACHE_SOUND( "stormt/st_die2.wav" );
	PRECACHE_SOUND( "stormt/st_die3.wav" );

	PRECACHE_SOUND( "stormt/st_pain1.wav" );
	PRECACHE_SOUND( "stormt/st_pain2.wav" );
	PRECACHE_SOUND( "stormt/st_pain3.wav" );
	PRECACHE_SOUND( "stormt/st_pain4.wav" );
	PRECACHE_SOUND( "stormt/st_pain5.wav" );
	PRECACHE_SOUND ("weapons/choc1.wav");
	PRECACHE_SOUND ("weapons/choc2.wav");
	PRECACHE_SOUND ("weapons/choc3.wav");
	PRECACHE_SOUND ("weapons/choc4.wav");
	PRECACHE_SOUND ("weapons/grill.wav");

	PRECACHE_SOUND( "stormt/st_reload1.wav" );
	PRECACHE_SOUND( "stormt/rail_reload.wav" );

	PRECACHE_SOUND( "weapons/railstart.wav" );
	PRECACHE_SOUND( "weapons/raillaunch.wav" );


	PRECACHE_SOUND("zombie/claw_miss2.wav");// because we use the basemonster SWIPE animation event

	m_Misprite = PRECACHE_MODEL( "sprites/hotglow.spr" ); //pour le tir du blaster
	m_Mismoke = PRECACHE_MODEL( "sprites/blasthit.spr" ); //pour l'impacte du blaster
	m_iSTrail = PRECACHE_MODEL( "sprites/zbeam4.spr" );
	
	PRECACHE_MODEL( "models/w_thermal.mdl" );
	PRECACHE_MODEL( "sprites/blaster.spr" );
	m_StWallSmoke = PRECACHE_MODEL( "sprites/bexplo.spr" );


	// get voice pitch
	if (RANDOM_LONG(0,1))
		m_voicePitch = 109 + RANDOM_LONG(0,7);
	else
		m_voicePitch = 100;
}	

//=========================================================
// start task
//=========================================================
void CStormt :: StartTask ( Task_t *pTask )
{
	m_iTaskStatus = TASKSTATUS_RUNNING;

	switch ( pTask->iTask )
	{
	case TASK_STORM_CHECK_FIRE:
		if ( !NoFriendlyFire() )
		{
			SetConditions( bits_COND_STORM_NOFIRE );
		}
		if ( FBitSet( pev->weapons, STORMT_RAILGUN) && !m_fThrowGrenade ) //pas de tir de railgun
		{
			SetConditions( bits_COND_STORM_NOFIRE );
		}
		TaskComplete();
		break;

	case TASK_STORM_SPEAK_SENTENCE:
		SpeakSentence();
		TaskComplete();
		break;
	
	case TASK_WALK_PATH:
	case TASK_RUN_PATH:
		// stormtrooper no longer assumes he is covered if he moves
		Forget( bits_MEMORY_INCOVER );
		CSquadMonster ::StartTask( pTask );
		break;

	case TASK_RELOAD:
		m_IdealActivity = ACT_RELOAD;
		break;

	case TASK_STORM_FACE_TOSS_DIR:
		break;

	case TASK_FACE_IDEAL:
	case TASK_FACE_ENEMY:
		CSquadMonster :: StartTask( pTask );
		if (pev->movetype == MOVETYPE_FLY)
		{
			m_IdealActivity = ACT_GLIDE;
		}
		break;

	default: 
		CSquadMonster :: StartTask( pTask );
		break;
	}
}

//=========================================================
// RunTask
//=========================================================
void CStormt :: RunTask ( Task_t *pTask )
{
	switch ( pTask->iTask )
	{
	case TASK_STORM_FACE_TOSS_DIR:
		{
			// project a point along the toss vector and turn to face that point.
			MakeIdealYaw( pev->origin + m_vecTossVelocity * 64 );
			ChangeYaw( pev->yaw_speed );

			if ( FacingIdeal() )
			{
				m_iTaskStatus = TASKSTATUS_COMPLETE;
			}
			break;
		}
	default:
		{
			CSquadMonster :: RunTask( pTask );
			break;
		}
	}
}

//=========================================================
// PainSound
//=========================================================
void CStormt :: PainSound ( void )
{
	if ( gpGlobals->time > m_flNextPainTime )
	{
#if 0
		if ( RANDOM_LONG(0,99) < 5 )
		{
			// pain sentences are rare
			if (FOkToSpeak())
			{
				SENTENCEG_PlayRndSz(ENT(pev), "ST_PAIN", STORMT_SENTENCE_VOLUME, ATTN_NORM, 0, PITCH_NORM);
				JustSpoke();
				return;
			}
		}
#endif 
		switch ( RANDOM_LONG(0,6) )
		{
		case 0:	
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_pain3.wav", 1, ATTN_NORM );	
			break;
		case 1:
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_pain4.wav", 1, ATTN_NORM );	
			break;
		case 2:
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_pain5.wav", 1, ATTN_NORM );	
			break;
		case 3:
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_pain1.wav", 1, ATTN_NORM );	
			break;
		case 4:
			EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_pain2.wav", 1, ATTN_NORM );	
			break;
		}

		m_flNextPainTime = gpGlobals->time + 1;
	}
}

//=========================================================
// DeathSound 
//=========================================================
void CStormt :: DeathSound ( void )
{
	switch ( RANDOM_LONG(0,2) )
	{
	case 0:	
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_die1.wav", 1, ATTN_IDLE );	
		break;
	case 1:
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_die2.wav", 1, ATTN_IDLE );	
		break;
	case 2:
		EMIT_SOUND( ENT(pev), CHAN_VOICE, "stormt/st_die3.wav", 1, ATTN_IDLE );	
		break;
	}
}

//=========================================================
// AI Schedules Specific to this monster
//=========================================================

//=========================================================
// StormFail
//=========================================================
Task_t	tlStormFail[] =
{
	{ TASK_STOP_MOVING,			0				},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE },
	{ TASK_WAIT,				(float)2		},
	{ TASK_WAIT_PVS,			(float)0		},
};

Schedule_t	slStormFail[] =
{
	{
		tlStormFail,
		ARRAYSIZE ( tlStormFail ),
		bits_COND_CAN_RANGE_ATTACK1 |
		bits_COND_CAN_RANGE_ATTACK2 |
		bits_COND_CAN_MELEE_ATTACK1 |
		bits_COND_CAN_MELEE_ATTACK2,
		0,
		"Storm Fail"
	},
};

//=========================================================
// Storm Combat Fail
//=========================================================
Task_t	tlStormCombatFail[] =
{
	{ TASK_STOP_MOVING,			0				},
	{ TASK_SET_ACTIVITY,		(float)ACT_IDLE },
	{ TASK_WAIT_FACE_ENEMY,		(float)2		},
	{ TASK_WAIT_PVS,			(float)0		},
};

Schedule_t	slStormCombatFail[] =
{
	{
		tlStormCombatFail,
		ARRAYSIZE ( tlStormCombatFail ),
		bits_COND_CAN_RANGE_ATTACK1	|
		bits_COND_CAN_RANGE_ATTACK2,
		0,
		"Storm Combat Fail"
	},
};

//=========================================================
// Victory dance!
//=========================================================
Task_t	tlStormVictoryDance[] =
{
	{ TASK_STOP_MOVING,						(float)0					},
	{ TASK_FACE_ENEMY,						(float)0					},
	{ TASK_WAIT,							(float)1.5					},
	{ TASK_GET_PATH_TO_ENEMY_CORPSE,		(float)0					},
	{ TASK_WALK_PATH,						(float)0					},
	{ TASK_WAIT_FOR_MOVEMENT,				(float)0					},
	{ TASK_FACE_ENEMY,						(float)0					},
	{ TASK_PLAY_SEQUENCE,					(float)ACT_VICTORY_DANCE	},
};

Schedule_t	slStormVictoryDance[] =
{
	{ 
		tlStormVictoryDance,
		ARRAYSIZE ( tlStormVictoryDance ), 
		bits_COND_NEW_ENEMY		|
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE,
		0,
		"StormVictoryDance"
	},
};

//=========================================================
// Establish line of fire - move to a position that allows
// the stormtrooper to attack.
//=========================================================
Task_t tlStormEstablishLineOfFire[] = 
{
	{ TASK_SET_FAIL_SCHEDULE,	(float)SCHED_STORM_ELOF_FAIL	},
	{ TASK_GET_PATH_TO_ENEMY,	(float)0						},
	{ TASK_STORM_SPEAK_SENTENCE,(float)0						},
	{ TASK_RUN_PATH,			(float)0						},
	{ TASK_WAIT_FOR_MOVEMENT,	(float)0						},
};

Schedule_t slStormEstablishLineOfFire[] =
{
	{ 
		tlStormEstablishLineOfFire,
		ARRAYSIZE ( tlStormEstablishLineOfFire ),
		bits_COND_NEW_ENEMY			|
		bits_COND_ENEMY_DEAD		|
		bits_COND_CAN_RANGE_ATTACK1	|
		bits_COND_CAN_MELEE_ATTACK1	|
		bits_COND_CAN_RANGE_ATTACK2	|
		bits_COND_CAN_MELEE_ATTACK2	|
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER,
		"StormEstablishLineOfFire"
	},
};

//=========================================================
// StormFoundEnemy - stormtrooper established sight with an enemy
// that was hiding from the squad.
//=========================================================
Task_t	tlStormFoundEnemy[] =
{
	{ TASK_STOP_MOVING,				0							},
	{ TASK_FACE_ENEMY,				(float)0					},
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,(float)ACT_SIGNAL1			},
};

Schedule_t	slStormFoundEnemy[] =
{
	{ 
		tlStormFoundEnemy,
		ARRAYSIZE ( tlStormFoundEnemy ), 
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER,
		"StormFoundEnemy"
	},
};

//=========================================================
// StormCombatFace Schedule
//=========================================================
Task_t	tlStormCombatFace1[] =
{
	{ TASK_STOP_MOVING,				0							},
	{ TASK_SET_ACTIVITY,			(float)ACT_IDLE				},
	{ TASK_FACE_ENEMY,				(float)0					},
	{ TASK_WAIT,					(float)1.5					},
	{ TASK_SET_SCHEDULE,			(float)SCHED_STORM_SWEEP	},
};

Schedule_t	slStormCombatFace[] =
{
	{ 
		tlStormCombatFace1,
		ARRAYSIZE ( tlStormCombatFace1 ), 
		bits_COND_NEW_ENEMY				|
		bits_COND_ENEMY_DEAD			|
		bits_COND_CAN_RANGE_ATTACK1		|
		bits_COND_CAN_RANGE_ATTACK2,
		0,
		"Combat Face"
	},
};

//=========================================================
// Suppressing fire - don't stop shooting until the clip is
// empty or stormtrooper gets hurt.
//=========================================================
Task_t	tlStormSignalSuppress[] =
{
	{ TASK_STOP_MOVING,					0						},
	{ TASK_FACE_IDEAL,					(float)0				},
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,	(float)ACT_SIGNAL2		},
	{ TASK_FACE_ENEMY,					(float)0				},
	{ TASK_STORM_CHECK_FIRE,			(float)0				},
	{ TASK_RANGE_ATTACK1,				(float)0				},
	{ TASK_FACE_ENEMY,					(float)0				},
	{ TASK_STORM_CHECK_FIRE,			(float)0				},
	{ TASK_RANGE_ATTACK1,				(float)0				},
	{ TASK_FACE_ENEMY,					(float)0				},
	{ TASK_STORM_CHECK_FIRE,			(float)0				},
	{ TASK_RANGE_ATTACK1,				(float)0				},
	{ TASK_FACE_ENEMY,					(float)0				},
	{ TASK_STORM_CHECK_FIRE,			(float)0				},
	{ TASK_RANGE_ATTACK1,				(float)0				},
	{ TASK_FACE_ENEMY,					(float)0				},
	{ TASK_STORM_CHECK_FIRE,			(float)0				},
	{ TASK_RANGE_ATTACK1,				(float)0				},
};

Schedule_t	slStormSignalSuppress[] =
{
	{ 
		tlStormSignalSuppress,
		ARRAYSIZE ( tlStormSignalSuppress ), 
		bits_COND_ENEMY_DEAD		|
		bits_COND_LIGHT_DAMAGE		|
		bits_COND_HEAVY_DAMAGE		|
		bits_COND_HEAR_SOUND		|
		bits_COND_STORM_NOFIRE		|
		bits_COND_NO_AMMO_LOADED,

		bits_SOUND_DANGER,
		"SignalSuppress"
	},
};

Task_t	tlStormSuppress[] =
{
	{ TASK_STOP_MOVING,			0							},
	{ TASK_FACE_ENEMY,			(float)0					},
	{ TASK_STORM_CHECK_FIRE,	(float)0					},
	{ TASK_RANGE_ATTACK1,		(float)0					},
	{ TASK_FACE_ENEMY,			(float)0					},
	{ TASK_STORM_CHECK_FIRE,	(float)0					},
	{ TASK_RANGE_ATTACK1,		(float)0					},
	{ TASK_FACE_ENEMY,			(float)0					},
	{ TASK_STORM_CHECK_FIRE,	(float)0					},
	{ TASK_RANGE_ATTACK1,		(float)0					},
	{ TASK_FACE_ENEMY,			(float)0					},
	{ TASK_STORM_CHECK_FIRE,	(float)0					},
	{ TASK_RANGE_ATTACK1,		(float)0					},
	{ TASK_FACE_ENEMY,			(float)0					},
	{ TASK_STORM_CHECK_FIRE,	(float)0					},
	{ TASK_RANGE_ATTACK1,		(float)0					},
};

Schedule_t	slStormSuppress[] =
{
	{ 
		tlStormSuppress,
		ARRAYSIZE ( tlStormSuppress ), 
		bits_COND_ENEMY_DEAD		|
		bits_COND_LIGHT_DAMAGE		|
		bits_COND_HEAVY_DAMAGE		|
		bits_COND_HEAR_SOUND		|
		bits_COND_STORM_NOFIRE		|
		bits_COND_NO_AMMO_LOADED,

		bits_SOUND_DANGER,
		"Suppress"
	},
};


//=========================================================
// stormtrooper wait in cover - we don't allow danger or the ability
// to attack to break a stormtrooper's run to cover schedule, but
// when a stormtrooper is in cover, we do want them to attack if they can.
//=========================================================
Task_t	tlStormWaitInCover[] =
{
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_SET_ACTIVITY,			(float)ACT_IDLE				},
	{ TASK_WAIT_FACE_ENEMY,			(float)1					},
};

Schedule_t	slStormWaitInCover[] =
{
	{ 
		tlStormWaitInCover,
		ARRAYSIZE ( tlStormWaitInCover ), 
		bits_COND_NEW_ENEMY			|
		bits_COND_HEAR_SOUND		|
		bits_COND_CAN_RANGE_ATTACK1	|
		bits_COND_CAN_RANGE_ATTACK2	|
		bits_COND_CAN_MELEE_ATTACK1	|
		bits_COND_CAN_MELEE_ATTACK2,

		bits_SOUND_DANGER,
		"StormWaitInCover"
	},
};

//=========================================================
// run to cover.
// !!!BUGBUG - set a decent fail schedule here.
//=========================================================
Task_t	tlStormTakeCover1[] =
{
	{ TASK_STOP_MOVING,				(float)0							},
	{ TASK_SET_FAIL_SCHEDULE,		(float)SCHED_STORM_TAKECOVER_FAILED	},
	{ TASK_WAIT,					(float)0.2							},
	{ TASK_FIND_COVER_FROM_ENEMY,	(float)0							},
	{ TASK_STORM_SPEAK_SENTENCE,	(float)0							},
	{ TASK_RUN_PATH,				(float)0							},
	{ TASK_WAIT_FOR_MOVEMENT,		(float)0							},
	{ TASK_REMEMBER,				(float)bits_MEMORY_INCOVER			},
	{ TASK_SET_SCHEDULE,			(float)SCHED_STORM_WAIT_FACE_ENEMY	},
};

Schedule_t	slStormTakeCover[] =
{
	{ 
		tlStormTakeCover1,
		ARRAYSIZE ( tlStormTakeCover1 ), 
		0,
		0,
		"TakeCover"
	},
};

//=========================================================
// drop grenade then run to cover.
//=========================================================
Task_t	tlStormGrenadeCover1[] =
{
	{ TASK_STOP_MOVING,						(float)0							},
	{ TASK_FIND_COVER_FROM_ENEMY,			(float)99							},
	{ TASK_FIND_FAR_NODE_COVER_FROM_ENEMY,	(float)384							},
	{ TASK_PLAY_SEQUENCE,					(float)ACT_SPECIAL_ATTACK1			},
	{ TASK_CLEAR_MOVE_WAIT,					(float)0							},
	{ TASK_RUN_PATH,						(float)0							},
	{ TASK_WAIT_FOR_MOVEMENT,				(float)0							},
	{ TASK_SET_SCHEDULE,					(float)SCHED_STORM_WAIT_FACE_ENEMY	},
};

Schedule_t	slStormGrenadeCover[] =
{
	{ 
		tlStormGrenadeCover1,
		ARRAYSIZE ( tlStormGrenadeCover1 ), 
		0,
		0,
		"GrenadeCover"
	},
};


//=========================================================
// drop grenade then run to cover.
//=========================================================
Task_t	tlStormTossGrenadeCover1[] =
{
	{ TASK_FACE_ENEMY,						(float)0							},
	{ TASK_RANGE_ATTACK2, 					(float)0							},
	{ TASK_SET_SCHEDULE,					(float)SCHED_TAKE_COVER_FROM_ENEMY	},
};

Schedule_t	slStormTossGrenadeCover[] =
{
	{ 
		tlStormTossGrenadeCover1,
		ARRAYSIZE ( tlStormTossGrenadeCover1 ), 
		0,
		0,
		"TossGrenadeCover"
	},
};

//=========================================================
// hide from the loudest sound source (to run from grenade)
//=========================================================
Task_t	tlStormTakeCoverFromBestSound[] =
{
	{ TASK_SET_FAIL_SCHEDULE,			(float)SCHED_COWER			},// duck and cover if cannot move from explosion
	{ TASK_STOP_MOVING,					(float)0					},
	{ TASK_FIND_COVER_FROM_BEST_SOUND,	(float)0					},
	{ TASK_RUN_PATH,					(float)0					},
	{ TASK_WAIT_FOR_MOVEMENT,			(float)0					},
	{ TASK_REMEMBER,					(float)bits_MEMORY_INCOVER	},
	{ TASK_TURN_LEFT,					(float)179					},
};

Schedule_t	slStormTakeCoverFromBestSound[] =
{
	{ 
		tlStormTakeCoverFromBestSound,
		ARRAYSIZE ( tlStormTakeCoverFromBestSound ), 
		0,
		0,
		"StormTakeCoverFromBestSound"
	},
};

//=========================================================
// Storm reload schedule
//=========================================================
Task_t	tlStormHideReload[] =
{
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_SET_FAIL_SCHEDULE,		(float)SCHED_RELOAD			},
	{ TASK_FIND_COVER_FROM_ENEMY,	(float)0					},
	{ TASK_RUN_PATH,				(float)0					},
	{ TASK_WAIT_FOR_MOVEMENT,		(float)0					},
	{ TASK_REMEMBER,				(float)bits_MEMORY_INCOVER	},
	{ TASK_FACE_ENEMY,				(float)0					},
	{ TASK_PLAY_SEQUENCE,			(float)ACT_RELOAD			},
};

Schedule_t slStormHideReload[] = 
{
	{
		tlStormHideReload,
		ARRAYSIZE ( tlStormHideReload ),
		bits_COND_HEAVY_DAMAGE	|
		bits_COND_HEAR_SOUND,

		bits_SOUND_DANGER,
		"StormHideReload"
	}
};

//=========================================================
// Do a turning sweep of the area
//=========================================================
Task_t	tlStormSweep[] =
{
	{ TASK_TURN_LEFT,			(float)179	},
	{ TASK_WAIT,				(float)1	},
	{ TASK_TURN_LEFT,			(float)179	},
	{ TASK_WAIT,				(float)1	},
};

Schedule_t	slStormSweep[] =
{
	{ 
		tlStormSweep,
		ARRAYSIZE ( tlStormSweep ), 
		
		bits_COND_NEW_ENEMY		|
		bits_COND_LIGHT_DAMAGE	|
		bits_COND_HEAVY_DAMAGE	|
		bits_COND_CAN_RANGE_ATTACK1	|
		bits_COND_CAN_RANGE_ATTACK2	|
		bits_COND_HEAR_SOUND,

		bits_SOUND_WORLD		|// sound flags
		bits_SOUND_DANGER		|
		bits_SOUND_PLAYER,

		"Storm Sweep"
	},
};

//=========================================================
// primary range attack. Overriden because base class stops attacking when the enemy is occluded.
// stormtrooper's grenade toss requires the enemy be occluded.
//=========================================================
Task_t	tlStormRangeAttack1A[] =
{
	{ TASK_STOP_MOVING,			(float)0		},
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,		(float)ACT_CROUCH },
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
};

Schedule_t	slStormRangeAttack1A[] =
{
	{ 
		tlStormRangeAttack1A,
		ARRAYSIZE ( tlStormRangeAttack1A ), 
		bits_COND_NEW_ENEMY			|
		bits_COND_ENEMY_DEAD		|
		bits_COND_HEAVY_DAMAGE		|
		bits_COND_ENEMY_OCCLUDED	|
		bits_COND_HEAR_SOUND		|
		bits_COND_STORM_NOFIRE		|
		bits_COND_NO_AMMO_LOADED,
		
		bits_SOUND_DANGER,
		"Range Attack1A"
	},
};


//=========================================================
// primary range attack. Overriden because base class stops attacking when the enemy is occluded.
// stormtrooper's grenade toss requires the enemy be occluded.
//=========================================================
Task_t	tlStormRangeAttack1B[] =
{
	{ TASK_STOP_MOVING,				(float)0		},
	{ TASK_PLAY_SEQUENCE_FACE_ENEMY,(float)ACT_IDLE_ANGRY  },
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_STORM_CHECK_FIRE,	(float)0		},
	{ TASK_RANGE_ATTACK1,		(float)0		},
};

Schedule_t	slStormRangeAttack1B[] =
{
	{ 
		tlStormRangeAttack1B,
		ARRAYSIZE ( tlStormRangeAttack1B ), 
		bits_COND_NEW_ENEMY			|
		bits_COND_ENEMY_DEAD		|
		bits_COND_HEAVY_DAMAGE		|
		bits_COND_ENEMY_OCCLUDED	|
		bits_COND_NO_AMMO_LOADED	|
		bits_COND_STORM_NOFIRE		|
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER,
		"Range Attack1B"
	},
};

//=========================================================
// secondary range attack. Overriden because base class stops attacking when the enemy is occluded.
// stormtrooper's grenade toss requires the enemy be occluded.
//=========================================================
Task_t	tlStormRangeAttack2[] =
{
	{ TASK_STOP_MOVING,				(float)0					},
	{ TASK_STORM_FACE_TOSS_DIR,		(float)0					},
	{ TASK_PLAY_SEQUENCE,			(float)ACT_RANGE_ATTACK2	},
	{ TASK_SET_SCHEDULE,			(float)SCHED_STORM_WAIT_FACE_ENEMY	},// don't run immediately after throwing grenade.
};

Schedule_t	slStormRangeAttack2[] =
{
	{ 
		tlStormRangeAttack2,
		ARRAYSIZE ( tlStormRangeAttack2 ), 
		0,
		0,
		"RangeAttack2"
	},
};


//=========================================================
// repel 
//=========================================================
Task_t	tlStormRepel[] =
{
	{ TASK_STOP_MOVING,			(float)0		},
	{ TASK_FACE_IDEAL,			(float)0		},
	{ TASK_PLAY_SEQUENCE,		(float)ACT_GLIDE 	},
};

Schedule_t	slStormRepel[] =
{
	{ 
		tlStormRepel,
		ARRAYSIZE ( tlStormRepel ), 
		bits_COND_SEE_ENEMY			|
		bits_COND_NEW_ENEMY			|
		bits_COND_LIGHT_DAMAGE		|
		bits_COND_HEAVY_DAMAGE		|
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER			|
		bits_SOUND_COMBAT			|
		bits_SOUND_PLAYER, 
		"Repel"
	},
};


//=========================================================
// repel 
//=========================================================
Task_t	tlStormRepelAttack[] =
{
	{ TASK_STOP_MOVING,			(float)0		},
	{ TASK_FACE_ENEMY,			(float)0		},
	{ TASK_PLAY_SEQUENCE,		(float)ACT_FLY 	},
};

Schedule_t	slStormRepelAttack[] =
{
	{ 
		tlStormRepelAttack,
		ARRAYSIZE ( tlStormRepelAttack ), 
		bits_COND_ENEMY_OCCLUDED,
		0,
		"Repel Attack"
	},
};

//=========================================================
// repel land
//=========================================================
Task_t	tlStormRepelLand[] =
{
	{ TASK_STOP_MOVING,			(float)0		},
	{ TASK_PLAY_SEQUENCE,		(float)ACT_LAND	},
	{ TASK_GET_PATH_TO_LASTPOSITION,(float)0				},
	{ TASK_RUN_PATH,				(float)0				},
	{ TASK_WAIT_FOR_MOVEMENT,		(float)0				},
	{ TASK_CLEAR_LASTPOSITION,		(float)0				},
};

Schedule_t	slStormRepelLand[] =
{
	{ 
		tlStormRepelLand,
		ARRAYSIZE ( tlStormRepelLand ), 
		bits_COND_SEE_ENEMY			|
		bits_COND_NEW_ENEMY			|
		bits_COND_LIGHT_DAMAGE		|
		bits_COND_HEAVY_DAMAGE		|
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER			|
		bits_SOUND_COMBAT			|
		bits_SOUND_PLAYER, 
		"Repel Land"
	},
};


///////////
//Recherche
///////////

Task_t tlStormRecherche[] =
{
	{ TASK_GET_PATH_TO_SPOT,		(float)0		},
	{ TASK_RUN_PATH,				(float)0		},
	{ TASK_WAIT_FOR_MOVEMENT,		(float)0		},
};

Schedule_t slStormRecherche[] =
{
	{
		tlStormRecherche,
		ARRAYSIZE ( tlStormRecherche ),
		bits_COND_NEW_ENEMY			|
		bits_COND_ENEMY_DEAD		|
		bits_COND_CAN_RANGE_ATTACK1	|
		bits_COND_CAN_MELEE_ATTACK1	|
		bits_COND_HEAR_SOUND,
		
		bits_SOUND_DANGER,
		"StormRecherche"
	},
};


DEFINE_CUSTOM_SCHEDULES( CStormt )
{
	slStormFail,
	slStormCombatFail,
	slStormVictoryDance,
	slStormEstablishLineOfFire,
	slStormFoundEnemy,
	slStormCombatFace,
	slStormSignalSuppress,
	slStormSuppress,
	slStormWaitInCover,
	slStormTakeCover,
	slStormGrenadeCover,
	slStormTossGrenadeCover,
	slStormTakeCoverFromBestSound,
	slStormHideReload,
	slStormSweep,
	slStormRangeAttack1A,
	slStormRangeAttack1B,
	slStormRangeAttack2,
	slStormRepel,
	slStormRepelAttack,
	slStormRepelLand,
	slStormRecherche,
};

IMPLEMENT_CUSTOM_SCHEDULES( CStormt, CSquadMonster );

//=========================================================
// SetActivity 
//=========================================================
void CStormt :: SetActivity ( Activity NewActivity )
{
	int	iSequence = ACTIVITY_NOT_AVAILABLE;
	void *pmodel = GET_MODEL_PTR( ENT(pev) );

	switch ( NewActivity)
	{
	case ACT_RANGE_ATTACK1:
		// stormtrooper is either shooting standing or shooting crouched
		if (FBitSet( pev->weapons, STORMT_STRIFLE))
		{
			if ( m_fStanding )
			{
				// get aimable sequence
				iSequence = LookupSequence( "standing_strifle" );
			}
			else
			{
				// get crouching shoot
				iSequence = LookupSequence( "crouching_strifle" );
			}
		}
		else
		{
			if ( m_fStanding )
			{
				// get aimable sequence
				iSequence = LookupSequence( "standing_railgun" );
			}
			else
			{
				// get crouching shoot
				iSequence = LookupSequence( "crouching_railgun" );
			}
		}
		break;
	case ACT_RANGE_ATTACK2:
		// stormtrooper is going to a secondary long range attack. This may be a thrown 
		// grenade, we must determine which and pick proper sequence
		if ( pev->weapons & STORMT_HANDGRENADE )
		{
			// get toss anim
			iSequence = LookupSequence( "throwgrenade" );
		}
		
		break;
	case ACT_RUN:
		if ( pev->health <= STORMT_LIMP_HEALTH )
		{
			// limp!
			iSequence = LookupActivity ( ACT_RUN_HURT );
		}
		else
		{
			iSequence = LookupActivity ( NewActivity );
		}
		break;
	case ACT_WALK:
		if ( pev->health <= STORMT_LIMP_HEALTH )
		{
			// limp!
			iSequence = LookupActivity ( ACT_WALK_HURT );
		}
		else
		{
			iSequence = LookupActivity ( NewActivity );
		}
		break;
	case ACT_IDLE:
		if ( m_MonsterState == MONSTERSTATE_COMBAT )
		{
			NewActivity = ACT_IDLE_ANGRY;
		}
		iSequence = LookupActivity ( NewActivity );
		break;
	default:
		iSequence = LookupActivity ( NewActivity );
		break;
	}
	
	m_Activity = NewActivity; // Go ahead and set this so it doesn't keep trying when the anim is not present

	// Set to the desired anim, or default anim if the desired is not present
	if ( iSequence > ACTIVITY_NOT_AVAILABLE )
	{
		if ( pev->sequence != iSequence || !m_fSequenceLoops )
		{
			pev->frame = 0;
		}

		pev->sequence		= iSequence;	// Set to the reset anim (if it's there)
		ResetSequenceInfo( );
		SetYawSpeed();
	}
	else
	{
		// Not available try to get default anim
		ALERT ( at_console, "%s has no sequence for act:%d\n", STRING(pev->classname), NewActivity );
		pev->sequence		= 0;	// Set to the reset anim (if it's there)
	}
}

//=========================================================
// Get Schedule!
//=========================================================
Schedule_t *CStormt :: GetSchedule( void )
{

	// clear old sentence
	m_iSentence = STORMT_SENT_NONE;

	// flying? If PRONE, barnacle has me. IF not, it's assumed I am rapelling. 
	if ( pev->movetype == MOVETYPE_FLY && m_MonsterState != MONSTERSTATE_PRONE )
	{
		if (pev->flags & FL_ONGROUND)
		{
			// just landed
			pev->movetype = MOVETYPE_STEP;
			return GetScheduleOfType ( SCHED_STORM_REPEL_LAND );
		}
		else
		{
			// repel down a rope, 
			if ( m_MonsterState == MONSTERSTATE_COMBAT )
				return GetScheduleOfType ( SCHED_STORM_REPEL_ATTACK );
			else
				return GetScheduleOfType ( SCHED_STORM_REPEL );
		}
	}

	// stormtroopers place HIGH priority on running away from danger sounds.
	if ( HasConditions(bits_COND_HEAR_SOUND) )
	{
		CSound *pSound;
		pSound = PBestSound();

		ASSERT( pSound != NULL );
		if ( pSound)
		{
			if (pSound->m_iType & bits_SOUND_DANGER)
			{
				// dangerous sound nearby!
				
				//!!!KELLY - currently, this is the stormtrooper's signal that a grenade has landed nearby,
				// and the stormtrooper should find cover from the blast
				// good place for "SHIT!" or some other colorful verbal indicator of dismay.
				// It's not safe to play a verbal order here "Scatter", etc cause 
				// this may only affect a single individual in a squad. 
				
				if (FOkToSpeak())
				{
					SENTENCEG_PlayRndSz( ENT(pev), "ST_GREN", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
					JustSpoke();
				}
				return GetScheduleOfType( SCHED_TAKE_COVER_FROM_BEST_SOUND );
			}
			/*
			if (!HasConditions( bits_COND_SEE_ENEMY ) && ( pSound->m_iType & (bits_SOUND_PLAYER | bits_SOUND_COMBAT) ))
			{
				MakeIdealYaw( pSound->m_vecOrigin );
			}
			*/
		}
	}
	switch	( m_MonsterState )
	{
	
	case MONSTERSTATE_IDLE:
	case MONSTERSTATE_ALERT:
		{
			
			if ( HasConditions ( bits_COND_HEAVY_DAMAGE ) || HasConditions ( bits_COND_LIGHT_DAMAGE ) )
			{
				//ALERT(at_console, "FUIT\n");
				return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN  );
			}

		}
		break;
	
	case MONSTERSTATE_COMBAT:
		{
			if ( pev->iuser4 == 2 )//on fuit si on brle
				{
					return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ORIGIN );
				}
// dead enemy
			if ( HasConditions( bits_COND_ENEMY_DEAD ) )
			{
				// call base class, all code to handle dead enemies is centralized there.
				return CBaseMonster :: GetSchedule();
			}

// new enemy
			if ( HasConditions(bits_COND_NEW_ENEMY) )
			{
				if ( InSquad() )
				{
					MySquadLeader()->m_fEnemyEluded = FALSE;

					if ( !IsLeader() )
					{
						return GetScheduleOfType ( SCHED_TAKE_COVER_FROM_ENEMY );
					}
					else 
					{
						//!!!KELLY - the leader of a squad of stormtroopers has just seen the player or a 
						// monster and has made it the squad's enemy. You
						// can check pev->flags for FL_CLIENT to determine whether this is the player
						// or a monster. He's going to immediately start
						// firing, though. If you'd like, we can make an alternate "first sight" 
						// schedule where the leader plays a handsign anim
						// that gives us enough time to hear a short sentence or spoken command
						// before he starts pluggin away.
						if (FOkToSpeak())// && RANDOM_LONG(0,1))
						{
							if ((m_hEnemy != NULL) && m_hEnemy->IsPlayer())
								// player
								SENTENCEG_PlayRndSz( ENT(pev), "ST_ALERT", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
							else if ((m_hEnemy != NULL) &&
									(m_hEnemy->Classify() != CLASS_PLAYER_ALLY) && 
									(m_hEnemy->Classify() != CLASS_HUMAN_PASSIVE) && 
									(m_hEnemy->Classify() != CLASS_MACHINE))
								// monster
								SENTENCEG_PlayRndSz( ENT(pev), "ST_MONST", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);

							JustSpoke();
						}
						
						if ( HasConditions ( bits_COND_CAN_RANGE_ATTACK1 ) )
						{
							return GetScheduleOfType ( SCHED_STORM_SUPPRESS );
						}
						else
						{
							return GetScheduleOfType ( SCHED_STORM_ESTABLISH_LINE_OF_FIRE );
						}
					}
				}
			}
// no ammo
			else if ( HasConditions ( bits_COND_NO_AMMO_LOADED ) )
			{
				//!!!KELLY - this individual just realized he's out of bullet ammo. 
				// He's going to try to find cover to run to and reload, but rarely, if 
				// none is available, he'll drop and reload in the open here. 
				return GetScheduleOfType ( SCHED_STORM_COVER_AND_RELOAD );
			}
			
// damaged just a little
			else if ( HasConditions( bits_COND_LIGHT_DAMAGE ) )
			{
				// if hurt:
				// 90% chance of taking cover
				// 10% chance of flinch.
				int iPercent = RANDOM_LONG(0,99);

				if ( iPercent <= 90 && m_hEnemy != NULL )
				{
					// only try to take cover if we actually have an enemy!

					//!!!KELLY - this stormtrooper was hit and is going to run to cover.
					if (FOkToSpeak()) // && RANDOM_LONG(0,1))
					{
						//SENTENCEG_PlayRndSz( ENT(pev), "ST_COVER", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
						m_iSentence = STORMT_SENT_COVER;
						//JustSpoke();
					}
					return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ENEMY );
				}
				else
				{
					return GetScheduleOfType( SCHED_SMALL_FLINCH );
				}
			}
// can kick
			else if ( HasConditions ( bits_COND_CAN_MELEE_ATTACK1 ) )
			{
				return GetScheduleOfType ( SCHED_MELEE_ATTACK1 );
			}

// can shoot
			else if ( HasConditions ( bits_COND_CAN_RANGE_ATTACK1 ) )
			{
				if ( InSquad() )
				{
					// if the enemy has eluded the squad and a squad member has just located the enemy
					// and the enemy does not see the squad member, issue a call to the squad to waste a 
					// little time and give the player a chance to turn.
					if ( MySquadLeader()->m_fEnemyEluded && !HasConditions ( bits_COND_ENEMY_FACING_ME ) )
					{
						MySquadLeader()->m_fEnemyEluded = FALSE;
						return GetScheduleOfType ( SCHED_STORM_FOUND_ENEMY );
					}
				}

				if ( OccupySlot ( bits_SLOTS_STORMT_ENGAGE ) )
				{
					// try to take an available ENGAGE slot
					return GetScheduleOfType( SCHED_RANGE_ATTACK1 );
				}
				else if ( HasConditions ( bits_COND_CAN_RANGE_ATTACK2 ) && OccupySlot( bits_SLOTS_STORMT_GRENADE ) )
				{
					// throw a grenade if can and no engage slots are available
					return GetScheduleOfType( SCHED_RANGE_ATTACK2 );
				}
				else
				{
					// hide!
					return GetScheduleOfType( SCHED_TAKE_COVER_FROM_ENEMY );
				}
			}
// can't see enemy
			else if ( HasConditions( bits_COND_ENEMY_OCCLUDED ) )
			{
				if ( HasConditions( bits_COND_CAN_RANGE_ATTACK2 ) && OccupySlot( bits_SLOTS_STORMT_GRENADE ) )
				{
					//!!!KELLY - this stormtrooper is about to throw or fire a grenade at the player. Great place for "fire in the hole"  "frag out" etc
					if (FOkToSpeak())
					{
						SENTENCEG_PlayRndSz( ENT(pev), "ST_THROW", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
						JustSpoke();
					}
					return GetScheduleOfType( SCHED_RANGE_ATTACK2 );
				}
				else if ( OccupySlot( bits_SLOTS_STORMT_ENGAGE ) )
				{
					//!!!KELLY - stormtrooper cannot see the enemy and has just decided to 
					// charge the enemy's position. 
					if (FOkToSpeak())// && RANDOM_LONG(0,1))
					{
						//SENTENCEG_PlayRndSz( ENT(pev), "ST_CHARGE", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
						m_iSentence = STORMT_SENT_CHARGE;
						//JustSpoke();
					}

					return GetScheduleOfType( SCHED_STORM_ESTABLISH_LINE_OF_FIRE );
				}
				else
				{
					//!!!KELLY - stormtrooper is going to stay put for a couple seconds to see if
					// the enemy wanders back out into the open, or approaches the
					// stormtrooper's covered position. Good place for a taunt, I guess?
					if (FOkToSpeak() && RANDOM_LONG(0,1))
					{
						SENTENCEG_PlayRndSz( ENT(pev), "ST_TAUNT", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
						JustSpoke();
					}
					return GetScheduleOfType( SCHED_STANDOFF );
				}
			}
			
			if ( HasConditions( bits_COND_SEE_ENEMY ) && !HasConditions ( bits_COND_CAN_RANGE_ATTACK1 ) )
			{
				return GetScheduleOfType ( SCHED_STORM_ESTABLISH_LINE_OF_FIRE );
			}
		}
	}
	
	// no special cases here, call the base class
	return CSquadMonster :: GetSchedule();
}

//=========================================================
//=========================================================
Schedule_t* CStormt :: GetScheduleOfType ( int Type ) 
{
	switch	( Type )
	{
	case SCHED_TAKE_COVER_FROM_ENEMY:
		{
			if ( InSquad() )
			{
				if ( g_iSkillLevel == SKILL_HARD && HasConditions( bits_COND_CAN_RANGE_ATTACK2 ) && OccupySlot( bits_SLOTS_STORMT_GRENADE ) )
				{
					if (FOkToSpeak())
					{
						SENTENCEG_PlayRndSz( ENT(pev), "ST_THROW", STORMT_SENTENCE_VOLUME, STORM_ATTN, 0, m_voicePitch);
						JustSpoke();
					}
					return slStormTossGrenadeCover;
				}
				else
				{
					return &slStormTakeCover[ 0 ];
				}
			}
			else
			{
				if ( RANDOM_LONG(0,1) )
				{
					return &slStormTakeCover[ 0 ];
				}
				else
				{
					return &slStormGrenadeCover[ 0 ];
				}
			}
		}
	case SCHED_TAKE_COVER_FROM_BEST_SOUND:
		{
			return &slStormTakeCoverFromBestSound[ 0 ];
		}
	case SCHED_STORM_TAKECOVER_FAILED:
		{
			if ( HasConditions( bits_COND_CAN_RANGE_ATTACK1 ) && OccupySlot( bits_SLOTS_STORMT_ENGAGE ) )
			{
				return GetScheduleOfType( SCHED_RANGE_ATTACK1 );
			}

			return GetScheduleOfType ( SCHED_FAIL );
		}
		break;
	case SCHED_STORM_ELOF_FAIL:
		{
			// stormtrooper is unable to move to a position that allows him to attack the enemy.
			return GetScheduleOfType ( SCHED_TAKE_COVER_FROM_ENEMY );
		}
		break;

	case SCHED_STORM_RECHERCHE:
		return &slStormRecherche[ 0 ];
		break;

	case SCHED_STORM_ESTABLISH_LINE_OF_FIRE:
		{
			return &slStormEstablishLineOfFire[ 0 ];
		}
		break;
	case SCHED_RANGE_ATTACK1:
		{
			// randomly stand or crouch
			if (RANDOM_LONG(0,9) == 0)
				m_fStanding = RANDOM_LONG(0,1);
		 
			if (m_fStanding)
				return &slStormRangeAttack1B[ 0 ];
			else
				return &slStormRangeAttack1A[ 0 ];
		}
	case SCHED_RANGE_ATTACK2:
		{
			return &slStormRangeAttack2[ 0 ];
		}
	case SCHED_COMBAT_FACE:
		{
			return &slStormCombatFace[ 0 ];
		}
	case SCHED_STORM_WAIT_FACE_ENEMY:
		{
			return &slStormWaitInCover[ 0 ];
		}
	case SCHED_STORM_SWEEP:
		{
			return &slStormSweep[ 0 ];
		}
	case SCHED_STORM_COVER_AND_RELOAD:
		{
			return &slStormHideReload[ 0 ];
		}
	case SCHED_STORM_FOUND_ENEMY:
		{
			return &slStormFoundEnemy[ 0 ];
		}
	case SCHED_VICTORY_DANCE:
		{
			if ( InSquad() )
			{
				if ( !IsLeader() )
				{
					return &slStormFail[ 0 ];
				}
			}

			return &slStormVictoryDance[ 0 ];
		}
	case SCHED_STORM_SUPPRESS:
		{
			if ( m_hEnemy->IsPlayer() && m_fFirstEncounter )
			{
				m_fFirstEncounter = FALSE;// after first encounter, leader won't issue handsigns anymore when he has a new enemy
				return &slStormSignalSuppress[ 0 ];
			}
			else
			{
				return &slStormSuppress[ 0 ];
			}
		}
	case SCHED_FAIL:
		{
			if ( m_hEnemy != NULL )
			{
				// stormtrooper has an enemy, so pick a different default fail schedule most likely to help recover.
				return &slStormCombatFail[ 0 ];
			}

			return &slStormFail[ 0 ];
		}
	case SCHED_STORM_REPEL:
		{
			if (pev->velocity.z > -128)
				pev->velocity.z -= 32;
			return &slStormRepel[ 0 ];
		}
	case SCHED_STORM_REPEL_ATTACK:
		{
			if (pev->velocity.z > -128)
				pev->velocity.z -= 32;
			return &slStormRepelAttack[ 0 ];
		}
	case SCHED_STORM_REPEL_LAND:
		{
			return &slStormRepelLand[ 0 ];
		}
	default:
		{
			return CSquadMonster :: GetScheduleOfType ( Type );
		}
	}
}


//=========================================================
// CStormtRepel - when triggered, spawns a monster_stormtrooper
// repelling down a line.
//=========================================================

class CStormtRepel : public CBaseMonster
{
public:
	void Spawn( void );
	void Precache( void );
	void EXPORT RepelUse ( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	int m_iSpriteTexture;	// Don't save, precache
};

LINK_ENTITY_TO_CLASS( monster_stormt_repel, CStormtRepel );

void CStormtRepel::Spawn( void )
{
	Precache( );
	pev->solid = SOLID_NOT;

	SetUse( RepelUse );
}

void CStormtRepel::Precache( void )
{
	UTIL_PrecacheOther( "monster_stormtrooper" );
	m_iSpriteTexture = PRECACHE_MODEL( "sprites/rope.spr" );
}

void CStormtRepel::RepelUse ( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	TraceResult tr;
	UTIL_TraceLine( pev->origin, pev->origin + Vector( 0, 0, -4096.0), dont_ignore_monsters, ENT(pev), &tr);
	/*
	if ( tr.pHit && Instance( tr.pHit )->pev->solid != SOLID_BSP) 
		return NULL;
	*/

	CBaseEntity *pEntity = Create( "monster_stormtrooper", pev->origin, pev->angles );
	CBaseMonster *pStorm = pEntity->MyMonsterPointer( );
	pStorm->pev->movetype = MOVETYPE_FLY;
	pStorm->pev->velocity = Vector( 0, 0, RANDOM_FLOAT( -196, -128 ) );
	pStorm->SetActivity( ACT_GLIDE );
	// UNDONE: position?
	pStorm->m_vecLastPosition = tr.vecEndPos;

	CBeam *pBeam = CBeam::BeamCreate( "sprites/rope.spr", 10 );
	pBeam->PointEntInit( pev->origin + Vector(0,0,112), pStorm->entindex() );
	pBeam->SetFlags( BEAM_FSOLID );
	pBeam->SetColor( 255, 255, 255 );
	pBeam->SetThink( SUB_Remove );
	pBeam->pev->nextthink = gpGlobals->time + -4096.0 * tr.flFraction / pStorm->pev->velocity.z + 0.5;

	UTIL_Remove( this );
}



//=========================================================
// DEAD STORMT PROP
//=========================================================
class CDeadStormt : public CBaseMonster
{
public:
	void Spawn( void );
	int	Classify ( void ) { return	CLASS_HUMAN_MILITARY; }

	void KeyValue( KeyValueData *pkvd );

	int	m_iPose;// which sequence to display	-- temporary, don't need to save
	static char *m_szPoses[3];
};

char *CDeadStormt::m_szPoses[] = { "deadstomach", "deadside", "deadsitting" };

void CDeadStormt::KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "pose"))
	{
		m_iPose = atoi(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else 
		CBaseMonster::KeyValue( pkvd );
}

LINK_ENTITY_TO_CLASS( monster_stormt_dead, CDeadStormt );

//=========================================================
// ********** DeadStormt SPAWN **********
//=========================================================
void CDeadStormt :: Spawn( void )
{
	PRECACHE_MODEL("models/stormt.mdl");
	SET_MODEL(ENT(pev), "models/stormt.mdl");

	pev->effects		= 0;
	pev->yaw_speed		= 8;
	pev->sequence		= 0;
	m_bloodColor		= BLOOD_COLOR_RED;

	pev->sequence = LookupSequence( m_szPoses[m_iPose] );

	if (pev->sequence == -1)
	{
		ALERT ( at_console, "Dead stormt with bad pose\n" );
	}

	// Corpses have less health
	pev->health			= 8;

	// map old bodies onto new bodies
	switch( pev->body )
	{
	case 0: // Stormtrooper with Gun
		pev->body = 0;
		pev->skin = 0;
		SetBodygroup( HEAD_GROUP, HEAD_STORM );
		SetBodygroup( GUN_GROUP, GUN_STRIFLE );
		break;
	case 1: // Commander with Gun
		pev->body = 0;
		pev->skin = 0;
		SetBodygroup( HEAD_GROUP, HEAD_COMMANDER );
		SetBodygroup( GUN_GROUP, GUN_STRIFLE );
		break;
	case 2: // Stormtrooper no Gun
		pev->body = 0;
		pev->skin = 0;
		SetBodygroup( HEAD_GROUP, HEAD_STORM );
		SetBodygroup( GUN_GROUP, GUN_NONE );
		break;
	case 3: // Commander no Gun
		pev->body = 0;
		pev->skin = 0;
		SetBodygroup( HEAD_GROUP, HEAD_COMMANDER );
		SetBodygroup( GUN_GROUP, GUN_NONE );
		break;
	}

	MonsterInitDead();
}
